	SUBROUTINE ALPHA
C  THIS SUBROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CALCULATE THE FOLLOWING:
C  ALPHAX(I,J,NL,K) - CHARGE EXCHANGE ABSORPTION COEFFICIENT IN ZONE I
C                  FOR HYDROGENIC NEUTRAL SPECIE K BORN IN ZONE J
C		   CREATING HYDROGENIC NEUTRAL SPECIES NL IN
C		   ZONE I.
C  ALPHAT(I,J,K) - TOTAL ABSORPTION COEFFICIENT IN ZONE I FOR
C                  HYDROGENIC NEUTRAL SPECIE K BORN IN ZONE J
C		   INTERACTING WITH ALL CHARGED SPECIES IN ZONE I.
C  BETAX(I,NL,K)    - CHARGE EXCHANGE ABSORPTION COEFFICIENT IN ZONE I
C                  FOR HYDROGENIC NEUTRAL SPECIES K BORN AT THE WALL
C                  CREATING HYDROGENIC NEUTRAL SPECIES NL IN
C                  ZONE I.
C  BETAT(I,K)    - TOTAL ABSORPTION COEFFICIENT IN ZONE I FOR
C                  HYDROGENIC NEUTRAL SPECIE K BORN AT THE WALL
C		   INTERACTING WITH ALL CHARGED SPECIES IN ZONE I.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
	DIMENSION DSVEL(CNT$3)
C
	EQUIVALENCE (DSVEL(1),TSTOR1(1,1))
C
	DATA CON1/1.929691E15/
C  CON1= 2*C*C/(KEV/AMU)
C
C  I NOW FILL TEMPORARY STORAGE ARRAY TSTOR1 WITH DENE*<SIGMA*V>EL
C  AT ALL ZONE LOCATIONS.
	DO 10 I=1,NZONES
	  DSVEL(I)=DEN(I,1)*SVEL(TEMP(I,1))
10	CONTINUE
C  I NOW LOOP OVER THE DIFFERENT NEUTRAL SPECIES.
	DO 80 K=1,NNEUT
C  I NOW DEFINE THE MASS, Z, AND WALL VELOCITY FOR THIS NEUTRAL SPECIES.
	  NK=NSPEC(K)
	  TSKA=APLAS(NK)
	  TSKZ=ZPLAS(NK)
C  CHECK TO SEE IF NEUTRAL IS HYDROGENIC.
	  IF(TSKZ.GT.1.01) GO TO 90
	  EWALL=1.5E00*TWALL(K)
	  VWALL=SQRT(CON1*EWALL/TSKA)
C  I NOW LOOP OVER ALL SPECIES WHICH THIS NEUTRAL CAN INTERACT WITH.
	  DO 70 L=2,NPLAS
	    LM1=L-1
	    TSLA=APLAS(L)
	    TSLZ=ZPLAS(L)
C  I NOW LOOP OVER THE BIRTH ZONES OF THE MULTI-GENERATION NEUTRALS.
C  NOTE THAT THIS SAME LOOP WILL BE USED AS DEATH ZONES FOR WALL
C  NEUTRALS.
	    DO 60 J=1,NZONES
C  I NOW FIND THE ENERGY AND VELOCITY OF THE BIRTH NEUTRAL.
	      EBORN=1.5E00*TEMP(J,NK)
	      ECL=1.5E00*TEMP(J,L)
	      VBORN=SQRT(CON1*EBORN/TSKA)
C  I NOW FIND THE CREATED NEUTRAL'S INDEX.
	      NL=ISPEC(LM1)
	      IF(.NOT.ESFLAG) GO TO 30
C  I NOW FIND THE RELATIVE ENERGY AND VELOCITY BETWEEN A WALL NEUTRAL
C  OF SPECIES K AND AN ION OF SPECIE L AT LOCATION J.
	      ERELW=EWALL+TSKA*ECL/TSLA
	      VRELW=SQRT(CON1*ERELW/TSKA)
C  I NOW FIND THE ION DENSITY AT THIS ZONE.
	      DENJ=DEN(J,L)
C  I NOW FIND (SIGMA)LOSS AND (SIGMA)CX FOR THE WALL NEUTRAL
C  INTERACTING AT ZONE J
	      SIGL=SIGLOS(ERELW,TSKA,TSLZ)
	      IF(TSLZ.GT.1.01) GO TO 20
C  WE CAN CHARGE EXCHANGE OFF A HYDROGENIC SPECIE.
C  I NOW CHECK IF THIS IS AN ALLOWED NEUTRAL.
	      IF(NL.EQ.0) GO TO 20
	      SIGX=SIGCX(ERELW,TSKA)
	      BETAX(J,NL,K)=DENJ*SIGX*VRELW/VWALL
20	      IF(L.EQ.2) THEN
	      TSWBII=BII(J,K)/VWALL
	      TSWBCX=BCX(J,K)/VWALL
	      BETAT(J,K)=DSVEL(J)/VWALL+TSWBII+TSWBCX
	      END IF
	      BETAT(J,K)=DENJ*SIGL*VRELW/VWALL+BETAT(J,K)
C  I NOW LOOP OVER ALL INTERACTION ZONES.
30	      IF(.NOT.MCFLAG) GO TO 60
	      DO 50 I=1,NZONES
C  I NOW FIND THE ENERGY OF THE ION IN THIS ZONE AND CALCULATE
C  THE RELATIVE ENERGY AND VELOCITY BETWEEN THIS ION AND THE
C  NEUTRAL FROM ZONE J.
	        EC=1.5E00*TEMP(I,L)
	        EREL=EBORN+TSKA*EC/TSLA
	        VREL=SQRT(CON1*EREL/TSKA)
	        DENI=DEN(I,L)
C  I NOW FIND (SIGMA)LOSS AND (SIGMA)CX FOR THE NEUTRAL
C  BORN IN ZONE J INTERACTING WITH THE IONS IN ZONE I.
	        SIGL=SIGLOS(EREL,TSKA,TSLZ)
	        IF(TSLZ.GT.1.01) GO TO 40
C  WE CAN CHARGE EXCHANGE OF A HYDROGENIC SPECIE.
	        IF(NL.EQ.0) GO TO 40
	        SIGX=SIGCX(EREL,TSKA)
	        ALPHAX(I,J,NL,K)=DENI*SIGX*VREL/VBORN
40	        IF(L.EQ.2) THEN
	        TSMBII=BII(I,K)/VBORN
	        TSMBCX=BCX(I,K)/VBORN
	        ALPHAT(I,J,K)=DSVEL(I)/VBORN+TSMBII+TSMBCX
	        END IF
	        ALPHAT(I,J,K)=DENI*SIGL*VREL/VBORN+ALPHAT(I,J,K)
50	      CONTINUE
60	    CONTINUE
70	  CONTINUE
80	CONTINUE
	RETURN
90	WRITE(LUN6,100) K
100	FORMAT(T2,'NEUTRAL SPECIES ',I2,' IS NOT HYDROGENIC')
	STOP
	END
	SUBROUTINE EDITR
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CONSTRUCT THE OUTPUT
C  RESULTS OF DNEUT, TNEUT, EDEP, RDOT AND SPECTRUM.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
	DATA CON2/2.8945365E15/
C  CON2 = 3*C*C/(KEV/AMU)
C  I NOW LOOP OVER ALL ORGINAL NEUTRAL SPECIES.
	DO 30 K=1,NNEUT
	  NNK=NSPEC(K)
	  TSKA=APLAS(NNK)
C  I NOW FIND THE WALL VELOCITY FOR THIS SPECIES.
	  VWALL=SQRT(CON2*TWALL(K)/TSKA)
C  I NOW LOOP OVER ALL ZONES
	  DO 20 I=1,NZONES
C  I NOW LOOP OVER ALL CHARGED PARTICLES WHICH COULD
C  RESULT IN AN INTERACTION.
	    IF(ESFLAG) DCOLD(I,K)=S0COLD(I,1,K)/
     >(BETAX(I,1,K)*VWALL)
	    NL=1
	    LP1=NSPEC(NL)
	    SUM2=0.E00
	    SUM3=0.E00
C  I NOW LOOP OVER ALL BIRTH ZONES
	    DO 10 J=1,NZONES
	      VNEUT=SQRT(CON2*TEMP(J,NNK)/TSKA)
C I NOW FIND THE SOURCE RATE
	      GF=-GFUN(I,J,NL,K)
	      IF(J.EQ.I.AND.NL.EQ.K) GF=GF+1.E00
C  TS2 CONTAINS THE SOURCE RATE OF NEUTRALS NL IN ZONE I
C  DUE TO NEUTRALS OF TYPE K BORN IN ZONE J CHARGE EXCHANGING
C  ON CHARGED PARTICLE L.
	      TS2=GF*TSTOR1(J,K)
C  TS4 IS THE NUMBER OF MULTI-GENERATION NEUTRALS IN ZONE I DUE
C  TO NEUTRAL K BORN IN ZONE J INTERACTING WITH CHARGED PARTICLE
C  L.
	      TS4=TS2/(ALPHAX(I,J,NL,K)*VNEUT)
	      SUM2=SUM2+TS4
	      SUM3=SUM3+TS4*TEMP(J,LP1)
10	    CONTINUE
	    DNEUT(I,K)=SUM2
	    TNEUT(I,K)=SUM3
20	  CONTINUE
30	CONTINUE
	IF(.NOT.ESFLAG) GO TO 60
	DO 50 K=1,NNEUT
	  DO 40 I=1,NZONES
	  TOTALN=DNEUT(I,K)+DCOLD(I,K)
	  IF(TOTALN.LT.1.E-10) GO TO 40
	    TNEUT(I,K)=(TNEUT(I,K)+DCOLD(I,K)*TWALL(K))/TOTALN
40	  CONTINUE
50	CONTINUE
	GO TO 90
60	DO 80 K=1,NNEUT
	  DO 70 I=1,NZONES
	    TNEUT(I,K)=TNEUT(I,K)/DNEUT(I,K)
70	  CONTINUE
80	CONTINUE
90	RETURN
	END
	SUBROUTINE EDITS
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CALCULATE THE NET SOURCES
C  FOR:
C  ELECTRONS
C  HYDROGENIC ION
C  HYDROGENIC NEUTRALS
C  POWER FLOW DUE TO CHARGE EXCHANGE  (IONS TO NEUTRALS)
C  POWER FLOW DUE TO IONIZATION  (NEUTRALS TO IONS)
C
	INCLUDE 'CNEUT.CMN/LIST'
C
	DATA CON1,CON2,TWOPI/1.6021892E-16,2.8945365E15,6.283185308E00/
C  CON1 = JOULES/KEV
C  CON2 = 3*C*C/(KEV/AMU)
C
	ELOSMG=0.E00
	VIPCX=0.E00
	VIPII=0.E00
	VIPRC=0.E00
	VIVS=0.E00
	VIPVS=0.E00
	FLUXMG=0.E00
	PLOSS1=0.E00
	ELOSW=0.E00
	FLUXT=0.E00
	EFLUX=0.E00
	FLUXW=0.E00
	WAREA=TWOPI*AMINOR
	DO 70 K=1,NNEUT

	  IF(.NOT.ESFLAG) GO TO 10
	  TWALLK=TWALL(K)
	  FLUXT=FLUXT+FLUXIN(K)
	  EFLUX=EFLUX+FLUXIN(K)*TWALLK
	  WWALL(K)=4.E00*WWALL(K)*FLUXIN(K)/WAREA
	  FLUXW=FLUXW+WWALL(K)
	  ELOSW=ELOSW+WWALL(K)*TWALLK
10	  KC=NSPEC(K)
C  KC GIVES THE ION NUMBER OF THE KTH NEUTRAL.
	  TKA=APLAS(KC)
C  I NOW FIND THE WALL VELOCITY FOR THIS NEUTRAL SPECIES.
	  VWALL=SQRT(CON2*TWALLK/TKA)
C  I NOW LOOP OVER ALL DEATH ZONES.
	  DO 60 I=1,NZONES
	  TS4=TSTOR1(I,K)/WAREA
	    FLUXMG=FLUXMG+WALL(I,K)*TS4
	    ELOSMG=ELOSMG+WALL(I,K)*TS4*TEMP(I,KC)
	    USUM1=0.E00
C  USUM1 WILL CONTAIN THE SOURCE OF ELECTRONS RESULTING FROM
C  THE INTERACTION OF THE K NEUTRAL SPECIES.
	    PCXMG=0.E00
C  PCXMG WILL CONTAIN THE CHARGE EXCHANGE POWER FLOW IN ZONE I
C  DUE TO MG NEUTRAL PARTICAL K INTERACTING WITH ALL CHARGED
C  PARTICLES.
	    PIIMG=0.E00
C  PIIMG WILL CONTAIN THE IONIZATION POWER FLOW IN ZONE I DUE
C  TO MG NEUTRAL PARTICLES BEING IONIZED.
	    PCXW=0.E00
C  PCXW WILL CONTAIN THE CHARGE EXCHANGE POWER FLOW IN ZONE I
C  DUE TO WALL NEUTRAL PARTICAL K INTERACTING WITH ALL CHARGED
C  PARTICLES.
	    PIIW=0.E00
C  PIIW WILL CONTAIN THE IONIZATION POWER FLOW IN ZONE I DUE TO
C  WALL NEUTRALS.
	    TSUM3=0.E00
C  TSUM3 WILL CONTAIN THE TOTAL MG REACTION RATE/CM**3 FOR THE
C  K NEUTRAL INTERACTING IN ZONE I.
	    TS1=DCOLD(I,K)*VWALL
	    SUM=0.E00
C  SUM CONTAINS THE WALL SOURCE OF NEUTRALS PRODUCED BY THE K NEUTRAL
C  INTERACTING WITH ALL CHARGED PARTICLES.
	    SUM2=0.E00
C  SUM2 CONTAINS THE POWER OF THE K MG NEUTRAL SPECIES IN ZONE I
C  SUMMED OVER ALL BIRTH ZONES AND OVER ALL CHARGED PARTICLES.
	    SUM3=0.E00
C  SUM3 CONTAINS THE POWER OF THE K WALL NEUTRAL SPECIES IN ZONE I
C  SUMMED OVER ALL CHARGED PARTICLES.
	    DO 40 NL=1,NNEUT
C  NL REPRESENTS THE CHARGED HYDROGENIC SPECIES
	      NLC=NSPEC(NL)
	      TSUM1=0.E00
C  TSUM1 WILL CONTAIN THE SOURCE OF ALL CX MG NEUTRALS IN ZONE I
C  SUMMED OVER ALL BIRTH ZONES FOR THE K NEUTRAL AND THE NLC PLASMA
C  SPECIE.
C  SAME AS ABOVE BUT FOR CROSS SPECIES.
	      TEMPI=TEMP(I,NLC)
C  TEMPI IS THE TEMPERATURE OF THE NLC PLASMA SPECIES IN ZONE I.
C  I NOW LOOP OVER THE BIRTH ZONES OF THE MULTI-GENERATION NEUTRALS.
	      DO 30 J=1,NZONES
	        TEMPJ=TEMP(J,KC)
C  TEMPJ IS THE TEMPERATURE OF THE K NEUTRAL SPECIE BORN IN J.
C  I NOW FIND THE VELOCITY AT WHICH THE NEUTRAL WAS BORN.
	        VNEUT=SQRT(CON2*TEMPJ/TKA)
C  I NOW FIND THE SOURCE OF NEUTRALS
	        SMG=-GFUN(I,J,NL,K)
	        IF(J.EQ.I.AND.NL.EQ.K) SMG=SMG+1.E00
	        SMG=SMG*TSTOR1(J,K)
C  SMG IS NOW THE SOURCE RATE OF MULTI-GENERATION NEUTRALS OF SPECIES
C  NL IN ZONE I DUE TO NEUTRALS OF TYPE K BORN IN ZONE J CHARGE
C  EXCHANGING ON CHARGED PARTICLE L.
	        TSUM1=TSUM1+SMG
	        PCXMG=PCXMG+(TEMPI-TEMPJ)*SMG
	        SUM2=SUM2+TEMPJ*SMG
	        IF(NL.GT.1) GO TO 20
C  INCLUDE TOTAL MG REACTIONS.
	        TS8=ALPHAT(I,J,K)*SMG/ALPHAX(I,J,1,K)
	        TSUM3=TSUM3+TS8
	        USUM1=USUM1+TS8		!SOURCE DUE TO MG IONIZATION
	        PIIMG=PIIMG+TS8*TEMPJ
C  NOTE THAT THE NEUTRAL POWER (SUM2) WILL BE SUBTRACTED LATER.
20	        CONTINUE
30	      CONTINUE
C  NEXT 3 LINES GIVE CONTRIBUTION DUE TO MG NEUTRALS.
	      SORI(I,NL)=SORI(I,NL)-TSUM1	!SINK DUE TO MG CX
	      SORN(I,NL)=SORN(I,NL)+TSUM1	!SOURCE DUE TO MG CX
	      USUM1=USUM1-TSUM1		!CORRECTION FOR MG CX
	      IF(.NOT.ESFLAG) GO TO 40
	      TS2=S0COLD(I,NL,K)
	      SORI(I,NL)=SORI(I,NL)-TS2		!SINK DUE TO WALL CX
	      SORN(I,NL)=SORN(I,NL)+TS2		!SOURCE DUE TO WALL CX
	      PCXW=PCXW+(TEMPI-TWALLK)*TS2
	      SUM3=SUM3+TS2*TWALLK
	      SUM=SUM+TS2
40	    CONTINUE
C  NEXT 3 LINES GIVE CONTRIBUTION DUE TO MG NEUTRALS.
	    PIIMG=PIIMG-SUM2
	    SORI(I,K)=SORI(I,K)+TSUM3		!SOURCE DUE TO MG ION.
	    SORN(I,K)=SORN(I,K)-TSUM3		!SINK DUE TO MG ION.
	    IF(.NOT.ESFLAG) GO TO 50
	    TS5=BETAT(I,K)*TS1
	    SORI(I,K)=SORI(I,K)+TS5		!SOURCE DUE TO WALL ION.
	    SORN(I,K)=SORN(I,K)-TS5		!SINK DUE TO WALL ION.
	    USUM1=USUM1+TS5-SUM		!SOURCE DUE TO WALL IONIZATION
	    PIIW=PIIW+TS5*TWALLK-SUM3
50	    CONTINUE
C  THE ELECTRON SOURCE IS NOW CORRECTED FOR BEAM CX.
	    IF(IBFLAG) USUM1=USUM1-BCX(I,K)*(DCOLD(I,K)+DNEUT(I,K))
	    PCX(I)=PCX(I)+PCXMG+PCXW
	    PII(I)=PII(I)+PIIMG+PIIW
	    SORE(I)=SORE(I)+USUM1
	    IF(.NOT.RCFLAG) GO TO 60
	    TS6=SVRC(I)*DEN(I,KC)
	    SORI(I,K)=SORI(I,K)-TS6
	    SORE(I)=SORE(I)-TS6
	    SORN(I,K)=SORN(I,K)+TS6
	    PRC(I)=PRC(I)+TS6*TEMP(I,KC)
60	  CONTINUE
70	CONTINUE
C  I NOW CONVERT POWER TERMS TO WATTS
	DO 100 I=1,NZONES
	  PCX(I)=PCX(I)*CON1*1.5E00
	  PII(I)=PII(I)*CON1*1.5E00
	  VOLI=VOL(I)
	  VIPII=VIPII+PII(I)*VOLI
	  VIPCX=VIPCX+PCX(I)*VOLI
	  IF(.NOT.RCFLAG) GO TO 80
	  PRC(I)=PRC(I)*CON1*1.5E00
	  VIPRC=VIPRC+PRC(I)*VOLI
80	  DO 90 K=1,NNEUT
	    KC=NSPEC(K)
	    PLOSS1=PLOSS1+SORN(I,K)*VOLI
	    IF(.NOT.VSFLAG) GO TO 90
	    TS3=VOLSOR(I,K)*VOLI
	    VIVS=VIVS+TS3
	    VIPVS=VIPVS+TS3*TEMP(I,KC)
90	  CONTINUE
100	CONTINUE
	PLOSS=WAREA*(FLUXMG+FLUXW-FLUXT)-VIVS
	ELOSS=(WAREA*(ELOSMG+ELOSW-EFLUX)-VIPVS)*1.5E00*CON1
	ELOSS1=VIPCX+VIPRC-VIPII
	RETURN
	END
	SUBROUTINE GMAT
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CALCULATE THE CHARGE
C  EXCHANGE RATE IN EACH ZONE DUE TO NEUTRALS BORN IN ALL ZONES
C  AND FROM THE WALL BY INTEGRATING ALONG EACH TRAJECTORY AND
C  SUMMING OVER ALL TRAJECTORIES.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
C
	DATA PID2/1.570796327E00/
C
C I NOW DEFINE THE H3 FUNCTION.
	H3(X)=H3X(X)
C
	IZMIN = NZCROS(MID)
	DELTR=RBOUND(IZMIN+1)-RBOUND(IZMIN)
C  I NOW TRACK FROM LEFT TO RIGHT AND CALCULATE THE WALL FLUX
C  WHICH CHARGE EXCHANGES IN ZONE I AND THE FRACTION WHICH GETS
C  TRANSMITTED TO THE RIGHT.
	DO 50 K=1,NNEUT
	  X1=0.E00
	  DO 40 I=1,NBCROS
	    IZA=NZCROS(I)
	    ZDISTI=DIST(I)
	    IF(.NOT.ESFLAG) GO TO 10
	    TAUB=BETAT(IZA,K)*ZDISTI
	    X2=X1+TAUB
C	    IF(X2.GT.87.5E00) GO TO 10  !CHECK
	    S0COLD(IZA,1,K)=S0COLD(IZA,1,K)+(H3(X1)-H3(X2))*DELTR
	    X1=X2
10	    CONTINUE
	IF(.NOT.MCFLAG) GO TO 40
C  THE ATTENUATION IN ZONE IZA DUE TO ATOMS BORN IN THE
C  SAME ZONE IS NOW FOUND.
	    TAUA=ZDISTI*ALPHAT(IZA,IZA,K)
	    TS1=PID2-H3(TAUA)
	    GFUN(IZA,IZA,1,K)=GFUN(IZA,IZA,1,K)+(2.E00*TAUA-TS1)*DELTR
	    IF(I.EQ.NBCROS) GO TO 30
C  THE ATTENUATION IN EACH ZONE IZB, TO THE RIGHT OF ZONE
C  IZA, DUE TO ATOMS BORN IN ZONE IZA AND THE FRACTION TRANSMITTED
C  TO THE NEXT ZONE IS NOW CALCULATED.
	    TAUC=0.E00
	    DO 20 J=I+1,NBCROS
	      IZB=NZCROS(J)
	      TAUC=TAUC+DIST(J)*ALPHAT(IZB,IZA,K)
	      TS2=H3(TAUC)-H3(TAUC+TAUA)
	      GFUN(IZB,IZA,1,K)=GFUN(IZB,IZA,1,K)+(TS1-TS2)*DELTR
	      TS1=TS2
20	    CONTINUE
C  ATOMS BORN IN IZA WHICH GET ALL THE WAY TO THE WALL.
30	    WALL(IZA,K)=WALL(IZA,K)+TS1*DELTR
40	  CONTINUE
	IF(ESFLAG) WWALL(K)=WWALL(K)+H3(X2)*DELTR
50	CONTINUE
	RETURN
	END
	FUNCTION H3X(XX)
	DATA PIO2/1.570796327E00/
	EEE1(Y)=PIO2-Y*(.4292037-Y*(.3561945-Y*(.462238-Y*(.404604
     >-Y*(.178267-Y*.030169)))))
	EEE2(Y)=SQRT(Y)*(2.50663-Y*(4.18947-Y*(13.9212-Y*(46.3264
     >-Y*(92.7633-Y*73.3359)))))
	IF(XX.LE.2.E00) THEN
	  H3X=EXP(-XX)*EEE1(XX)
	ELSE IF (XX .LT. 86.) THEN
	  H3X=EXP(-XX)*EEE2(1.E00/XX)
	ELSE
	  H3X = 0.
	END IF
	RETURN
	END
	SUBROUTINE KERNEL
C  THIS SUBROUTINE WAS WRITTEN BY HARRY. H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL FINISH CONSTRUCTING
C  THE S0COLD AND GFUN ARRAYS.  IT WILL ALSO CONSTRUCT THE KERNEL
C  FROM GFUN.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
C  I NOW LOOP OVER ALL NEUTRAL SPECIES.
	DO 40 K=1,NNEUT
	  TS1=4.E00*FLUXIN(K)
C  I NOW LOOP OVER ALL BIRTH ZONES.
	  DO 30 I=1,NZONES
	    IF(.NOT.MCFLAG) GO TO 50
	    TS3=1.E00/ALPHAT(I,I,K)
	    WALL(I,K)=WALL(I,K)*TS3
50	    IF(.NOT.ESFLAG) GO TO 100
	    TS2=BETAT(I,K)*VOL(I)
	    TS4=S0COLD(I,1,K)
C  I NOW LOOP OVER ALL ALLOWED CHARGED PARTICLES.
100	    DO 20 NL=NNEUT,1,-1
	      IF(ESFLAG) S0COLD(I,NL,K)=TS4*TS1*BETAX(I,NL,K)/TS2
	      IF(.NOT.MCFLAG) GO TO 20
C  I NOW LOOP OVER ALL ZONES.
	      DO 10 J=1,NZONES
	        GA=GFUN(J,I,1,K)*TS3
	        TS0=-ALPHAX(J,I,NL,K)/(ALPHAT(J,I,K)*VOL(J))
	        GFUN(J,I,NL,K)=GA*TS0
	        IF(J.EQ.I.AND.NL.EQ.K) GFUN(J,I,NL,K)=GFUN(J,I,NL,K)+
     >          1.E00
10	      CONTINUE
20	    CONTINUE
30	  CONTINUE
40	CONTINUE
	RETURN
	END
	SUBROUTINE NEUTS
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL DETERMINE THE NEUTRAL
C  DENSITY FOR MULTIPLE SPECIES. THE BASIC ALGORITHEM USED
C  IS THAT DEVELOPED BY STEVE TAMOR IN HIS ANTIC CODE.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
	PARAMETER NN1=CNT$3*CNT$1,NN2=NN1+1
C
	DIMENSION DD(NN1,NN2),TSTOR2(NN1)
C
	DATA PI,MSFLAG/3.141592654E00,.FALSE./
C
	IF(.NOT.ESFLAG.AND..NOT.VSFLAG.AND..NOT.RCFLAG) GO TO 280
	IF(IBFLAG) GO TO 30
	DO 20 K=1,NNEUT
	  DO 10 I=1,NZONES
	    BII(I,K)=0.E00
	    BCX(I,K)=0.E00
10	  CONTINUE
20	CONTINUE
30	DO 50 I=1,NZONES
	  PCX(I)=0.E00
	  PII(I)=0.E00
	  PRC(I)=0.E00
	  SORE(I)=0.E00
50	CONTINUE
	DO 40 J=1,NNEUT
	  DO 35 I=1,NZONES
	    SORI(I,J)=0.E00
	    SORN(I,J)=0.E00
	    DCOLD(I,J)=0.E00
35	  CONTINUE
40	  CONTINUE
	IF(MSFLAG) GO TO 70
	IF(INFLAG.AND.MCFLAG) GO TO 70
	WRITE(LUN6,60)
60	FORMAT(T2,'NO MATRIX CALCULATED.  DOING INITIALIZATION',
     >' WITH MATRIX CALCULATION')
	INFLAG=.TRUE.
	MCFLAG=.TRUE.
70	IF(.NOT.INFLAG) GO TO 120
C  I NOW FILL ISPEC ARRAY
	DO 100 I=1,NPLAS-1
	  IP1=I+1
	  DO 80 J=1,NNEUT
	    INEUT=J
	    IF(NSPEC(J).EQ.IP1) GO TO 90
80	  CONTINUE
	  ISPEC(J)=0
	  GO TO 100
90	  ISPEC(I)=INEUT
100	CONTINUE
C  I NOW FIND THE BOUNDARY VALUES FOR EACH ZONE. NOTE THAT ZONE
C  CENTER I LIES BETWEEN ZONE BOUNDARY I AND I+1.
	NBOUND=NZONES+1
	RBOUND(1)=0.E00
	TS1=0.E00
	DO 110 I=1,NZONES
	  TS2=2.E00*ZONEC(I)-RBOUND(I)
	  RBOUND(I+1)=TS2
	  VOL(I)=PI*(TS2*TS2-TS1*TS1)
	  TS1=TS2
110	CONTINUE
C  I NOW INITALIZE ALPHA AND BETA ARRAYS
120	IF(MCFLAG.OR.ESFLAG) CALL ALPHA
	IF(.NOT.INFLAG) GO TO 150
C  I NOW FIND ARRAY Y WHICH CONTAINS THE DISTANCE OF CLOSEST
C  APPOACH. SEE PROGRAM WRITE UP FOR DETAILS.
	Y(1)=RBOUND(2)*.6189908923E00
	DO 140 I=2,NZONES
	  RPI=RBOUND(I+1)
	  RPI2=RPI*RPI
	  R=RBOUND(I)
	  VLT=.25E00*PI*RPI2
	  VLM=0.E00
	  DO 130 J=1,I-1
	    VLM=VLM+(RBOUND(J+1)-RBOUND(J))*SQRT(RPI2-Y(J)**2)
130	  CONTINUE
	  TS=(VLT-VLM)/(RPI-R)
	  Y(I)=SQRT(RPI2-TS*TS)
140	CONTINUE
C  I NOW INITIALIZE SOME ARRAYS.
150	IF(.NOT.ESFLAG.AND..NOT.MCFLAG) GO TO 180
	DO 160 K=1,NNEUT
	  WWALL(K)=0.E00
	  DO 160 I=1,NZONES
	    IF(MCFLAG) WALL(I,K)=0.E00
	    IF(ESFLAG) S0COLD(I,1,K)=0.E00
	    DO 160 J=1,NZONES
	      IF(MCFLAG) GFUN(J,I,1,K)=0.E00
160	CONTINUE
C  I NOW LOOP OVER ALL TRAJECTORIES
	DO 170 I=1,NZONES
	  CALL TRACKR(Y(I))
	  CALL GMAT
170	CONTINUE
	CALL KERNEL
	MSFLAG=.TRUE.
C  I NOW MAKE A COPY OF 4-D ARRAY GFUN + S0COLD SO THAT
C  THE SOLUTION VECTOR CAN BE FOUND.
C  I NOW LOOP OVER THE NEUTRAL WHICH WILL DIE IN ZONE I.
180	DO 190 K=1,NNEUT
	  JSTART=NZONES*(K-1)
C  I NOW LOOP OVER THE NEUTRAL CREATED IN ZONE I.
	  DO 190 NL=1,NNEUT
	    ISTART=NZONES*(NL-1)
C  I NOW LOOP OVER ALL BIRTH ZONES OF THE K NEUTRAL.
	    DO 190 J=1,NZONES
	      J1=JSTART+J
C  I NOW LOOP OVER ALL DEATH ZONES OF THE K NEUTRAL.
	      DO 190 I=1,NZONES
	        I1=ISTART+I
	        DD(I1,J1)=GFUN(I,J,NL,K)
190	CONTINUE
C  I NOW ATTACH THE SOLUTION VECTOR.
	IROW=NZONES*NNEUT
	JCOL=IROW+1
	DO 200 I=1,IROW
200	DD(I,JCOL)=0.E00
	IF(.NOT.ESFLAG) GO TO 220
	DO 210 K=1,NNEUT
	  DO 210 NL=1,NNEUT
	    ISTART=NZONES*(NL-1)
	    DO 210 I=1,NZONES
	      I1=ISTART+I
	      DD(I1,JCOL)=S0COLD(I,NL,K)+DD(I1,JCOL)
210	CONTINUE
220	IF(.NOT.VSFLAG) GO TO 240
	DO 230 NL=1,NNEUT
	  ISTART=NZONES*(NL-1)
	  DO 230 I=1,NZONES
	    I1=ISTART+I
	    DD(I1,JCOL)=VOLSOR(I,NL)+DD(I1,JCOL)
230	CONTINUE
240	IF(.NOT.RCFLAG) GO TO 270
C  I NOW FILL UP ARRAY SVRC
	DO 250 I=1,NZONES
	  SVRC(I)=DEN(I,1)*SIGVRC(TEMP(I,1))
250	CONTINUE
	DO 260 NL=1,NNEUT
	  L=NSPEC(NL)
	  ISTART=NZONES*(NL-1)
	  DO 260 I=1,NZONES
	    I1=ISTART+I
	    DD(I1,JCOL)=SVRC(I)*DEN(I,L)+DD(I1,JCOL)
260	CONTINUE
C  I NOW CALL GAUSSE TO SOLVE THE EQUATIONS
270	CALL GAUSSE(DD,NN1,IROW,JCOL,TSTOR2,1.E-4,6)
C
C  FILL TSTOR1 ARRAY.
C
	DO 272 K = 1,NNEUT
	  IOFF = (K - 1)*NZONES
	  DO 271 I = 1,NZONES
	    TSTOR1(I,K)=TSTOR2(IOFF+I)
271	CONTINUE
272	CONTINUE
	CALL EDITR
	CALL EDITS
	RETURN
280	WRITE(LUN6,290)
290	FORMAT(T2,'NO SOURCES MEANS NO NEUTRALS MEANS NO WORK.  NEUTS')
	STOP
	END
	SUBROUTINE PRES(LUN)
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB. THIS ROUTINE WILL PRINT THE RESULTS FROM
C  SUBROUTINE NEUTS.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
C  PARAMETER:
C  LUN    - THE LOGICAL UNIT USED TO PRINT THE RESULTS.
C
	WRITE(LUN,10)
10	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T21'TEMPE',T32'TEMPI',
     >T44'DENE')
	DO 30 I=1,NZONES
	  WRITE(LUN,20) I,ZONEC(I),TEMP(I,1),TEMP(I,2),DEN(I,1)
20	  FORMAT(2X,I2,1X,4(2X,1PE9.2))
30	CONTINUE
	IC1=1
40	IC2=IC1+1
	IF(IC2.GT.NNEUT) GO TO 80
	WRITE(LUN,50) IC1,IC1,IC1,IC2,IC2,IC2
50	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T20'DCOLD(',I1,')',
     >T31'DNEUT(',I1,')',T42'TNEUT(',I1,')',:,T53'DCOLD(',I1,')',
     >T64'DNEUT(',I1,')',T75'TNEUT(',I1,')')
	DO 70 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),((DCOLD(I,K),DNEUT(I,K),TNEUT(I,K)),
     >K=1,2)
60	  FORMAT(2X,I2,1X,7(2X,1PE9.2))
70	CONTINUE
	IC1=IC2+1
	IF(IC1.GT.NNEUT) GO TO 100
	GO TO 40
80	WRITE(LUN,50) IC1,IC1,IC1
	DO 90 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),DCOLD(I,IC1),DNEUT(I,IC1),TNEUT(I,IC1)
90	CONTINUE
100	K=1
	KC=NSPEC(K)
	K2=K+1
	IF(K2.GT.NNEUT) GO TO 120
	KC2=NSPEC(K2)
	WRITE(LUN,110) KC,K,KC2,K2
110	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T22'SORE',
     >T32'SORI(',I1,')',T43'SORN(',I1,')',:,T53'SORI(',I1,')',
     >T65'SORN(',I1,')')
	GO TO 130
120	WRITE(LUN,110) KC,K
	K2=K
130	DO 140 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),SORE(I),((SORI(I,J),SORN(I,J)),J=K,K2)
140	CONTINUE
150	K=K2+1
	IF(K.GT.NNEUT) GO TO 200
	KC=NSPEC(K)
	K2=K+1
	IF(K2.GT.NNEUT) GO TO 170
	KC2=NSPEC(K2)
	WRITE(LUN,160) KC,K,KC2,K2
160	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T21'SORI(',I1,')',
     >T31'SORN(',I1,')',:,T43'SORI(',I1,')',T53'SORN(',I1,')')
	GO TO 180
170	WRITE(LUN,160) KC,K
	K2=K
180	DO 190 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),((SORI(I,J),SORN(I,J)),J=K,K2)
190	CONTINUE
	GO TO 150
200	WRITE(LUN,210)
210	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T23'PCX',T34'PII')
	DO 220 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),PCX(I),PII(I)
220	CONTINUE
	WRITE(LUN,230)
230	FORMAT(3(/),T2'CONSERVATION CHECKS')
	WRITE(LUN,240)
240	FORMAT(2(/),T2'NEUTRALS ABSORBED IN PLASMA (#/SEC)')
	WRITE(LUN,250) -PLOSS
250	FORMAT(T5'EDGE+VOLUME SOURCE BALANCE: ',1PE11.4)
	WRITE(LUN,260) -PLOSS1
260	FORMAT(T13'VOLUME INTEGRATION: ',1PE11.4)
	WRITE(LUN,270)
270	FORMAT(2(/),T2'POWER OUT FLOW FROM PLASMA (WATTS)')
	WRITE(LUN,250) ELOSS
	WRITE(LUN,260) ELOSS1
	RETURN
	END
	SUBROUTINE TRACKR(ELEV)
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL TRACK A GIVEN
C  TRAJECTORY THROUGH THE MESH FOR A GIVEN Y.  IT WILL CALCULATE
C  THE NUMBER OF BOUNDARIES CROSSED (NBCROS), THE NUMBER OF
C  THE ZONE CROSSED FROM LEFT TO RIGHT (NZCROS(I)) AND THE
C  PATH LENGTH IN EACH ZONE (DIST(I)).
C
C  PARAMETER:
C  ELEV  - THE DISTANCE OF CLOSEST APPROACHED.  (CM)
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
	ELEV2=ELEV*ELEV
C  I NOW LOOP OVER ALL BOUNDARIES TO FIND THE BOUNDARY WHICH
C  IS GREATER THAN Y.
	DO 10 I=2,NBOUND
	  IC=I
	  RB=RBOUND(I)
	  IF(RB.GE.ELEV) GO TO 20
10	CONTINUE
C  MZONE CONTAINS THE ZONE NUMBER FOR THE CENTER REGION.
20	MZONE=IC-1
C  NBCROS CONTAINS THE NUMBER OF BOUNDARIES CROSSED = NUMBER OF ZONES
C  TRAJECTORY PASSED THROUGH.
	NBCROS=2*(NZONES-MZONE)+1
C  I NOW FIND THE ARRAY ELEMENT FOR THE MID ZONE.
	MID=NBCROS/2+1
C  I NOW FILL NZCROS AND DIST FOR THE CENTER REGION.
	NZCROS(MID)=MZONE
	DISTM=SQRT(RB*RB-ELEV2)
	DIST(MID)=2.E00*DISTM
	IF(MID.EQ.1) GO TO 40
C  DUE TO SYMMETRY I NOW ONLY HAVE TO LOOP OVER MID-1 POINTS
C  TO FILL UP DIST AND NZCROS ARRAYS
	DO 30 I=1,MID-1
	  IP=MID+I
	  IM=MID-I
	  MZP1=MZONE+I
	  DISTP=SQRT(RBOUND(IC+I)**2-ELEV2)
	  XDIST=DISTP-DISTM
	  DISTM=DISTP
	  NZCROS(IP)=MZP1
	  NZCROS(IM)=MZP1
	  DIST(IP)=XDIST
	  DIST(IM)=XDIST
30	CONTINUE
40	RETURN
	END
