	FUNCTION BMFP(WNEUT,TEMP,zAPLAS,ANEUT,DEN,Z,ZNEUT,iNPLAS)
C
C  CALCULATE THE MEAN FREE PATH OF A NEUTRAL PARTICLE.
c
c originally written by Harry H. Towner 3/28/79
c updated by Greg Hammett 9/23/88 to use new cross-section routines
c
C  PARAMETERS:
C  WNEUT  - THE ENERGY OF THE NEUTRAL PARTICLE.  (KEV)
C  TEMP   - ARRAY GIVING THE TEMPERATURE OF THE BACKGROUND SPECIES.  (KEV)
C  zAPLAS - ARRAY GIVING THE A.M.U. OF THE BACKGROUND SPECIES.
C  ANEUT  - THE A.M.U. OF THE NEUTRAL PARTICLE.
C  DEN	  - ARRAY GIVING THE DENSITY OF THE BACKGROUND SPECIES.  (1/CM**3)
C  Z	  - ARRAY GIVING THE Z OF THE BACKGROUND SPECIES.
C  ZNEUT  - THE Z OF THE NEUTRAL PARTICLE.
c	    If using TRANSP atomic cross-sections, this work properly
c	    only for zneut=1.
c	    Will also work for He3 neutrals in a primarily Helium plasma
c	    if FPP's standalone atomic cross-sections are used and
c	    azmain is set to 2 in the he4com common block.
C  iNPLAS  - THE # OF BACKGROUND SPECIES.
C
C  THE MEAN FREE PATH IS RETURNED BY BMFP AND HAS UNITS OF CM.
C
C  NOTE THAT THE FIRST ELEMENT IN THE ABOVE BACKGROUND ARRAYS
C  IS ASSUMED TO BE THE ELECTRON.
C
	DIMENSION TEMP(iNPLAS),zAPLAS(iNPLAS),DEN(iNPLAS),Z(iNPLAS)
C
	DATA CON1/1.929691111E+15/
C  CON1 = 2*C*C/AMUKEV
C
C  I NOW FIND <SIGMA*V> FOR ELECTRON IMPACT IONIZATION.
	zTev=temp(1)*1000.
	sig3=psgvei(zTev)
C  I NOW FIND VNEUT.
	VNEUT=SQRT(WNEUT*CON1/ANEUT)
 
C  I NOW FIND THE CONTRIBUTION FROM CHARGE EXCHANGE AND ION IMPACT
C  IONIZATION.
 
	SUM=0.E00
	zenev=wneut*1000./aneut
 
	DO 100 I=2,iNPLAS
 
	if(Z(i) .eq. zneut)then
c add charge exchange and impact ionization:
	sum=sum+den(i)*(sigvbt(zenev,temp(i)*1000./zAplas(i),'CX')
	1	+sigvbt(zenev,temp(i)*1000./zAplas(i),'II'))
 
	else
c add electron loss to impurities:
c (need to look for proper helium CX with impurity cross-sections,
c until then assume its the same as hydrogen):
 
	sum=sum+den(i)*psgvls(zenev,z(i))
	endif
 
100	CONTINUE
 
C  THE MEAN FREE PATH IS NOW FOUND.
	BMFP=VNEUT/(SUM+DEN(1)*SIG3)
	RETURN
	END
