      subroutine chiorg(istart,thconv,iconv)
c
c     CHIORG refines the initial chi ray directions such that the
c     rays intersect the vessel at roughly equidistant points.  The
c     variable, Origin=Origin(chi), gives the angle of the initial
c     ray direction relative to the rmaj axis.
c
c The variables output from the subroutine are origin, istart, iconv,
c thconv, and (on the first iteration only) Rmaj and Zed at the outer
c boundary.
c
c iconv=.false. on output if it hasn't converged yet.
c iconv=.true. on output if it has converged.
c
      implicit none
      include 'SPRINC'
      real rmaj(nchi,npsi),zed(nchi,npsi),hpsi(nchi,npsi),
     %     hchi(nchi,npsi),psi(npsi),chi(nchi),pquad(npsi),pcoef(nchi),
     %     origin(nchi),
     %     radius(nvvgrid),drdth(nvvgrid),theta(nvvgrid)
      real findR,findY
      real rmajor,rminor,draytol
      real vvperim(nvvgrid)
      integer ichirz
      common /metric/ rmaj,zed,hpsi,hchi,psi,chi,pquad,pcoef
     %       /vessel/ rmajor,rminor,radius,drdth,theta,findR,findY,
     %		      draytol,vvperim
     %       /ray/ ichirz,origin
 
      real fpintrp    ! external function
 
c declare local variables
      real orgwrk(nchi+1),perim(nchi+1),pi
      real rknots(nchi+5),cscoef(nchi+5),work(6*(nchi+1)+16)
      integer nchim1,nchip1,ifail,nwork,istart,i
      real thconv,dchi,zr,zth,dperim,xperim
      logical iconv
      real thtol
      data thtol /.1/ ! tolerance for theta grid convergence.
 
      nchim1=nchi-1
      nchip1=nchi+1
      pi=abs(acos(-1.))
c
c     Initial guess before first iteration
c
      if (istart.eq.0) then
      dchi=2*pi/nchi
      do 10 i=1,nchi
      origin(i)=(i-1)*dchi
      zr=fpintrp(origin(i),radius,theta,nvvgrid)
      rmaj(i,npsi)=rmajor+zr*cos(origin(i))
      zed(i,npsi) =       zr*sin(origin(i))
10    continue
      endif
c
c     Refinement algorithm
 
      orgwrk(1)=0.
      perim(1)=0.
      do 20 i=2,nchi
      orgwrk(i)=origin(i)
      zth=atan2(zed(i,npsi),rmaj(i,npsi)-rmajor)
      if(zth .lt. 0.0) zth=zth+2*pi
      perim(i)=fpintrp(zth,vvperim,theta,nvvgrid)	
20    continue
      orgwrk(nchip1)=2.*pi
      perim(nchip1)=vvperim(nvvgrid)
      dperim=vvperim(nvvgrid)/float(nchi)
 
c thconv is a measure of the relative convergence of the
c theta grid towards unifrom spacing at the outer edge.
 
      thconv=0.0
      do 40 i=2,nchi/2+1
      thconv=max(thconv,abs(perim(i)-perim(i-1)-dperim))
40    continue
      thconv=thconv/dperim
 
c if it has converged, don't bother resetting origin(i):
      if (istart .ne. 0 .and. thconv .lt. thtol) then
      iconv=.true.
      return
      endif
      iconv=.false.
      istart=1
 
c interpolate to find what initial angles (origin(i)) should
c give uniform spacing of the grid at the outer edge.
c use a cubic rather than a linear interpolation to try to
c speed up the convergence:
 
      ifail=0
      nwork=6*(nchi+1)+16
      call e01bae(nchip1,perim,orgwrk,rknots,cscoef,nchi+5,
     %	      work,nwork,ifail)
 
      do 30 i=2,nchi
      xperim=float(i-1)*dperim
      call e02bbe(nchi+5,rknots,cscoef,xperim,origin(i),ifail)
30    continue
 
      return
      end
