	subroutine comprs
!
! performs major radius compression on f
!
! derived from conservation of mu and toroidal angular momentum:
!
!	v xsi ( R0 + r ) = constant
!
! and conservation of particles in phase space.  The total # of
! particles in a flux surface is given by:
!
! N = 2 pi R0 2 pi r dr 2 pi sqrt(2) integral over dE
!	integral over dxsi sqrt(E) abs(xsi) (1+r/R0) btime(xsi) f
!
! quick and dirty version:
!	only works for unifrom E and xsi grids, nonconservative
!		B.C.'s at E=0
!
!	could be improved to interpolate f and time instead of just
!		using closest value
!
!	could be improved to conserve particles exactly
!
!	only works well if E-max > VBeamInj * Cratio**2
!
	include 'FPPCOM'
 
	cratii=1./cratio	!inverse compression ratio
	crout=cratio*(1.+rOverR/cratio**.5)/(1+ rOverR)
	crouti=1./crout
 
	do 200 i=1,maxen
	do 200 j=1,maxxsi
	Erati=xsi(j)**2 *crouti**2 + (1-xsi(J)**2)*crouti
	Eold=enerev(i)*erati
	xsiold=sqrt(crouti)*xsi(j)/(1.+(crouti-1.)*xsi(j)**2)**.5
	dxrati=sqrt(crouti)/(1.+(Crouti-1.)*xsi(j)**2)**1.5
 
!find the nearest grid points iold, jold.  This only works for a
! uniform grid:
	iold=.5+maxen*Eold/enerev(maxen)
	jold=.5+1+(xsiold+1.)*(maxxsi-1)/2.0
 
! an improved version would do an interpolation between the two nearest
! grid points, now we will just tak the closest values:
	f2(i,j)=f1(iold,jold)*btimeo(jold)/btime(j)*cratio**2
	1	*(1.+rOverR/cratio**.5)/(1.+rOverR)
	1	*erati**1.5 *abs(xsiold)/abs(xsi(j))*dxrati
200	continue
 
	do 300 i=1,maxen
	do 300 j=1,maxxsi
300	f1(i,j)=f2(i,j)
	return
	end
