C
C	THIS SUBROUTINE SETS UP AN ARRAY FISHL(I,J)
C	WHICH GIVES THE FRACTION OF THE PARTICLES OF ENERGY I,
c	AND PITCH ANGLE J WHICH ARE TO SURVIVE AT THIS RADIUS
c	AT THIS TIME STEP
C
C	ROB GOLDSTON -- MAY 3, 1982
C
c	modified for arbitrary grids by greg hammett jan 16, 1984.
c	my changes have not been extensively checked, and may contain
c	bugs.
C
	SUBROUTINE FISH
C
C
	include 'FPPCOM'
C
	DIMENSION IFSHE(0:maxen)
C
C	***** GENERAL DESCRIPTION OF FISH-BONE MODELS ******
C
C	MODEL 0		NO FISH-BONE EFFECTS
C
C	MODEL 1		LOSE 1-FSHFRC OF ALL PASSING IONS
C
C	MODEL 2		LOSE 1-FSHFRC OF ALL IONS WITHIN WFISH2/2 OF
C			THE TRAPPING BOUNDARIES
C
C	MODEL 3		LOSE 1- FSHFRC OF ALL IONS WITHIN WFISH3/2 OF
C			VPAR/V = 0
C
C	MODEL 4		LOSE 1-FSHFRC OF ALL BANANA-TRAPPED IONS
C
c
C	MODEL 5		LOSE 1-FSHFRC OF ALL IONS
C
C	MODEL 6 	COMBINATION OF MODELS 2 AND 3
C
c	Model 7	    Lose 1-fshfrc of all ions with Vpar/V > Wfish2
c
c	Model 8	    Lose 1-fshfrc of all ions trapped outside of Rmaj.
c
C	****** LOSSES ARE LIMITED TO : **********
C	
C	RFMIN <= R <= RFMAX
C
C	EFMIN <= E <= EFMAX
C
C
C
C	DATA STATEMENTS WHICH CONTROL THE NATURE OF THE
C	FISH-BONE
C	THESE HAVE BEEN MOVED TO INPUT DATA READ BY INPUT.FOR
C		ROB GOLDSTON 10-OCT-1982
C
C	DATA LFishMdl/0/		!MODEL FOR FISH-BONE LOSS --
C				!0...6. SEE ABOVE AND TEXT OF CODE
C				!FOR DETAILS
C	DATA TFISH/0.0025/	!TIME BETWEEN FISH-BONES (SEC'S)
C
C	DATA WFISH2/0.05/	!FULL WIDTH OF LOSS REGIONS IN VPAR/V
C				!FOR MODEL 2
C	DATA WFISH3/0.3/	!FULL WIDTH OF LOSS REGION IN VPAR/V
C				!FOR MODEL 3
C	DATA FSHFRC/0.0/	!FRACTION OF PTCLS IN LOSS REGION
C				!THAT SURVIVE AN INDIVIDUAL FISH-BONE
C	DATA EFMIN/35000./	!MINIMUM ENERGY AFFECTED BY FISH-BONES
C	DATA EFMAX/100000./	!MAXIMUM ENERGY AFFECTED BY FISH-BONES
C
C	DATA RFMIN/00./		!MINIMUM RADIUS AFFECTED BY FISH-BONES
C	DATA RFMAX/100./	!MAXIMUM RADIUS AFFECTED BY FISH-BONES
C
 
C	START BY SETTING UP FOR COMPLETE CONFINEMENT
C
	DO 100 I=0,maxen
	DO 100 J=1,maxxsi
	FISHL(I,J)=1.0
100	CONTINUE
 
C
C	DECIDE IF THERE IS A FISH-BONE OSCILLATION THIS TIME-STEP
C
!take care about round-off errors in this and next time step:
	tnext=time
	do 110 i=1,4
110	tnext=tnext+dtFPP/2.0
	i=tnext/tfish
	t=i*tfish
	if(t .le. time) goto 1000
C
C	DETERMINE IF THE FISH-BONE IS OPERATIVE AT THIS RADIUS
C
	ZRAD=(NSHE-0.5)*RMIN/FLOAT(NSHELL)
	IF(ZRAD.GT.RFMAX.OR.ZRAD.LT.RFMIN) GOTO 1000
C
C	SET UP ENERGY ARRAY FOR FISH-BONE EFFICACY
C	IF IFSHE(J) = 0 FISH-BONE DOESN'T HIT ENERGY J
C	IF IFSHE(J) = 1 FISH-BONE HITS ENERGY J
C
	DO 50 I=0,maxen
	ENGY=enerev(i)
	IFSHE(I)=1
	IF(ENGY.LE.EFMAX.AND.ENGY.GE.EFMIN) GOTO 50
	IFSHE(I)=0
50	CONTINUE
C
	fsh4=fshfrc
C
C
C
C	THERE IS A FISH-BONE
C
C
C	***  MODEL 1 ***
C
C	IN THIS MODEL WE ASSUME THAT 1-FSHFRC OF ALL IONS ON PASSING
C	ORBITS ARE LOST
C
	IF(LFishMdl.NE.1) GOTO 200
C
c--original code:
c	DO 150 JSYM=njTrap+1,25
c	JLESS=27-JSYM
c	JMORE=26+JSYM
c	DO 150 I=2,51
c--replaced by:
	do 150 jsym=njTrap+1,maxxsi/2
	jless=maxxsi/2+1-jsym
	jmore=maxxsi/2  +jsym
	do 150 i=0,maxen
	IF(IFSHE(I).EQ.0) GOTO 150
	FISHL(I,JLESS)=FSH4
	FISHL(I,JMORE)=FSH4
150	CONTINUE
	GOTO 1000
C
C
C	*** MODEL 2 ***
C
C	*** MODEL 6 ***  (HAS BOTH "MODEL 2" AND "MODEL 3" LOSSES)
C
C	IN THIS MODEL WE ASSUME THAT 1-FSHFRC OF ALL IONS WITH VPAR/V
C	WITHIN WFISH2/2 OF THE TRAPPING BOUNDARIES ARE LOST
C
200	IF(LFishMdl.NE.2.AND.LFishMdl.NE.6) GOTO 300
C
c--original code:
c	JRANG=INT(WFISH2*12.5)
c	DO 250 I=1,52
c	IF(IFSHE(I).EQ.0) GOTO 250
c	DO 250 JSYM=-JRANG,JRANG
c	FISHL(I,26+njTrap+JSYM)=FSH4
c	FISHL(I,27-njTrap+JSYM)=FSH4
c250	CONTINUE
 
c--replace by:
 
c--find the indics j which correspond to xsi(njTrap)+-wfish2/2
	do 210 j=maxxsi/2+1,maxxsi
	if((xsi(njTrap)-wfish2/2) .gt. xsi(j))jless=j+1
210	if((xsi(njTrap)+wfish2/2) .gt. xsi(j))jmore=j
 
	do 251 i=0,maxen
	if(ifshe(i) .eq. 0)goto 251
	do 250 j=jless,jmore
	fishl(i,j)=fsh4
	fishl(i,maxxsi+1-j)=fsh4
250	continue
251	continue
	IF(LFishMdl.NE.6) GOTO 1000
C
C	*** MODEL 3 ***
C
C	*** MODEL 6 ***  (HAS BOTH "MODEL 2" AND "MODEL 3" LOSSES)
C
C	IN THIS MODEL WE ASSUME THAT 1-FSHFRC OF ALL IONS WITH
C	ABS(VPAR/V) LESS THAN WFISH3/2 ARE LOST
C
300	IF(LFishMdl.NE.3.AND.LFishMdl.NE.6) GOTO 400
C
c--the original coding for the rest of this subroutine can
c	be found in [nbeam.fpp].
c--find the index j corresponding to wfish3/2
	do 310 j=maxxsi/2+1,maxxsi
310	if(xsi(j) .lt. wfish3/2)jrang=j
	do 351 i=0,maxen
	if(ifshe(i) .eq. 0)goto 351
	do 350 j=maxxsi/2+1,jrang
	fishl(i,j)=fsh4
	fishl(i,maxxsi+1-j)=fsh4
350	CONTINUE
351	continue
	GOTO 1000
C
C
C	*** MODEL 4 ***
C
C	IN THIS MODEL WE ASSUME THAT 1-FSHFRC OF ALL BANANA-TRAPPED
C	IONS ARE LOST
C
400	IF(LFishMdl.NE.4) GOTO 500
C
	DO 450 JSYM=1,njTrap
	JLESS=maxxsi/2+1-JSYM
	JMORE=maxxsi/2+JSYM
	DO 450 I=0,maxen
	IF(IFSHE(I).EQ.0) GOTO 450
	FISHL(I,JLESS)=FSH4
	FISHL(I,JMORE)=FSH4
450	CONTINUE
	GOTO 1000
C
C
C	***  MODEL 5  ***
C
C	IN THIS MODEL WE ASSUME THAT 1-FSHFRC OF ALL IONS ARE LOST
C
500	IF(LFishMdl.eq.5) then
	DO 551 I=0,maxen
	IF(IFSHE(I).EQ.0) GOTO 551
	DO 550 J=1,maxxsi
	FISHL(I,J)=FSH4
550	CONTINUE
551	continue
	endif
 
C
c	*** Model 7  ***
c
	if(LFishMdl .eq. 7) then
	    do 650 j=1,maxxsi
	    if(xsi(j) .gt. wfish2)then
		do 640 i=0,maxen
640		if(ifshe(i) .eq. 1) fishl(i,j) = fshfrc
	    endif
650	    continue
	endif
 
c
c	*** Model 8  ***
c
	if(LFishMdl .eq. 8) then
 
	    !find index of particles which are barely trapped inside of Rmaj:
	    zdeep=sqrt(1.-1./(1.+rOverR))
	    do 810 j=maxxsi/2+1,maxxsi
		jover=j
810		if(xsi(j) .gt. zdeep)goto 820
820	    if(jover .gt. maxxsi/2)then
		do 830 j= maxxsi+1-(jover-1),jover-1
		do 830 i=0,maxen
830		if(ifshe(i) .eq. 1) fishl(i,j) = fshfrc
	    endif
 
	endif
 
1000	RETURN
	END
