 
	subroutine fpcf
 
 
c Calculates the Fokker-Planck coefficients for drag, energy diffusion,
c and pitch angle scattering for an arbitrary grid in energy space.
 
 
C	Last Updated:
C  09/15/93  TBT  Added Dragi=0 and Diffei=0
 
 
c     COMMON BLOCKS
c
	include 'FPPCOM'
C
C 5-May-1989 Greg Hammett:  Upgraded to use the full expressions as
C given in the NRL formulary for collisions with a Maxwellian
C background.  Previously, we had been using expressions valid only in
C the vti<<v<<vte limit.
 
c Sept. 12, 1984 Greg Hammett:	In order to prevent f from
c oscillating around 0 above the injection energy in an unstable
c manner, Rob and I used to artificially enhance the diffusion
c term relative to the drag term.  Later I discovered that by
c  differencing the drag term in a different way, stability could be
c achieved, f would remain positive, and we would all be happy.
c This improved differencing scheme is implemented in preadi.
 
C
C     ...ENERGY DEPENDENCE
C
c Confusing but true:
c Drag(i)	= drag at 	Enerev(i-.5)
c Diffe(i)	= diffusion at  Enerev(i+.5)
c Out(i)	= Sqrt(E) at 	Enerev(i)
c Scat(i)	= PAS rate at   Enerev(i)
c
 
	real zloglam(mxions+1)
	real zab(mxions+1)
	real zzb(mxions+1)
	real ztempb(mxions+1)	! temperature (keV) of each species
	real zdenb(mxions+1)	! density (/m**3) of each species
 
c zpsicon = 2 / sqrt(pi):
	zpsicon=2.0/sqrt(4.0*atan(1.0))
 
	zdenb(1)=drelec(nshe)*1.e6
	ztempb(1)=trelec(nshe)/1.e3
	zzb(1)=-1
	zab(1)=1./1836.1
	do 5 iion=1,nions
	zdenb(iion+1)=drions(nshe,iion)*1.e6
	ztempb(iion+1)=trions(nshe,iion)/1000.
	zzb(iion+1)=nZions(iion)
5	zab(iion+1)=nAions(iion)
 
c initialize to zero:
	do 10 i=0,maxen
	drag(i)=0.0
	drage(i)=0.0
	DragI(I)=0.0        ! tbt
	diffe(i)=0.0
	DiffeI(I)=0.0       ! tbt
	diffee(i)=0.0
	out(i)=0.0
	scat(i)=0.0
10	continue
 
	if(nthermi .eq. nfasti) then
		lselfc=1
	else
		lselfc=0
	endif
 
c scattering rates are undefined at i=0 (Enerev=0.0):
	DO 100 I=1,maxen
 
	en=(enerev(i)+enerev(i-1))/2.
 
c Coulomb logarithms:
	call coulog(en/1000.,ab,rZbeam,zdenb,ztempb,zab,zzb,zloglam,
	1   nions+1,Btor/1.e4,llstix)
 
	do 25 ispec=1,nions+1
	data lselfc /0/ !1 to do self-collsions
	if(lselfc .ne. 1 .and. ispec .eq. nfasti+1) goto 25
	if(zdenb(ispec) .eq. 0.0) goto 25
 
c rnu0 = 4 pi e1**2 e2**2 n2 zloglam / (m1**2 v1**3) :
c standard definitions from the NRL formulary:
 
	znu0 = 9.0159e-8 *rZbeam**2 *zzb(ispec)**2
	1   *(zdenb(ispec)/1.e6)*zloglam(ispec)/( ab**.5 * en**1.5)
 
	zx=zab(ispec)*en/(ab*ztempb(ispec)*1000.)
	zpsi=zpsicon*gami(1.5,zx)
	znupar=znu0*zpsi/zx
 
	drag(i)   =drag(i)    +2*en**2.5*znupar/(ztempb(ispec)*1000.)
	diffe(i-1)=diffe(i-1) +2*en**2.5*znupar
	if(ispec .eq. 1)then
	    drage(i)=2*en**2.5*znupar/(ztempb(ispec)*1000.)
	    diffee(i-1)=2*en**2.5*znupar
	endif
 
C dragi and diffei are used in fstart to calculate a (numerically)
C Maxwellian f which is in equilibrium with the thermal ions.
c Use the thermal species equivalent to fast ion species, or if that
C isn't available, use the first ion:
 
	if(ispec .eq. nthermi+1)then
	    dragi(i)=2*en**2.5*znupar/(ztempb(ispec)*1000.)
	    diffei(i-1)=2*en**2.5*znupar
	endif
	if(ispec .eq. 2 .and. nthermi .eq. nfasti) then
	    dragi(i)=2*en**2.5*znupar/(ztempb(ispec)*1000.)
	    diffei(i-1)=2*en**2.5*znupar
	endif
 
25	continue
 
c Now calculate SCAT(i) and OUT(i):
 
	en=enerev(i)
	out(i)=1./sqrt(enerev(i))
 
	do 50 ispec=1,nions+1
	if(lselfc .ne. 1 .and. ispec .eq. nfasti+1) goto 50
	if(zdenb(ispec) .eq. 0.0) goto 50
 
c rnu0 = 4 pi e1**2 e2**2 n2 zloglam / (m1**2 v1**3) :
c standard definitions from the NRL formulary:
 
	znu0 = 9.0159e-8 *rZbeam**2 *zzb(ispec)**2
	1   *(zdenb(ispec)/1.e6)*zloglam(ispec)/( ab**.5 * en**1.5)
 
	zx=zab(ispec)*en/(ab*ztempb(ispec)*1000.)
	zpsi=zpsicon*gami(1.5,zx)
C
C begin DMC patch -- faulty unix workstation math library
	if(zx.gt.85.0) then
	  zexp=0.0
	else
	  zexp=exp(-zx)
	endif		! end DMC patch
 
	znuperp=2*znu0*(zpsi *(1.-0.5/zx) + zpsicon*sqrt(zx)*zexp)
 
	scat(i)=scat(i)+znuperp/4.
 
50	continue
 
100	continue
 
C
C     CALCULATE TAU-CX AS A FUNCTION OF ENERGY
C
 
	if(lcxlos)then
 
	DO 300 I=0,maxen
 
	en=enerev(i)
! CX rate including finite neutral temperature:
	erel=en/ab
	s1cxv=sigvbt(erel,tnus/ap,'CX')
	RATE(I)=DENUS*s1cxv
300	continue
 
	else
 
	do 310 i=0,maxen
310	rate(i)=0.0
 
	endif
 
	return
	END
