	subroutine fpgeo(zth0,zlam,zRlocal,zJacB,zBlocal,zthdot,
	1   zvpar)
c
c Fokker-Planck geometry map.
c
c Subroutine to return various magnetic geometry parameters as a
c function of poloidal angle.  All of the geometry dependent
c features of SCCF are in this subroutine and fpgeoinv and fpgeogrd.
c At the moment, we use low beta, concentric circular flux surface
c approximations, but this could be improved.
c
c Input:
c  zth0	  poloidal angle
c  zlam	  mu/E pitch angle variable of particle orbit
c
c Output:
c  zRlocal    R (local major radius)
c  zJacB      Jacobian_local * zBlocal
c  zBlocal    local magnetic field
c  zthdot     d theta /dt
c  zvpar      vpar/v
c
c (Implicit inputs also taken from fppcom common block).
 
	include 'FPPCOM'
 
	real zj(2,2)
 
C With the proper choice of the poloidal angle (unfortunately, TRANSP
C and FPP do not use this special choice for the poloidal angle, so the
C Jacobian has to be explicitly calculated), the exact Jacobian J is
c J=(q R**2 / It), where It= (R B_tor) is a flux surface quantity and
c q is the inverse rotational transform and is also a flux surface
c quantity.
C
C For Bp<<Bt plasmas this can be approximated by:
c Jac*B = q*R
c
c for concentric circular flux surfaces we will further approximate
c R as a constant Rmaj, until we get a more detailed map between
c theta and real space:
c
c	zRlocal=Rmaj+rminpro(nshe)*cos(zth0)
c	zJacB=qpro(nshe)*Rmaj
c
c we now have a more detailed map.  Use the real thing:
 
	zx=rminpro(nshe)/rmin
	call momry(zx,zth0,1,zRlocal,zY,zj)
	zj2=zj(1,1)*zj(2,2)-zj(1,2)*zj(2,1)
 
c assuming Btor*R is a constant:
	zBlocal=Btor*Rmaj/zRlocal
 
	zJacB=pifpp*qpro(nshe)/zx/tflxtot*zj2*zRlocal*zBlocal
 
	zvpar2=1.-zlam*zBlocal
	if(zvpar2 .lt. 0.0)then
c	  write(lsysout,*)'fpgeo: zvpar2=',zvpar2
	  zvpar2=0.0
	endif
        zvpar=sqrt(zvpar2)
        zthdot=zvpar/zJacB
 
	return
	end
