C******************** START FILE FPICHMN.FOR *************************
C-----------------------------------------------------------
C  FPICHMN
c  Greg Hammett July 14, 1988
c  Interface control routine for FPP's model for ICH heating.
c
c evolved from ICRFMN.FOR, written by D. McCune in June 1982
C
C  DMC -- APR 1991
C   LOGIC COMMUNICATIONS BACK TO TRANSP REVISED:  IF THE ICRF MINORITY
C  SPECIES IS NON-THERMAL SET   NTHRF=2 ... IF THE ICRF MINORITY SPECIES
C  IS THERMAL SET NTHRF=1
C
C--------------------------------------------------
C  DMC -- TIMESTEP CONTROL ADJUSTMENT -- MAY 1990
C   THIS MODULE FITS BETTER INTO THE TRANSP TIMESTEP CONTROL IF
C   THE TEST FOR ICRF TURNING ON/OFF IS DONE USING COMMON VARIABLE
C   "TB" INSTEAD OF "TA".  "TA", "TB" ARE THE TIMES OF THE BEGINNING
C   AND END OF THE CURRENT ENGERY BALANCE TIMESTEP, RESPECTIVELY.
C
C   REFERENCES TO TRANSP COMMON VARIABLE "TA" HAVE BEEN REPLACED
C   WITH REFERENCES TO "TB"
C
C--------------------------------------------------
C  DMC REVIEWING THE CODE APR 1988
C   THE MODEL SEEMS TO WORK AS FOLLOWS:
C   ICRF MINORITY SPECIES DENSITY RHMIN = FRMINI * ELECTRON DENSITY
C    (RHOEL).  WHEN THE ICRF IS ON THE CURRENT MODELS RETURN
c    the average parallel and perpendicular energies of the minority
c    ions.
C
C   THE DENSITY IS CONTINUOUS (CF PRFNSP SUBROUTINE) BUT THE 'TEMPERATURE'
C   IS A STEP FUNCTION CHANGING AS PER DTICRF CONTROL.  Thus the minority
c   energy is DISCONTINUOUS, AND SO THEN IS THE TOTAL PLASMA BETA
C
C   THE INTERFACE SUPPORTS ONE ON TIME AND ONE OFF TIME (THERE CAN NOT
C   BE MULTIPLE ICRF HEATING PULSES IN ONE DISCHARGE).
C
C--------------------------------------------------
C
C  (1) DECIDE WHEN TO CALL ICRF MODEL
C  (2) WHEN ICRF HEATING IS TURNED ON, "DE-THERMALIZE" THE
C      MINORITY SPECIES BY DELETING IT FROM THE THERMAL ION DENSITY
C  (3) WHEN THE ICRF HEATING IS TURNED OFF, "RE-THERMALIZE" THE
C      MINORITY SPECIES-- LET IT COOL DOWN GRADUALLY; WHEN
C      EQUILIBRIUM IS REACHED, ADD THE MINORITY SPECIES BACK INTO
C      THE THERMAL ION DENSITY AND RESET THE ICRF TIME SWITCHES
C      SO THAT ICRF REMAINS OFF FOR THE REMAINDER OF THE RUN.
c
C
	SUBROUTINE fpichmn
C
	INCLUDE 'TRCOM'
	logical jicrf ! .t. = set up fpp to do an icrf calculation
	real zdepdum(mifpen,mifpxi,mifpsh)  ! dummy beam source
 
c initialize dummy beam source:
	do ien=1,mifpen
	do jxi=1,mifpxi
	do ish=1,mifpsh
	zdepdum(ien,jxi,ish)=0.0
	enddo
	enddo
	enddo
 
	jicrf=.true.
C
C--------------------------------------------------------------
c
C  IS ICRF ON YET?
C
	IF(TB.LE.TCRFON) GO TO 1000
C
C  IT HAS BEEN TURNED ON.
C
C  CHECK TIMESTEP CONTROL-- TIME TO DO ANOTHER ICRF STEP YET?
	IF(TB .lt. TLICRF) GOTO 1000
 
	CALL DATCPU(ZTIM1) !Time ICH heating model
	ztimenow=TB
	call rfpdat(ztimenow) !Set ICH parameters for present time
 
c set defaults for the on and off times of the antennas:
	do 15 iant=1,nicha
	if(tonicha(iant) .eq. -epsinv
	1   .and. tofficha(iant) .eq. -epsinv) then
	    tonicha(iant)=tcrfon
	    tofficha(iant)=tcrfof
	endif
15	continue
 
c find maximum ICH power observed to date:
        zprftot=0.0
        do 30 iant=1,nicha
	if(tb .ge. tonicha(iant) .and. tb .le. tofficha(iant))
	1   zprftot=zprftot+prficha(iant)
30	continue
	pichtot=zprftot
	if(zprftot .gt. fpHmax) fpHmax=zprftot
 
c don't bother starting FPP yet if the RF power is still zero:
	if(tlicrf .lt. tcrfon .and. zprftot .le. 0.0) goto 1000
 
c determine whether f has thermalized and we can turn off the RF model:
	if(fpHmin .le. 0.0) fpHmin=.01
	if(TB .ge. tcrfof) then
            zptot=0.0
            do 50 j=lcentr,ledge
50          zptot=zptot+(piicrf(j)+peicrf(j))*dvol(j,2)
	      if(zptot .eq. 0.0) call aborter(nonlin,
	1     'powers should not be zero yet!')
            if(zptot .le. fpHmin*fpHmax) goto 200
	endif
 
c if this is the first call, initialize FPP:
	if(tlicrf .lt. tcrfon)then
	tlicrf=tcrfon
c set up FPP default values, or values (such as the Energy grid
c maximum) that must be set only once:
	call fpdflt
	endif
 
	call bmzvol !find beam zone volumes, needed even without beams.
	call tkzero	! initalize straightline tracker for PARMS:
	CALL PARMS		!SET UP MAGNETICS SPLINES, ETC.
 
	if(nicrf .ne. 1)then
	call fpichsim
	else
 
c initialize FPP's velocity space grids:
	call fppblk2  ! initialize some of FPP's common block
	call fpvgrid(jicrf)
 
c read FPP input data from TRANSP common:
	call trtofp(jicrf)
 
c Do an FPP calculation, storing the minority ion f in the
c proper place in fpfrad, and noting that there is no
c beam deposition allowed for the minority ions in TRANSP at
c the moment.  (We will work towards allowing RF heating of
c a "beam" species, which might be a hydrogen minority.  But
c because TRANSP only has a single non-hydrogenic species,
c we still have to treat He3 minority heating in a special way.
c In particular, we are specifying the minority density profile
c at input, while TRANSP would solve for the beam density and
c its corresponding thermal species density.)
 
	call fpprf(fpfrad(1,1,1,mibsp1),zdepdum)
 
c Convert FPP output to TRANSP format:
	call fptotric
	endif
c
	CALL DATCPU(ZTIM2)
	ZCTIM=ZTIM2-ZTIM1
	WRITE(NOUT,2000) ZCTIM
	WRITE(NONLIN,2000) ZCTIM
 2000	FORMAT(' *** FPP ICRF TIMESTEP COMPLETED ... CPU = ',1PE10.3,
     >  ' SECONDS.')
C
	CPICRF=CPICRF+ZCTIM
C
C  DMC APR 1991
C  MARK THE MINORITY SPECIES AS DETHERMALIZED.  IF ON THIS CALL THE
C  THERMALIZATION STATUS CHANGED, RE-EVALUATE ION-ELECTRON COUPLING.
C  UPDATE THERMAL ION-ELECTRON COUPLING (QIE)
C
	NTHRF=2
	IF(NTHRF.NE.NTHRFP) CALL HEATI
C
 
	GO TO 1000
C
C---------------------------------------
C
C  THERMALIZATION COMPLETE-- TURN OFF ICRF FOR GOOD,
C  CLEAR ELECTRON HEATING TERM, RECOMPUTE THERMAL ION DENSITY AND
C  ION-ELECTRON COUPLING
C
 200	CONTINUE
C
	WRITE(NONLIN,1001)
	WRITE(NOUT,1001)
 1001	FORMAT(' %MINORITY SPECIES THERMALIZED')
C
	DO 250 J=1,MZONES
	  PIICRF(J)=0.0
	  PEICRF(J)=0.0
	  pminthi(j)=0.0
	  pmine(j)=0.0
	  pmincx(j)=0.0
	  pminloss(j)=0.0
	  pminstor(j)=0.0
	  piche(j)=0.0
	  pichmc(j)=0.0
	  pichmin(j)=0.0
	  pichthi(j)=0.0
	  pichall(j)=0.0
	    eminpar(J)=0.5*TI(J,2)
	    eminper(J)=TI(J,2)
 250	CONTINUE
 
C
C  TURN OFF ICRF
C
	TCRFON=EPSINV
	TCRFOF=EPSINV
	pichtot=0.0
C
C  RETHERMALIZE THE MINORITY ION SPECIES
C  RECOMPUTE QIE
	NTHRF=1
	IF(NTHRF.NE.NTHRFP) CALL HEATI
C
C------------------------------------
C  EXIT
C
 1000	CONTINUE
	RETURN
C
	END
