C******************** START FILE FPMAIN.FOR ; GROUP FPMAIN *************
C==============================================================
c Greg Hammett (GWH) March 19, 1987.
c
c This interfaces TRANSP to the FPP package, a bounce-averaged
c Fokker-Planck treatment of neutral beam injection and ICRF heating in an
c axisymmetric tokamak.  It includes the usual collisional effects such as
c energy drag, energy diffusion, and pitch-angle scattering.  It also has
c models for unconfined orbit losses, charge-exchange losses, and radial
c transport of fast ions.
c
c FPMAIN is based loosely on SOURCE:NBMAIN.FNB.  Fragments of the
c original NBMAIN are preceeded by "CNBM:".
c FPMAIN is called for FPP calculation of beam heating.
c FPICHMN is called for FPP calculation of ICRF heating.
c parts of the code which can be improved in the future are marked
c with a "?".
c There are a number of limitations at the moment which may
c be lifted in the future:
c
c 2. compression turned off.
c 3. Arbitrarily assumed 50% fast ion CX recaptured immediately.
c 4. Beam-beam CX and collisions ignored.
c 6. Radial transport of beam turned off.
c 7. Simple loss model, particles can't orbit outside of the plasma.
c 9. No FLR effects in beam deposition.
c 10. Only a poloidally symmetric, single species neutral profile is used.
c
c For more notes about limitations, see the subroutines TRtoFP, FPtoTR,
c and FPdflt.
c
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C Subroutine FPMAIN Architecture Summary:
C **************************************
C
C	ON FIRST CALL:
C
C		INITIALIZES THE CALCULATION
C
C		ZEROES VARIOUS QUANTITIES
C		
C		CALLS SUB. NBCLR, WHICH ZEROES MORE QUANTITIES
C
c 		Calls FPdflt initialization routines.
C
C		CALLS SUB. BMZVOL, WHICH SETS UP BEAM ZONE VOLUMES
C
C		CALLS SUB. RDBBSG WHICH READS BB FUSION NEUTRON X-SECTS
C
C	ON LATER CALLS:
C
C		FIGURES OUT BEAM PARAMETERS FOR THIS TIMESTEP
C
C		CALLS SUB. ZERO AGAIN
C
C               CALLS LIMCHK - BE SURE PLASMA (MHD EQUILIBRIUM DESCR.)
C                       IS CLEAR OF ALL LIMITERS (LIMLOS DESCR.)
C	
C		CALLS SUB. PARMS, WHICH PUTS PLASMA PARAMETERS INTO
C		THE FORM REQUIRED BY THE BEAM CODE
C
C                  PARMS CALLS BPARMS - MAGNETIC FIELD COEFFICIENTS
C               PARMS SETS UP THE FAST BILINEAR
C                  (MAGNETIC COORDS) --> (R,Y) "INVERSE MAP" USED
C               TO EVALUATE NEUTRAL BEAM FLR EFFECTS.
c		    (No FLR effects of this kind in FPP.)
C
c		call TRtoFP  Converts Transp --> FPP data
c		Calls FPP routines.
c		call FPtoTR  Converts FPP --> Transp output
c
cnbm:C	CALLS SUB. OUTPTB TO NORMALIZE AND SMOOTH BEAM CODE OUTPUT
cnbm:C	(THIS INVOLVES SOME FURTHER CALCULATIONS)
cnbm:C
cnbm:C		CALLS SUB. FLIPR TO REARRANGE THE BEAM DISTRIBUTION
cnbm:C		FUNCTION FOR EASIER ACCESS
cnbm:C
cnbm:C		CALLS BBNEUT TO CALCULATE BEAM-BEAM NEUTRON PROD.
cnbm:C
cnbm:C		CALLS SUB. CENSM TO SMOOTH BEAM CODE OUTPUT
C
C	
C
C
cnbm:C	FOOTNOTE:  MUCH OF THE RANDOM NUMBER GENERATOR SET UP
cnbm:C		   IS DONE IN SUB. MCINIT.FOR, WHICH IS CALLED OUTSIDE
cnbm:C		   OF THE BEAM CODE.  THIS SUB. ALSO INITIALIZES
cnbm:C		   THE 2-D (ENERGY X TEMP) MAXWELLIAN AVERAGED RATE
cnbm:C		   COEFFICIENTS FOR ATOMIC PROCESSES
C
C
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
cnbm:C=======================================
cnbm:C
cnbm:C	UPGRADE TO ALLOW PARTICLES TO TRAVEL OUTSIDE OF THE PLASMA,
cnbm:C	BUT WITHIN A DEFINED LIMITER SYSTEM (DEFINED BY SUBROUTINE
cnbm:C	LIMLOS).
cnbm:C	ROB GOLDSTON  JULY 21, 1983
cnbm:C
cnbm:C=======================================
C==========================================================
cnbm:C  D. MC CUNE MAR 1985
cnbm:C   GEOMETRY UPGRADE
cnbm:C  FOR ALL GEOMETRIES BEAM PTCLS ARE CONSIDERED TO ORBIT IN
cnbm:C  (XI,THETA) SPACE WHERE XI IS THE NORMALIZED FLUX COORDINATE
cnbm:C  AND THETA THE GENERALIZED POLOIDAL ANGLE
cnbm:C   FOR CONCENTRIC CIRCULAR GEOMETRY THIS IS A SLIGHT CHANGE
cnbm:C  (XION=RION/RMINOR)
cnbm:C
cnbm:C  BEAM ION COORDINATES
cnbm:C   GUIDING CENTER
cnbm:C    NEW COORDINATES
cnbm:C     XION  XI POSITION
cnbm:C     TH    THETA POSITION
cnbm:C     **> ALL MONTECARLO PTCLS STORED IN ARRAYS XIAY, THAY BTW. STEPS
cnbm:C    OLD COORDINATES
cnbm:C     RION  **NOT USED**   STORAGE ARRAY RAY **NOT USED** (MINOR RADIUS)
cnbm:C     TH    USED, DEFINED FROM XION, TH
cnbm:C     RMJION  USED, DEFINED FROM XION, TH (MAJOR RADIUS)
cnbm:C     YION    USED, DEFINED FROM XION, TH (VERTICAL DISPLACEMENT)
cnbm:C
cnbm:C   FLR
cnbm:C    NEW COORDINATES
cnbm:C     XIFLR  XI POSITION OF PTCL ON GYRO (FOR CHOSEN PHASE ANGLE)
cnbm:C     THFLR  THETA POSITION OF PTCL ON GYRO
cnbm:C    OLD COORDINATES
cnbm:C     RMNFLR **NOT USED** MINOR RADIUS
cnbm:C     RMJFLR  USED, DEFINED FROM XIFLR,THFLR (MAJOR RADIUS)
cnbm:C     YFLR    USED, DEFINED FROM XIFLR,THFLR (VERTICAL DISPLACEMENT)
cnbm:C
cnbm:C  ** BEAM DISTRIBUTION FUNCTION **
cnbm:C    THE SPATIAL GRID IS REINDEXED BY (XIFLR,THFLR) ZONES INSTEAD
cnbm:C    OF (RMJFLR,YFLR) ZONES.
cnbm:C    THE NUMBER OF THETA ZONES IS INDEPENDENTLY CONTROLLABLE FOR EACH
cnbm:C    THETA ZONE ROW - CF SUBROUTINE BMZINI
cnbm:C      INDEXING INFORMATION  **NOTE DUMMY INDEX**
cnbm:C        NXSKPB - EVERY NXSKPB'TH PLASMA ZONE BDY IS A BEAM ZONE
cnbm:C                 "XI" BDY; NXSKPB MUST BE A DIVISOR OF NZONES, THE
cnbm:C                 NUMBER OF PLASMA ZONES.
cnbm:C        DXITK  - "XI" WIDTH OF **ALL** BEAM ZONES
cnbm:C        NXTBZN - THERE ARE THIS MANY BEAM ZONE "XI" BOUNDARIES
cnbm:C                 *BEYOND* THE PLASMA BDY (XI=1) (EXTERNAL ZONES)
cnbm:C
cnbm:C
cnbm:C      FOR EACH REGION J BTW. BEAM ZONE "XI" BDY'S THERE MAY BE
cnbm:C      ONE OR MORE BEAM DIST FCN ZONES PARAMETRIZED BY "THETA":
cnbm:C        LCM1=LCENTR-1 (**DUMMY ZONE OFFSET**)
cnbm:C        NTHBBM(J+LCM1) - NUMBER OF THETA ZONES IN REGION J
cnbm:C        NTHZSM(J+LCM1) - TOTAL NUMBER OF ZONES, REGIONS 1 THRU J INCL.
cnbm:C              (NTHZSM(LCM1)=0)
cnbm:C
cnbm:C  THIS INFORMATION SUFFICES FOR A QUICK CALCULATION OF ...
cnbm:C        NFLR2 - THE SPATIAL BEAM DIST FCN INDEX OF THE BEAM ION IN ITS
cnbm:C                CURRENT GYRO POSITION (*NOT GUIDING CENTER*)
cnbm:C        (NFLR - THE TRANSP ZONE INDEX OF THE GYRO POSITION IS STILL
cnbm:C                DEFINED)
cnbm:C
cnbm:C  *****************
cnbm:C  BEAM-BEAM REACTIONS WILL BE EVALUATED ON THE BEAM DIST FCN (NFLR2)
cnbm:C  GRID INSTEAD OF THE TRANSP GRID
cnbm:C
cnbm:C  MISCELLANEOUS QUANTITIES DESIRED ON A 2D GRID WILL ALSO BE ADDED.
C
C
	subroutine FPmain(icall)
C
C  INTERFACE WITH TRANSP
C    ICALL = 1  INITIALIZE FOR CALCULATION
C          = 2  PERFORM CALCULATION FOR TIMESTEP  TBM1  TO  TBM2
C
C  INPUT PROFILES--
C      FROM TRANSP:
C         TE,TI,RHOEL,RHOI,DEN0MN,Q,etc. (see subroutine TRTOFP)
C
C  OUTPUT PROFILES CALCULATED --
C         PBE,PBI, SORBE,SORBH,SORBN0
C	  FBM (FCT. OF 2-D SPACE, PITCH-ANGLE, AND ENERGY)
c	  (see subroutine FPtoTR)
C
C
C-----------------------------------------------------------------
C
C
C  -- DMC -- JULY 1990 -- CODE SHARED WITH NBMAIN HAS BEEN REMOVED
C  TO SEPARATE CALLABLE ROUTINES:
C     NBINIT0 -- CALLED ONCE AT START OF RUN
C     NBSTART -- CALLED AT BEGINNING OF EACH BEAM TIMESTEP
C
C  -- DMC -- JAN 1992 -- added XZBEAMI=0.0 after loops to clear
C     what was set by INISPC; treat similar to ABEAM, LSBEAM, NBS.
C
C-----------------------------------------------------------------
C
C  TRANSP COMMON BLOCKS --
C
	INCLUDE 'TRCOM'
	logical jicrf ! .t. for TRtoFP to setup for an ICRF calculation
c
C
C
C  START OF EXECUTABLE CODE
C
C  ICALL=1: INITIALIZATION CALL
C
	jicrf=.false. !will be setting up for NBI not ICRF calculation
	GO TO (10,20) ICALL
C
C  1.  INITIALIZATION
C
10	CONTINUE
 
	CALL NBINIT0
 
c set up FPP default values, or values (such as the Energy grid maximum)
c that must be set only once:
 
	Call FPdflt
 
C
	RETURN
C
C
C
C
C  2.  STEP ON THE CALCULATION
C
20	CONTINUE
C
c don't perform Fokker-Planck calculations if the beam or ICRF are not
c on yet:
	zstart=min(tbon,tcrfon)
	if(tbm1 .lt. zstart) return
 
c FPP stays on after the beams turn off, until the power from the
c slowing down of the beams drops to a sufficiently low level:
 
	if(fpHmin .le. 0.0) fpHmin=.001
	zstop=max(tboff,tcrfof)	
	if(tbm1 .gt. zstop)then
 
c calculate present power from slowing down of the beams:
	    zptot=0.0
	    do 30 j=lcentr,ledge
30	    zptot=zptot+pbi(j)+pbe(j)+pbth(j)
	    if(zptot .lt. fpHmin*fpHmax)then
		call nbclr
		nlbeam=.false.
		return
	    endif
	endif
 
 
	CALL NBSTART
 
 
c Set up FPP's velocity space grid, which may also be used by TRANP's
c DEP routine:
	call fppblk2  ! initialize some of FPP's common blocks:
	call fpvgrid(jicrf)
 
c*********************************************************************
c Beginning of Monte-Carlo Beam deposition, if desired.
c FPP has its own deposition routine (DEPO).  But DEP
c has the advantage of already working with rotation, it has a
c more realistic beam divergence model, and it is faster
c if there are lots of beam lines (it uses the same number of MC tracks
c but splits them up between the beam lines!).
c
c DEP is called NDEP0 times.  Each call follows a single track through
c the plasma.  Additional calls will improve statistics on the
c DEP output:  SORBE (Elec. sce) SORBH (CX therm ion loss term)
c SORBN0 (CX therm neut sce term) BN00 (1st gen beam neut. dens.)
c BPSHIN (Beam shine-thru power)
c
 
	if(nfpdepmd .eq. 2) then !use Monte Carlo Deposition for FPP
 
	CALL DATCPU(ZTIM1)	!CPU time of deposition
 
C	SET UP TRKRB ZONES:  NO PITCH ANGLE ZONES, NO ZONES OUTSIDE THE
C       PLASMA - CF SUBROUTINES SHRINK, RBLOAT (DEP.FOR)
	CALL SHRINK
c
c beam species loop:
c
	do 80 isb=1,nsbeam
	lsbeam=isb
	call inispc
	zwgrat=0.0    !default energy normalizer
	zwrats=0.0    !default particle normalizer
 
	if(xninjs(lsbeam) .le. 0.0) goto 75 ! skip if beams are off now
	ZENNEW=PINJS(LSBEAM)*6.2418E18*DELT ! energy deposited this time step
	ZBNEW=BMCURS(LSBEAM)*6.2418E18*DELT ! particles " " "
 
	zwgtot=0.0 !Monte Carlo weight of particles deposited so far
	do 50 idep=1,ndep0
	call dep(zwgtot)
50	continue
 
c Normalize DEP outputs:
	zwgrat=zennew/zwgtot	    !energy normalizer
	zwrats=zbnew/float(ndep0)   !particle normalizer
75	call depnrm(zwgrat,zwrats)
 
80	continue
 
C-----------------------------------------------------------------------
C  dmc June 1992 -- option to use DEP to compute a fast neutral density
C  in a 3d box along a selected beam axis
C
	IF(NBBOX.GT.0) CALL DEPBOX
C
C-----------------------------------------------------------------------
 
c zero to avoid accidental use without proper INISPC calls:
	nbs=0
	lsbeam=0
	abeam=0.0
	XZBEAMI=0.0
 
C  RESTORE EXPANDED BEAM DIST FCN ZONE GRID FOR ORBITTING
	call rbloat
 
C  DEP CPU TIME
	CALL DATCPU(ZTIM2)
	CPBDEP=CPBDEP+(ZTIM2-ZTIM1)
 
	endif
 
c end of Monte-Carlo beam deposition DEP
c*********************************************************************
 
C
C  ZERO OUT NB*SIGMA*V SUMS (BEAM-BEAM INTERACTIONS USED IN DEP)
C
	CALL NBSV00
c
c At present, FPP can only handle one beam species:
c
 
	do 100 isb=1,nsbeam
	lsbeam=isb
	call inispc  !get parameters for this beam species
 
C read FPP input data from TRANSP common:
	call TRtoFP(jicrf)
 
C we should be all set up for an FPP run now.  So do it:
	call fpprf(fpfrad(1,1,1,lsbeam),fpdep(1,1,1,lsbeam))
 
C convert FPP output to TRANSP formats:
	call FPtoTR
 
c normalize all outputs, and copy to arrays for each beam species:
	call outptb(TBM2)
 
100	continue
 
c zero to avoid accidental use without proper INISPC calls:
	nbs=0
	lsbeam=0
	abeam=0.0
	XZBEAMI=0.0
 
c calculate outputs which are summed over all beams:
	call outptb2
 
	RETURN
	END
