	subroutine fpminf0(f,x,n,xlb,xub,xtol,ferr)
c
c minimize a multivariate function f which is known to have
c a minimum value of 0 and behaves quadratically near the minimum.
c The x(i) are constrained to lie within the boundaries
c xlb(i) < x(i) < xub(i).  The minimization iteration finishes
c when dx<xtol or f<ferr.
c
c The subroutine uses a modified Newton algorithm, with precautions
c to avoid overly large step sizes.
c
c This is to replace the IMSL routines which have bugs!
c
	external f  ! subroutine to evaluate the function to be minimized
	real x(n)   ! input: initial guess at location of minimum
		    ! output: final value for location of minimum
	integer n   ! dimension of the problem
	real xlb(n) ! lower bounds for x(n)
	real xub(n) ! upper bounds for x(n)
	real xtol   ! iteration completed if dx<xtol or
	real ferr   ! if f<ferr.
 
	real xtest(10)	! work space
	real gradf(10)	! grad f work space:
 
	integer maxiter ! maximum number of iterations allowed
	integer niter   ! number of iterations so far
 
	if(n .gt. 10) call aborter(6,'minf error: n>10!')
 
	deltax=5*xtol	!standard step size for derivatives.
	maxiter=10000
	niter=0
 
	call f(n,x,f0)
	if(f0 .lt. ferr) return
 
c evaluate gradient:
15	do 30 i=1,n
	gradf(i)=0.0
	do 20 j=1,n
20	xtest(j)=x(j)
	deltax=abs(deltax)
	if(xtest(i) .eq. xub(i))deltax=-deltax
	xtest(i)=xtest(i)+deltax
	call f(n,xtest,f1)
	gradf(i)=(f1-f0)/deltax
	if(x(i) .ge. xub(i)) gradf(i)=max(0.0,gradf(i))
	if(x(i) .le. xlb(i)) gradf(i)=min(0.0,gradf(i))
30	continue
 
c find optimum step size for the correction to x:
	sum=0.0
	do 40 i=1,n
40	sum=sum+gradf(i)*gradf(i)
	if(sum .eq. 0.0) then
	    deltax=deltax*3
	    niter=niter+1
	    if(niter .gt. maxiter) goto 999
	    goto 15
	endif
	sumrt=sqrt(sum)
 
c sufficiently near the minimum, f is quadratic, so the optimum
c step size is twice the Newton iteration value.  Try this
c larger step size first:
	alpha=2*f0/sum
	dx=alpha*sumrt
 
c find new provisional x:
	do 50 i=1,n
	xtest(i)=x(i)-alpha*gradf(i)
	xtest(i)=max(xtest(i),xlb(i))
	xtest(i)=min(xtest(i),xub(i))
50	continue
 
	call f(n,xtest,ftest)
	if(ftest .le. f0/2) goto 60
 
c make a new guess for f with a smaller step size:
53	alpha=alpha/2
	dx=alpha*sumrt
	do 55 i=1,n
	xtest(i)=x(i)-alpha*gradf(i)
	xtest(i)=max(xtest(i),xlb(i))
	xtest(i)=min(xtest(i),xub(i))
55	continue
	call f(n,xtest,ftest)
	if(dx .lt. deltax) goto 60
	if(ftest .ge. f0) then
	    niter=niter+1
	    if(niter .gt. maxiter) goto 999
	    goto 53
	endif
 
c one iteration completed, here is the new x:
60    	do 65 i=1,n
65	x(i)=xtest(i)
 
c magnitude of dx was:
	f0=ftest
	if(dx .lt. xtol) return
	if(f0 .lt. ferr) return
	deltax=min(xtol*5,dx/2)  ! set up derivative step size
	niter=niter+1
	if(niter .gt. maxiter) goto 999
	go to 15
 
999	write(6,*) ' Failure in FPMINF0.  One possible problem is'
	write(6,*) ' that the plasma is not inside the vacuum vessel'
	write(6,*) ' everywhere.'
	call aborter(6,' Failure to converge in FPMINF0')
	end
