c FPPONLY.FOR contains routines needed for the standalone version of
c FPP only!  Some of these routines are simplified versions of
c equivalent TRANSP routines, and so these should not be used with
c TRANSP!
c
	subroutine nbimap(zR,zY,zXI,zTh,ier)
c
c A version of TRANSP's nbimap routine for the standalone FPP.
c Given real space coordinates (zR,zY), does an inverse map to
c find flux coordinates (zXI,zTH).
c
c IER= 0:normal, 1:marginal, 2:completely ouside of map.
c
c nbimap uses 2D interpolation, so it is very fast.  But fpinvmap
c must have been called already to set up the tables to be interpolated.
c fpinvmap uses nbimap2, which is a slow version of nbimap, because
c it does the actual inverse mapping by iteration.
c
	include 'FPPCOM'
	include 'FPONCOM'   ! FPP-only common block
 
c do 2D interpolation:
	zi=1+(zR-fpgRmin)/fpgdR
	zj=1+(zY-fpgYmin)/fpgdY
 
	ier=0
	if(zi .lt. 1.0)then
	    zi=1.0
	    ier=2
	endif
	if(zi .ge. nfpgR)then
	    zi=nfpgR-1
	    ier=2
	endif
 
c we are assuming updown symmetry:
	if(zj .lt. 1.0)zj=2.0-zj
 
	if(zj .ge. nfpgY)then
	    zj=nfpgY-1
	    ier=2
	endif
 
	i=zi
	j=zj
	zwgti1=zi-i
	zwgtj1=zj-j
	zwgti=1.0-zwgti1
	zwgtj=1.0-zwgtj1
 
	zXi=(fpgxi(i,j)*zwgti+fpgxi(i+1,j)*zwgti1)*zwgtj
	1   +(fpgxi(i,j+1)*zwgti+fpgxi(i+1,j+1)*zwgti1)*zwgtj1
 
	zTh=(fpgth(i,j)*zwgti+fpgth(i+1,j)*zwgti1)*zwgtj
	1   +(fpgth(i,j+1)*zwgti+fpgth(i+1,j+1)*zwgti1)*zwgtj1
 
	if(zXi .lt. 0.1) then
c call iterative invertor for more accuracy near the axis:
	    call nbimap2(zR,zY,zXi,zTh,ier)
	endif
 
	return
	end
 
	subroutine fpinvmap
c
c Sets up the inverse map arrays Xi(R,Y) and Th(R,Y) for use by
c the fast inverse map routine nbimap.
c
	include 'FPPCOM'
	include 'FPONCOM'   ! FPP-only common block
 
	real zRaxis !position of the magnetic axis
	real ztol   !tolerance for inverse map (cm)
	real ztolxi !ztol/rmin
	common /fpgonly2/ zRaxis,ztol,ztolxi
 
c set up fpgonly2 common block:
	call momry(0.0,0.0,0,zRaxis,zY,zjac)  !find the magnetic axis
	ztol=rmin/maxshe/10.
	ztolxi=ztol/rmin
 
	zpi=abs(acos(-1.0))
 
	fpgYmin=0.0
	fpgYmax=0.0
	call momry(1.0,0.0,0,fpgRmin,zy,zjac)
	fpgRmax=fpgRmin
 
c find minimum and maximum R and maximum Y:
 
	do 5 i=1,nfpgR
	zth=zpi*i/float(nfpgy)
	call momry(1.0,zth,0,zR,zY,zjac)
	fpgYmax=max(fpgYmax,zY)
	fpgRmax=max(fpgRmax,zR)
	fpgRmin=min(fpgRmin,zR)
5	continue
 
c expand boundaries a bit for safety:
c (if you change the factor of 1.2 here, change it in NBIMAP2 also):
 
	fpgYmax=fpgYmax*1.2
	zR=(fpgRmax+fpgRmin)/2.0
	fpgRmax=zR+(fpgRmax-zR)*1.2
	fpgRmin=zR+(fpgRmin-zR)*1.2
 
c assume up-down symmetry and only produce a map for the upper half:
 
	fpgdR=(fpgRmax-fpgRmin)/(nfpgR-1)
	fpgdY=(fpgYmax-fpgYmin)/(nfpgY-1)
 
	do 10 i=1,nfpgR
	zR=fpgRmin+(i-1)*fpgdR
	do 10 j=1,nfpgY
	zY=fpgYmin+(j-1)*fpgdY
	call nbimap2(zR,zY,fpgxi(i,j),fpgth(i,j),ier)
10	continue
 
	return
	end
 
	subroutine nbimap2(zR,zY,zXI,zTh,ier)
c
c A version of TRANSP's nbimap routine for the standalone FPP.
c Given real space coordinates (zR,zY), does an inverse map to
c find flux coordinates (zXI,zTH).
c
c IER= 0:normal, 1:marginal, 2:completely ouside of map.
c
c upon return, zxi will be between -pi and pi (probably).
c
c nbimap2 is slower than nbimap because it does the actual inverse
c mapping by iteration.
c
C I am now using my own Newton's method algorithm to do the inverse
C mapping.  It is much faster (by a factor of 10) than the IMSL routine
C NEQNF, which was 20% faster than NEQNJ (NEQNJ may be faster than
C NEQNF for large systems of equations, but is actually slower for a 2
C equation system).
C
C One may be able to further improve the speed of the algorithm my
C providing a better initial guess, perhaps using information about the
C elongation, etc.
C
C I am just using a simple Newton's method for now.  If the geometry is
C extremely complicated, perhaps with a concave boundary, then this
C algorithm may fail.  In that case, one can improve the algorithm by
C providing a better inital guess, and by providing reasonble limits on
C the step size to avoid increasing errors.
c
 
	include 'FPPCOM'
 
	real zRaxis !position of the magnetic axis
	real ztol   !tolerance for inverse map (in cm)
	real ztolxi !ztol/rmin
	common /fpgonly2/ zRaxis,ztol,ztolxi
 
	real zjac(2,2)
 
c provide a rough initial guess based on polar coordinates,
c (This insures that we at least start off on the correct side of
c the axis, which is a singularity in this coordinate system,
c and was found to be faster than using an initial guess based
c on the previous call of nbimap2):
 
	zxi=sqrt( (zR-zRaxis)*(zR-zRaxis) + zy*zy ) / rmin
c zxi is assumed to extend safely beyond the plasma edge by 20%:
c (if you change the factor of 1.2 here, do so in FPINVMAP also):
	if(zxi .gt. 1.2) zxi=1.2
	if(zxi .eq. 0.0) then
	    ier=0
	    zth=0.0
	    return
	endif
 
	zth=atan2(zy,zR-zRaxis)
 
	if(zxi .le. ztolxi) then
	    ier=0
	    return
	endif
 
c search for the root ourselves using Newton's method:
	do 20 iter=1,200  ! allow up to 200 iterations
 
	call momry(zxi,zth,1,zR1,zY1,zjac)
	zdR=zR-zR1
	zdY=zY-zY1
	zerr=abs(zdR)+abs(zdY)
 
	if(zerr .lt. ztol) then
 
c	if(zth .lt. -3.1415927 .or. zth .gt. 3.1415927) then
c	    type *,' zth out of bounds? in nbimap?'
c	endif
 
	ier=0
	return
	endif
 
	zdet=zjac(1,1)*zjac(2,2)-zjac(1,2)*zjac(2,1)
	zdxi=(zjac(2,2)*zdR-zjac(1,2)*zdY)/zdet
	zdth=(zjac(1,1)*zdY-zjac(2,1)*zdR)/zdet
	zxi=zxi+zdxi
20	zth=zth+zdth
 
	call aborter(lsysout,'nbimap2 failed!')
 
99	return
	end
 
	subroutine momry(zx,zth,ijac,zr,zy,zjac)
c
c This is a version of the TRANSP routine MOMRY for the standalone
c version of FPP.
c
c input: flux coordinates (zx, zth) and switch ijac.
c output: cartesion coordinates (zr,zy)
c 	  and, if ijac=1, jacobian zjac(2,2).
c
c zjac(1,1) = dR/dx
c zjac(1,2) = dR/dth
c zjac(2,1) = dY/dx
c zjac(2,2) = dY/dth
c
	include 'FPPCOM'
	include 'FPONCOM'
	real zjac(2,2)
 
c values of fourier moments interpolated to flux surface ZX:
	real zRmom(0:mxgmom)
	real zymom(0:mxgmom)
 
c values of dRm/dx and dYm/dx interpolated to flux surface zx:
	real zRmomp(0:mxgmom)
	real zymomp(0:mxgmom)
 
c
c For now, the standalone version of FPP uses a very simple model
c for the plasma geometry.
c
c	if(zx .le. 1.0)then
c	  zR0mom=Rmaj+fpshift0*(1.0-zx**2)
c	else
c	  zR0mom=Rmaj
c	endif
c
c  	zrmom(1)=zx*rmin/sqrt(fpellip)
c	zymom(1)=zx*rmin*sqrt(fpellip)
c
c	ngmom=1
 
	i1=lkupr(zx,xradzb,nshell)
c avoid non-physical extrapolation formulas (see below) if we
c are right at the boundary:
	if(zx .eq. 1.0) i1=nshell-1
 
	if(i1 .lt. nshell) then
 
	i2=i1+1
	if(i1 .gt. 0) then
	    zx1=xradzb(i1)
	else
	    zx1=0.0
	endif
	zx2=xradzb(i2)
 
	w1=(zx2-zx)/(zx2-zx1)
	w2=1.-w1
	do 10 im=0,ngmom
	zrmom(im)=w1*prmom(i1,im) + w2*prmom(i2,im)
10	zymom(im)=w1*pzmom(i1,im) + w2*pzmom(i2,im)
 
	else	! extrapolate for r>a:
 
c This extrapolation does not satisfy Maxwell's equations outside
c the plasma boundary, but is meant only to provide a coordinate
c system outside the plasma.  For example, q will not be properly
c evaluated from the Jacobian outside the plasma.
 
	do 15 im=0,ngmom
	zrmom(im)=prmom(nshell,im)
15	zymom(im)=pzmom(nshell,im)
	zrmom(1)=zrmom(1)*zx
	zymom(1)=zymom(1)*zx
 
	endif
 
	if(ijac .eq. 1) then
 
c simple model geometry information needed to find the Jacobian:
c	if(zx .le. 1.0)then
c	  zR0momp=-2*zx*fpshift0
c	else
c	  zR0momp=0.0
c	endif
c
c  	zrmomp(1)=rmin/sqrt(fpellip)
c	zymomp(1)=rmin*sqrt(fpellip)
 
	if(i1 .lt. nshell) then
	zdx=zx2-zx1
	do 17 im=0,ngmom
	zrmomp(im)=(prmom(i2,im) - prmom(i1,im))/zdx
17	zymomp(im)=(pzmom(i2,im) - pzmom(i1,im))/zdx
 
	else	! extrapolate for r>a:
 
	do 19 im=0,ngmom
	zrmomp(im)=0.0
19	zymomp(im)=0.0
	zrmomp(1)=prmom(nshell,1)
	zymomp(1)=pzmom(nshell,1)
 
	endif
 
	endif
 
c the rest of the code is valid for general geometry:
 
	zr=zrmom(0)
	zy=0.0
 
	if(ijac .ne. 1) then
 
c
c if one ever uses more than 1 moment, it may be faster
c to evaluate the cos(i*zth) and sin(i*zth) using recursion
c formulas.
c
	do 20 i=1,ngmom
	zr=zr+zrmom(i)*cos(i*zth)
20	zy=zy+zymom(i)*sin(i*zth)
 
	else
 
	zjac(1,1)=zRmomp(0)
	zjac(1,2)=0.0
	zjac(2,1)=0.0
	zjac(2,2)=0.0
 
	do 30 i=1,ngmom
	zcos=cos(i*zth)
	zsin=sin(i*zth)
	zr=zr+zrmom(i)*zcos
	zy=zy+zymom(i)*zsin
	zjac(1,1)=zjac(1,1)+zRmomp(i)*zcos
	zjac(1,2)=zjac(1,2)-zRmom(i)*i*zsin
	zjac(2,1)=zjac(2,1)+zymomp(i)*zsin
30	zjac(2,2)=zjac(2,2)+zymom(i)*i*zcos
	endif
 
	return
	end
 
	subroutine walltime(itime)
c
c this calls a vax routine to find the current time of day!
c I can't call the TIME routine directly from INPUTNML because
c it conflicts with a variable name in FPPCOM.CMN called TIME also!
 
	character itime*(*)
	call time(itime)  !@
c@	itime=' '
	return
	end
 
	subroutine fdump(f1rad,fnbsours)
c dump out f(E,angle,r) for Greg Rewoldt's linear instability code
 
	include 'FPPCOM'
	include 'F1COM'
	character ifilnam*20
 
c name the *.fdm file:
	ifilnam=runname(1:lrunname)//'.fdm'
	ilun=37
	open(unit=ilun,file=ifilnam,status='new')
 
c # of radii:
	write(ilun,*) nshell
c list of radii r(i) in (cm):
	write(ilun,*) (rminpro(i),i=1,nshell)
c list of nbeam(r) in (/cm**3):
	write(ilun,*) (rbden1(i),i=1,nshell)
c list of Tbeam(r) in (eV):
	write(ilun,*) (rbtemp1(i),i=1,nshell)
 
c # of energies:
	ien=maxen+1
	write(ilun,*) ien
c array of energies (eV):
	write(ilun,*) (enerev(i),i=0,maxen)
 
c # of pitch angles:
	ian=maxxsi
	write(ilun,*) ian
c array of angles:
	write(ilun,*) (xsi(i),i=1,maxxsi)
 
c array of f(E,xsi,r):
	write(ilun,*)
	1   ((((f1rad(i,j,k)),i=0,maxen),j=1,maxxsi),k=1,nshell)
 
	close(unit=ilun)
 
	return
	end
 
 
 
	subroutine fpgeopl
c
c plot FPP's geometry
c
	include 'FPPCOM'
	include 'FPONCOM'
	include 'SPRINC'
 
c local arrays for plotting:
	parameter (imax=300)
	real zr(imax)   ! R(theta) for VV and flux surfaces
	real zz(imax)   ! Z(theta) for VV and flux surfaces
 
	real zjac(2,2)	! MOMRY argument.
 
	if(nchi+1 .gt. imax .or. maxxsi .gt. imax)
	1   call aborter(lsysout,'imax not large enough in FPGEOPL')
 
	zpi=2.*abs(acos(0.))
 
c first fill zr and zz with vacuum vessel shape:
 
	do 21 i=1,nchi+1
	zth=(i-1)*2*zpi/(1.0*nchi)
	zr(i)=0.0
	zz(i)=0.0
	do 21 im=0,mxvvmoms
	zr(i)=zr(i)+vvrmoms(im)*cos(im*zth)
21	zz(i)=zz(i)+vvzmoms(im)*sin(im*zth)
 
c
	call initt(960)
	call binitt
	call fpgrlab
 
	call npts(nchi+1)
 
c calculate the maximum and minimums for the plot scale:
	zrmin=1.e38
	zrmax=-1.e38
	call mnmx(zr,zrmin,zrmax)
	zzmin=1.e38
	zzmax=-1.e38
	call mnmx(zz,zzmin,zzmax)
 
	zRav=(zrmax+zrmin)/2
	zrhw=(zrmax-zrmin)/2
	zzhh=(zzmax-zzmin)/2
	zsiz=1.1*max(zrhw,zzhh)
 
	call xneat(0)
	call yneat(0)
	call slimx(150,725)
	call slimy(150,727)
	call dlimx(zRav-zsiz,zRav+zsiz)
	call dlimy(-zsiz,zsiz)
	call xfrm(2)
	call yfrm(2)
 
c first draw the Vacuum Vessel:
	call line(0)
	call check(zr,zz)
	call dsplay(zr,zz)
 
c then draw flux surfaces:
 
	call npts(maxxsi)
	data iline /0/
	call line(iline)
 
	do 50 ish=1,nshell
	zxi=xradzb(ish)
	do 40 j=1,maxxsi+1
	zth=(j-1)*2*zpi/(maxxsi-1.0)
40	call momry(zxi,zth,0,zr(j),zz(j),zjac)
 
50	call cplot(zr,zz)
 
	call frame !@
c@	call dframe
	call agtchs('Rmajor (cm)')
	call agtcvs('Z (cm)')
	call agtcgt(0,'Vacuum Vessel and Flux Surfaces')
	call upause(0)
	call finitt(0,0)
 
	return
	end
 
	SUBROUTINE GAUSSE(A,IROW,N,M,X,EPS,IW)
C	11/7/76
C	THIS PROGRAM WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C	PLASMA PHYSICS LAB.  THIS ROUTINE WILL SOLVE A SYSTEM OF LINEAR
C	EQUATIONS BY USING GAUSS ELIMATION AND BACKWORD SUBSITUTION.
C	PARAMETER LIST:
C	A - THE N BY M AUGMENTED MATRIX WITH THE RIGHT HAND VECTOR
C		IN COLUMN M.
C	IROW - THE ROW DIMENSION OF A AS DEFINED BY THE CALLING PROGRAM.
C	N - THE # EQUATIONS.
C	M - N+1.
C	X - SOLUTION VECTOR WITH N ELEMENTS.
C	EPS - EVERY PIVOT ELEMENT MUST BE GREATER THAN EPS.  IF
C		A PIVOT ELEMENT IS SMALLER THE EQUATIONS ARE
C		DECLARED SINGULAR.
C	IW - THE LOGICAL UNIT USED FOR PRINTING.
C	NOTE THAT THE MATRIX A IS DESTROYED.
	DIMENSION A(IROW,M),X(N)
	NN=N-1
C	I FIRST REDUCE MATRIX A.
	DO 320 J=1,NN
C	MAXROW WILL GIVE THE ROW IN WHICH THE MAXIMUM ELEMENT WAS FOUND.
	MAXROW=J
	AMAX=0.E00
C	I NOW SEARCH FOR THE MAXIMUM ELEMENT IN ROW J AND GREATER.
	DO 270 I=J,N
	ABSA=ABS(A(I,J))
	IF(ABSA-AMAX) 270,270,260
260	AMAX=ABSA
	MAXROW=I
270	CONTINUE
C	NOW THE MAXIMUM ELEMENT IN COL. J HAS THE VALUE OF AMAX AND
C	IS IN ROW=MAXROW.
C	I NOW CHECK TO SEE IF EQUATIONS ARE SINGULAR.
	IF(AMAX.GT.EPS) GO TO 275
	WRITE(IW,50) MAXROW,J
50	FORMAT(1X'A PIVOT ELEMENT WAS FOUND TO BE LESS THAN EPS',
     >' THE ELEMENT IS (',I3,','I3,')')
	STOP
C	IF MAXROW DOES NOT EQUAL J SWITCH ROWS.
275	IF(MAXROW.EQ.J) GO TO 290
	DO 280 I=J,M
	STORE=A(J,I)
	A(J,I)=A(MAXROW,I)
280	A(MAXROW,I)=STORE
C	ROWS ARE NOW SWITCHED.
C	I NOW NORMALIZE ROW J SO THAT A(J,J)=1.
290	ANORM=A(J,J)
	DO 295 I=J,M
295	A(J,I)=A(J,I)/ANORM
C	I NOW SUBTRACT ROW J FROM ROWS GREATER THEN J SUCH THAT
C	THERE ARE ZEROS IN COLUMN J.
	JJ=J
300	JJ=JJ+1
	ASUB=A(JJ,J)
	DO 310 I=J,M
310	A(JJ,I)=A(JJ,I)-A(J,I)*ASUB
	IF(JJ.LT.N) GO TO 300
320	CONTINUE
C	I NOW PREFORM BACKWARD SUBSITION.
	X(N)=A(N,M)/A(N,N)
	DO 340 K=1,NN
	SUM=0.E00
	L1=N-K
	DO 330 I=1,K
330	SUM=SUM+A(L1,L1+I)*X(L1+I)
340	X(L1)=A(L1,M)-SUM
	RETURN
	END
