	subroutine FPtoTR
c
c converts FPP output to TRANSP formats
c
c places that can be improved are marked with a "?".
c
! this is based in part on the original TRANSP routine OUTPTB.FNB.
! Some of original lines from OUTPTB are preserved here, with the
! string "coutptb:" at the beginning of the line.
!
! Note that all of the quantities calculated by FPP and given to
! TRANSP (such as Pbe, Pbi, Jbeam, Nbeam) are calculated at the end
! of the beam time step.  What TRANSP may actually want is the average
! of Pbe during the whole beam time step (although it apparently wants
! Nbeam at the end of the beam time step, and then interpolates to find
! Nbeam at any time during the beam time step).  This introduces errors
! of order dt.
!
! I have set lots of the output desired by TRANSP (which would be
! calculated by the Monte Carlo package) to zero because FPP doesn't
! calculate them at the moment.  (NBZERO zero's most of these quantities
! already.  Some of these quantities are calculated by TRANSP's
! subroutine DEP, which then are scaled by ZDTI here.)
! Some of these things are complicated, and will require detailed
! additional coding (such as beam-beam CX, CX recapture, etc).
! Some of these things can be easily added (such as more details on the
! beam particle balance).  Look at the below coding for everything that is
! blindly set to zero to find out what needs improving.  I have tried to
! provide a fairly complete set of output on the beam power balance,
! although beam thermalization is treated somewhat differently in FPP and
! TRANSP.
!
! most of the output from the original Monte Carlo code (subroutine
! NBMAIN) had to be renormalized.  This renormalization was done mostly by
! OUTPTB, although some output was defined by DEPNRM also.
!
c--------------------------------------------------------------------------
 
! Most outputs have the volume normalization removed, because with the
! generalized geometry beam code, the geometry (and hence the volume of a
! zone) can change from beam timestep to beam timestep.  Therefore,
! outputs have units such as #, eV, Watts, #/sec, and **NOT** #/cm3,
! j/cm3, watts/cm3, #/cm3/sec.
 
	INCLUDE 'TRCOM'
	include 'FPPCOM'
 
C  WORK ARRAYS USED TO DEFINE THE POLOIDAL DISTRIBUTION FCNS
C  (used in the 2-D zone section:
CX*	REAL ZWX(MINR2)
CX*	REAL ZWTH(MJMC)
CX*	REAL ZVOL(MJMC,MINR2)
CX*	REAL ZRAV(MJMC,MINR2)
C CX* -- LOCALS NOT USED -- DMC 8/90
C
C  CONSTANTS USED IN NORMALIZATION
C
C
C  TIMESTEP NORMALIZATION
C
        ZDTI=1.0/DELT
C
C  TURN PTCL*EV INTO JOULES
C
        ZBPF=XE*ZDTI
C
C  NORMALIZE ANG. MOMENTUM BALANCE TERMS
C
	ZDTI7=1.0E-7*ZDTI
 
c find maximum Total beam power to the present time:
	zbptot=0.0
	do 30 i=1,numnbi
30	zbptot=zbptot+pbeams(i)
c convert to Watts:
	zbptot=zbptot*1.e6
	if(zbptot .gt. fpHmax)fpHmax=zbptot
 
c total beam density:
	bntots(lsbeam)=fpvint(rbden1)
 
C	ENOLDs = ENERGY CONTENT IN BEAM IONS
C	@ END OF CURRENT TIMESTEP, IN EV
c convert to eV (xe= 1.60219e-19 Joules/eV)
C
	enolds(lsbeam)=fpvint(rbwlab)/xe
 
C
C	BPHOLD=Angular momentum content in beam ions
C	@ END OF CURRENT TIMESTEP
C
	BPHOLDs(lsbeam)=fpvint(rbtmom1)
 
	if(nfpdepsw .ne. 2) then
c if we are using FPP's DEPO, then calculate some deposition
c quantities:
c
	BPLs(lsbeam)=0.0    ! beam power to limiter (prompt loss) (W)
	BPSHINs(lsbeam)=0.0 ! shine thru power (W)
	do 35 i=1,numnbi
	bpshins(lsbeam)=bpshins(lsbeam)+BPshthru(i)
35	bpls(lsbeam)=bpls(lsbeam)+BPprompt(i)
 
c ? ignore global particle and momentum balance for now if using DEPO:
	BXFLWs(lsbeam)=0.0  ! # of beam ions/sec to limiter ?ignore
	BPHORBS(lsbeam)=0.0 ! momentum orbit loss from DEPO ?ignore
	BPHDEPS(lsbeam)=1.0e-7*fpvint(rmsbeam)	!total injected momentum
	BPHJXBS(lsbeam)=0.0 ! JxB Torque ?ignore
	BPJXBS(lsbeam)=0.0  ! JxB Work ?ignore
 
	else
c normalize the quantities calculated by TRANSP's DEP and DEPRNM
c so they have the same units as other FPP calculated quantities:
	bpshins(lsbeam)=bpshins(lsbeam)*zbpf
	bpls(lsbeam)=bpls(lsbeam)*zbpf
	BXFLWs(lsbeam)=bxflws(lsbeam)*zdti
	BPHORBS(lsbeam)=BPHORBS(lsbeam)*zdti7
	BPHDEPS(lsbeam)=BPHDEPS(lsbeam)*zdti7
	BPHJXBS(lsbeam)=BPHJXBS(lsbeam)*zdti7
	BPJXBS(lsbeam)=BPJXBS(lsbeam)*zbpf
 
	endif
 
c Fokker-Planck contribution to losses which will be added in OUTPTB:
c ? ignore some of these for now:
	BPL=FPVINT(rpbl)
	BPSHIN=0.0
	BXFLW=0.0
 
c fishbone power and particle losses (ignore for now):
	BPFSH=0.0
	BXFSH=0.0
 
c FPP does not allow particle orbits to go "outside" the plasma, and
c therefore does not have CX events occuring out there.  FPP also ignores
c recapture of CX neutrals.  Once a fast ion CX's, FPP assumes it is lost.
c We have therefore added a very crude model for CX recapture:  we
c assume that a fraction FPCXRE of the fast neutrals are recaptured,
c and reduce the neutral density passed to FPP by (1.-FPCXRE), which
c assumes that they are recaptured immediately on the same flux surface.
 
C  EXTERNAL CX N/SEC
	SNBCXO=0.0
 
C Beam power net losses (after recapture) inside/outside the plasma:
	BPCXIN=FPVINT(rpbc)
	BPCXOT=0.0
 
c CX source subject to recapture (inside/outside the plasma):
	BPCXI0=bpcxin/(1.-fpcxre)
	BPCXX0=0.0
 
c CX recapture (inside/outside the plasma):
	BPCXRI=bpcxi0*fpcxre
	BPCXRX=0.0
 
	bpcxer=0.0
 
c*****************************************************************
c Global Rotation Balance
c*****************************************************************
 
c FPP's rotation model makes a number of assumptions.  It is
c basically a zero-banana width model in the frame of reference
c moving with the plasma rotation speed.  The momentum balance
c equation:
c
c dPtor/dt = Tbeam -Tcx -Te -Ti -Ttherm -TJxb +Toh -Tdomega/dt
c
c is used.  dPtor/dt is calculated, along with the source due
c to the beam, and the sink due to collisions with electrons,
c and the sink due to CX.  It is then assumed that the rest
c of the torque goes to the ions, and the JxB and thermalization
c torques are lumped together with the ion Torque.  The torque
c from the OH electric field is ignored for now.
c
c Also, there should be a term in the Fokker-Planck equation
c proportional to d(Vrot)/dt, since it is being done in the
c plasma frame.  But this term is being ignored for now.
c
 
c momentum source rate from orbitting (CX Recapture?), ?ignore:
	bphdep=0.0
 
c CX losses of fast ion momentum:
c Note: bphcxl=bphrca+bph0wa
	bph0wa=1.0e-7*fpvint(fptorqcx)	! net CX losses
	bphcxl=bph0wa/(1.-fpcxre)    !subject to recapture
	bphrca=bphcxl*fpcxre  ! CX recapture of Momentum
 
	BPHJXB=0.0   !orbitting JxB torque?  ?ignore?
 
	BPHORB=0.0  !beam momentum to limiter (orbit loss) ignore?
	BPHFSH=0.0  !beam momentum lost by fishbones ?ignore
 
	bpherr=0.0
	bperr=0.0
	BPJXB=0.0   ! orbitting JxB work?  ignore?
 
	BPTHSF=0.0 !Beam power thermalization rotation source-friction
 
C *******************************************************************
C
C  NORMALIZE PROFILE QUANTITIES, AND ACCUMULATE SOME GLOBAL ONES
C (only need to accumulate SBCXX, since most other sums are
c calculated by OUTPTB):
c
	SBCXX=0.0   !net internal fast ion CX rate (particles/sec)
C
c***********************************************************************
C  Start of the 1D radial profile renormaliztion loop (do 3000 j ...)
c***********************************************************************
 
	DO 3000 J=LCENTR,LEDGE	!j = TRANSP radial index
	    jshell=j-lcentr+1	!jshell = FPP radial index
 
c PBI and PBE are beam to ion and electron powers (Watts):
	  PBI(J)=rpbi(jshell)*dvol(j,2)
	  PBE(J)=rpbe(jshell)*dvol(j,2)
C compression power to beams (ignored for now):
	  PBCPRS(J)=0.0
 
C OH-->Beam power ignored for now??:
	  PBOH(J)=0.0
C
C  FUSION PRODUCTS OUTPUT
c
c The reactions 1 through 6 are as given in FURATE.FOR
c and SIGMA.FOR.  The reactions 7-12 are the same but with the
c beam and target species switched.  The factors of DELT will be
c normalized out in OUTPTB2.FNB.
 
C  DMC 5 Feb 1992 -- changed logic expressions for safer portability
 
	iab = abeams(lsbeam) + 0.5
	izb = xzbeams(lsbeam) + 0.5
 
c For D beams:
CDMC	if(abeams(lsbeam) .eq. 2 .and. xzbeams(lsbeam) .eq. 1)then
	if(iab .eq. 2 .and. izb .eq. 1)then
	  btntns(j,1)=fpfuspro(jshell,1)*dvolfp(jshell)*delt
	  btntns(j,2)=fpfuspro(jshell,2)*dvolfp(jshell)*delt
	  btntns(j,4)=fpfuspro(jshell,4)*dvolfp(jshell)*delt
	endif
 
c For T beams:
CDMC	if(abeams(lsbeam) .eq. 3 .and. xzbeams(lsbeam) .eq. 1)then
	if(iab .eq. 3 .and. izb .eq. 1)then
	  btntns(j,7)=fpfuspro(jshell,1)*dvolfp(jshell)*delt
	  btntns(j,5)=fpfuspro(jshell,5)*dvolfp(jshell)*delt
	  btntns(j,6)=fpfuspro(jshell,6)*dvolfp(jshell)*delt
	endif
 
c For He3 beams:
CDMC	if(abeams(lsbeam) .eq. 3 .and. xzbeams(lsbeam) .eq. 2)then
	if(iab .eq. 3 .and. izb .eq. 2)then
	  btntns(j,8)=fpfuspro(jshell,2)*dvolfp(jshell)*delt
	  btntns(j,12)=fpfuspro(jshell,6)*dvolfp(jshell)*delt
	endif
 
c Although I am now calculating the thermal ion source rate
c due to thermalization of the beam, this only works with df/dE=0
c boundary conditions at E=0.  For RF heating, or combined RF+NBI,
c we would like to use conservative B.C.'s at the origin, which then
c requires us to find a new way to "de-thermalize" and "thermalize".
 
C beam power thermalizing (FPP's model will approach zero as the energy
c grid gets finer!):
	  PBTH(J)=rBpth(jshell)*dvol(j,2)
 
c WARNING: CRUDE MODELS BEING USED! FPP IS CAPABLE OF MUCH BETTER!!!
c??????????????????????????????
c There are a number of crude approximations between here and 2990.  In
c particular:
c
c We assume that the fraction of beam ion deposition
c due to ionization is ZDEPBYI, with the rest of the deposition due
c to CX with thermal ions.  TRANSP then calls FRANTIC to figure out
c how far these halo neutrals make it before they ionize.
c I have set ZDEPBYI=0.5 arbitrarily.  It would not take too much
c work to upgrade DEPO to calculate deposition due to ionization and
c CX separately.  (If TRANSP's DEP routine is being used instead of
c FPP's DEPO routine, then the halo neutrals are being properly
c calculated).
 
c Realize that there are 3 classes of neutral particles:
c Beam neutrals -- "1st generation neutrals" from Neutral Beam Injectors
c Fast neutrals -- from fast ions which CX while they are slowing down
c Thermal neutrals -- CX from Maxwellian thermal ions
 
c local quantities used below:
c total hydrogenic ion density:
	zrho=0.0
	do 2910 ig=1,ng
2910	zrho=zrho+rhob(j,ig,2)
c total beam deposition rate:
	zbdrate=rnsbeam(jshell)*dVolfp(jshell)
 
	zdepbyi=0.50	!set fraction of deposition due to ionization
c electron source due to ionization of the beam
	if(nfpdepsw .ne. 2) then
	    SORBES(J,LSBEAM)=zbdrate*zdepbyi  !calculated by FPP's DEPO
	else
	    SORBES(J,LSBEAM)=sorbes(j,lsbeam)*zdti !calc by TRANSP's DEP
	endif
 
C NUTRAV OUTPUTS
c dep handles beam deposition
c nutrav handles CX losses and reionization of fast neutrals
c frantic handles neutral transport of thermal neutrals
 
c CX fast neutral source (crude approximation for particle source rate--
c note that I am ignoring the difference in cross sections for different
c species, weighting the reaction rates only by the relative density
c of each species):
	sbcx0(j)=0.0
	if(rbtemp1(jshell) .gt. 0.0)
	1   SBCX0(J)=rpbc(jshell)*dvol(j,2)/
	1   (xe*1000.*1.5*rbtemp1(jshell))
	sbcxx=sbcxx+sbcx0(j)   !net internal fast ion CX rate (#/sec)
c correction for recaptured fraction:
	sbcx0(j)=sbcx0(j)/(1.-fpcxre)
c additional electron source due to ionization of recaptured
c fast neutrals:
	sorbe(j)=sbcx0(j)*fpcxre*zdepbyi
c fast neutral recapture due to
c sbcxri   thermal ion impact ionization
c sbcxrb   fast ion CX or impact ionization
c sbcxrx   thermal ion CX, by species
	  SBCXRI(J)=0.0
	  SBCXRB(J)=0.0
	  DO 2990 IG=1,NG
	    SBCXRX(IG,J)=sbcx0(j)*fpcxre*(1.-zdepbyi)*rhob(j,ig,2)/zrho
 
c hydrogenic thermal ion source by specie (=thermalization-CX, so its
c negative initially! (thermalization source added below).  First
c calculate sink due to CX with beam neutrals:
	if(nfpdepsw .ne. 2) then
	    SORBHS(J,IG,LSBEAM)=
	1   -zbdrate*rhob(j,ig,2)/zrho*(1.-zdepbyi)
	else
	    SORBHS(J,IG,LSBEAM)=
	1   sorbhs(j,ig,lsbeam)*zdti !found by TRANSP's DEP
	endif
c add on term due to fast neutrals:
	    sorbh(j,ig)=-sbcxrx(ig,j)
 
c thermal neutral source, by specie, first the halo neutrals due to
c CX with the neutral beam injector neutrals:
	if(nfpdepsw .ne. 2) then
	    SORBN0S(J,IG,LSBEAM)=
	1   +zbdrate*rhob(j,ig,2)/zrho*(1.-zdepbyi)
	else
	    SORBN0S(J,IG,LSBEAM)=
	1   sorbn0s(j,ig,lsbeam)*zdti !TRANSP's DEP
	endif
c plus the term due to CX with fast neutrals which get recaptured:
	    sorbn0(j,ig)=sbcxrx(ig,j)
 
c thermal neutral sink rates:
c approximate n0 nb <sigma v>_cx dVol:
	    SINBN0(J,IG)=sbcx0(j)*rhob(j,ig,2)/zrho
	    zErel=1.5*rbtemp1(jshell)/Ab*1000.+1.5*Tnupro(jshell)/Ap
c approximate n0 nb <sigma v>_ii:
	    SINB0I(J,IG)=sinbn0(j,ig)*psgvii(zErel)/psgvcx(zErel)
 2990	  CONTINUE
 
c add in thermalization source:
	sorbh(j,nbs)=sorbh(j,nbs)+dvol(j,2)*rpsour(jshell)
	sorbth(j)=dvol(j,2)*rpsour(jshell)
 
c FPP assumes that the ions which thermalize are distributed
c uniformly over the flux surface, so that the moment of inertia
c for rotation is not changed.  To force the final PBTHR2 to be
c zero after OUTPTB's calculation:
	pbthr2(j)=gr2(j,2)*sorbth(j)
 
C------------------
C  TORQUES
C
C Standard units of torque is (d/dt) m r v = gram*cm**2/sec**2 in CGS,
c and torque density is gram*cm**2/sec**2/cm**3.
c TRANSP uses Newton-meter/cm**3 for torque density, where
c a Newton is kilogram*meter/sec**2.  So to convert from CGS
c to TRANSP units, multiply by 10**-7:
c
 
	TQOHB(J)=0.0*1.0e-7 !OH to beam Torque (ignored)
	TQBXF(J)=0.0*1.0e-7 !Fraction of TQBI going to impurities (ignored)
	TQBTH(J)=0.0*1.0E-7 !Torque to ions from beam thermalizations (ignore)
 
	TQBI(J)=fptorqbi(jshell)*1.0E-7*dVolfp(jshell)
	TQBE(j)=fptorqbe(jshell)*1.0E-7*dVolfp(jshell)
 
	if(nfpdepsw .ne. 2)then
! what does FPP's DEPO do with first orbit JxB torque?
! ignore it for now????:
	    tqjxbs(j,lsbeam)=0.0
! torque on thermal ions due to CX of thermal ions with
! primary beam neutrals:
	    tqbcxs(j,lsbeam)=
	1	-pphi(j,2)/rhoth(j,2)*zbdrate*(1.-zdepbyi)
	else
	TQJXBS(J,lsbeam)=
	1   tqjxbs(j,lsbeam)*zdti*1.0E-7 !first orbit JxB from DEP
	TQBCXS(J,lsbeam)=
	1   tqbcxs(j,lsbeam)*zdti*1.0E-7 !calculated by TRANSP's DEP
	endif
 
	tqjxb(j)=0.0
! torque on thermal ions due to CX of thermal ions with
! fast neutrals:
	tqbcx(j)=-pphi(j,2)/rhoth(j,2)*sbcx0(j)*fpcxre*(1.-zdepbyi)
 
c set average energy, rotation, and density of fast neutrals to zero:
c (FPP is using the crudest of models for fast neutral recatupre, by
c assuming instantaneous recapture of the fraction FPCXRE of the fast
c neutrals.  So FPP doesnt calculate any of these parameters about
c the fast neutrals):
	be0cx(j,lsbeam)=0.0
	bv0cx(j,lsbeam)=0.0
	bn0cx(j,lsbeam)=0.0
 
C----------------------------------------------------------------------
C  BEAM DRIVEN CURRENT
 
C
c OUTPTB2 will multiply in a shielding factor XJBFAC
c calculated by TRANSP's FOKKER routine.
c XJBFAC is not consistent with the Estar used in FPP:
c
c Unshielded dV*<J.B>:
	xcurbg(j)=xcurbg(j)+
	1   1.e-4*rbjb(jshell)*dVolfp(jshell)/xe/zdti
 
c ignore breakdown of co vs. ctr current for now:
	XCURBC(J,1)=0.0 ! Unshielded DV*<J.B> (DRIVEN BY CO SCES)
 
C
C---------------------------------------------------------------------
C # of beam ions in a zone:
	BDENS(J)=rbden1(jshell)*dvol(j,2)
 
C
C  AVERAGE TOTAL, PARALLEL, AND PERP BEAM ION AVERAGE ENERGIES
c (in eV in the lab frame) and the average beam rotation speed
c (beam betas are calculated from these quantities):
 
	if(rbden1(jshell) .le. 0.0)then
	    zErot=0.0
	else
	    zErot=romega(jshell)*rbtmom1(jshell)+romega(jshell)**2*
	1	rbtmi1(jshell)/2.0
	    zErot=zErot/1.6022e-12/rbden1(jshell)
	endif
 
	EBAVP(J)=1.5*rbtemp1(jshell)*1.e3+zErot
	EBAPL2(J)=0.5*rbtpar1(jshell)*1.e3+zErot
	EBAPP2(j)=ebavp(j)-ebapl2(j)
 
c average beam rotation frequency <Vphi/R> in lab frame:
	omegab(j)=rbtomega(jshell)+romega(jshell)
 
c renormalize for use in OUTPTB:
	EBAVP(J)=ebavp(j)*bdens(j)
	EBAPL2(J)=ebapl2(j)*bdens(j)
	EBAPP2(j)=ebapp2(j)*bdens(j)
	omegab(j)=omegab(j)*bdens(j)
 
C
C	BEAM-BEAM REACTION RATE ARRAYS
C
c nb*sigma*v curves by beam source, energy, fraction:
c SN00X beam-beam charge exchange rate table
c SN00I beam-beam ionization rate table
 
c set all to zero for now:
	  DO 2999 IEN=1,3
	  do 2995 ib=1,nbeam
	  sn00x(ib,ien,j)=0.0
	  sn00i(ib,ien,j)=0.0
2995	continue
 
c presently ignoring beam neutral density.  If we calculated
c it, we could use it to estimate beam-beam charge-exchange.
c (would also have to read in bn00 in TRtoFP somehow):
c ???:
c	bn00(ib,ien,j)=0.0
 
2999	continue
 
c***********************************************************************
C  END OF 1D RADIAL PROFILE RENORM (J) LOOP
3000	CONTINUE
c***********************************************************************
 
c Here we fill TRANSP's beam f array, fbm(nznbme,nznbma,nfbzns,lsbeam).
c This will then be used to calculate CX eflux and beam-beam neutrons.
c FPTOTRBM also calculates some other beam quantities on the 2D
c beam grid.
c
	IF(NLFBM) call fptotrbm
 
	RETURN
	END
