	subroutine FPtoTRbm
c
C MOD DMC 12 OCT 1992 -- ADD USER REQUESTED FEATURE -- output DENSFI,
C  a set of (lab frame) energy constrained fast ion density profiles
C  this matches a feature of the Monte Carlo code.
C
C MOD DMC 25 SEPT 1991 -- The TRANSP fast ion distribution function
C has been restructured to support a species dependent unevenly spaced
C energy grid -- accordingly I have made changes here.  In particular,
C NZNBMEA(species) is used instead of NZNBME, and the actual energy
C spacing EFBMB(IEN+1,species)-EFBMB(IEN,species) is used instead of
C DELEBD -- DELEBD the (old even) energy spacing is defunct.
C
C MOD DMC 19 SEPT 1991 -- The TRANSP distribution function FBM has
C been restructured to not include any zones outside the plasma--
C i.e. the zone index ranges from 1 to NFBZNSI instead of 1 to NFBZNS.
C I have patched the code to make sure that no attempt is made to
C reference FBM zones beyond NFBZNSI...
C
c Converts from FPP's beam distribution function FPFRAD
c to TRANSP's FBM(nznbme,nznbma,nfbzns,lsbeam).
c FBM will be used to calculate beam-beam fusion
c reactions, and charge exchange and bolometer signals.
c
c ??: This routine assumes that B is proportional to 1/R,
c as does most of FPP.  This could be improved.
c
	INCLUDE 'TRCOM'
	include 'FPPCOM'
 
C  WORK ARRAYS USED TO DEFINE THE POLOIDAL DISTRIBUTION FCNS
C  (used in the 2-D zone section:
CX*	REAL ZWX(MINR2)
CX*	REAL ZWTH(MJMC)
CX*	REAL ZVOL(MJMC,MINR2)
CX*	REAL ZRAV(MJMC,MINR2)
C  CX*   -- COMMENT OUT UNUSED LOCAL ARRAY DECLARATIONS -- DMC 8/90
	real zradvol(mtbl)  ! volume of each beam zone radial row
	real zbmin(maxshe)  ! minimum magnetic field on each flux surface
	real zjac(2,2)
	real zni(mig)	! interpolated  thermal ion densities
	character icbeam*1,ictarget(mxions)*1
 
c zero out TRANSP's beam distribution function:
	do 200 k=1,nfbznsi
	do 200 j=1,nznbma
	do 200 i=1,nznbmea(lsbeam)
200	fbm(i,j,k,lsbeam)=0.0
 
c DMC Oct 1992 -- zero out DENSFI
	if(NERNGFI.gt.0) then
	  do ie=1,nerngfi
	    do jtr=lcentr,ledge
	      DENSFI(jtr,ie,lsbeam)=0.0
	    enddo
	  enddo
	endif
 
c calculate the minimum magnetic field on each flux surface:
	do 220 i=1,nshell
	zxi=(i-0.5)/nshell
	call momry(zxi,0.0,0,zr,zy,zjac)
220	zbmin(i)=btor*rmaj/zr
 
c calculate the volume of each beam zone radial row to use
c in later normalizations:
 
C  LOOP OVER BEAM ZONE RADIAL ROWS
	DO 300 IZR=1,NZNBMR
	    zradvol(izr)=0.0
 
C  BEAM ZONE INDICES.  This radial row goes from iz1 to iz2:
	    IZ1=NTHZSM(lcm1+izr-1)+1
	    IZ2=NTHZSM(lcm1+izr)
 
	    DO 320 iz=IZ1,IZ2
	    zradvol(izr)=zradvol(izr)+bmvol(iz)
320	    continue
300	continue
 
c The plasma xi grid is equally space and runs from xi=0 to 1.
c (xi is the relative toroidal flux.)  The spacing is:
	ZDXI=DXI(LCENTR,2)
 
C  LOOP OVER BEAM ZONE RADIAL ROWS
	DO 1500 IZR=1,NZNBMR
 
c beam zone radial row izr corresponds to TRANSP/FPP radial
c zones ir1 to ir2:
	ir1=1+(izr-1)*nxskpb
	ir2=ir1+nxskpb-1
	ir2=min(nshell,ir2)
	if(ir1 .gt. maxshe) goto 1500
 
C  BEAM ZONE INDICES.  This radial row goes from iz1 to iz2:
	    IZ1=NTHZSM(lcm1+izr-1)+1
	    IZ2=NTHZSM(lcm1+izr)
	    ZDTH=XPI/(iz2-iz1+1.0)
 
	    DO 1480 IZ=IZ1,IZ2
		ZTH=(IZ-IZ1+0.5)*zdth
 
	    do 1300 irfpp=ir1,ir2
		irtr=irfpp-1+lcentr
		zxi=zdxi*(irfpp-0.5)
C  GET (R,Y) LOCATION OF CTR OF ZONE
		CALL MOMRY(zxi,zth,0,zr,zy,zjac)
		zblocal=btor*rmaj/zr
c zweight = volume weighted contribution of fpp shell irfpp to
c beam zone izr:
		zweight=dvolfp(irfpp)/zradvol(izr)
c find the plasma rotation velocity at this radius:
		zerot=5.2197e-13*abeam*(zr*romega(irfpp))**2
		zvrot=(2*zerot)**0.5
 
c loop over all beam pitch-angle zones:
		do 1200 jba=1,nznbma
		zxlab=-1.0+(jba-0.5)*2./nznbma
 
c loop over all energies:
		do 1150 ibe=1,nznbmea(lsbeam)
		zelab=EFBM(ibe,lsbeam)
 
c transform from the lab frame (zelab,zxlab) to the rotating
c plasma frame (zep,zxp):
		zeperp=zelab*(1.0-zxlab**2)
		zvlab=(2*zelab)**0.5
		zvparlab=zxlab*zvlab
		zep=zeperp+0.5*(zvparlab-zvrot)**2
		zxp=(zvparlab-zvrot)/(2*zep)**0.5
		zxp=max(-1.0,min(1.0,zxp))
 
c transform from the local pitch angle zxp to the midplane
c pitch angle zxsi0:
		zxsi2=1.0-(1.0-zxp**2)*zbmin(irfpp)/zblocal
		zxsi2=max(0.0,min(1.0,zxsi2))
		zxsi0=sign(zxsi2**0.5,zxp)
 
c find the nearest fpp pitch angle zone:
		jxfpp=lkupr(zxsi0,xsi,maxxsi)
		jxfpp=max(1,min(maxxsi,jxfpp))
		if( (xsi(jxfpp+1)-zxsi0) .lt. (zxsi0-xsi(jxfpp)) )
	1	    jxfpp=jxfpp+1
 
c find the nearest fpp energy zone:
		iefpp=lkupr(zep,enerev,maxen+1)-1
		iefpp=max(0,min(maxen-1,iefpp))
		if( (enerev(iefpp+1)-zep) .lt. (zep-enerev(iefpp)) )
	1	    iefpp=iefpp+1
		zcontrib =
	1	    50.*enerev(iefpp)**0.5*zweight*
	1	    fpfrad(iefpp+1,jxfpp,irfpp,lsbeam)
		fbm(ibe,jba,iz,lsbeam) =
	1	    fbm(ibe,jba,iz,lsbeam) + zcontrib
c
c  mod dmc oct 1992 -- add DENSFI calculation
c
		if(NERNGFI.gt.0) then
		  do ie=1,nerngfi
		    ze1=erngfi(ie)
		    if(ie.lt.nerngfi) then
		      ze2=erngfi(ie+1)
		    else
		      ze2=epsinv
		    endif
		    ze = efbm(ibe,lsbeam)
		    if((ze1.le.ze).and.(ze.lt.ze2)) then
		      zde = efbmb(ibe+1,lsbeam)-efbmb(ibe,lsbeam)
		      densfi(irtr,ie,lsbeam) = densfi(irtr,ie,lsbeam) +
	1		zcontrib * zde * bmvol(iz) / nznbma
		    endif
		  enddo
		endif
 
1150		continue    ! ibe beam energy loop
1200		continue    ! iba beam angle loop
1300	    continue	    ! irfpp fpp radial loop
1480	    continue	    ! iz beam 2D zone loop
1500	continue	    ! izr beam radial row loop
 
c*****************************************************************
c Renormalize fbm to contain exactly the same number of particles
c as in fpfrad:
C  LOOP OVER BEAM ZONE RADIAL ROWS
	DO 1800 IZR=1,NZNBMR
 
c beam zone radial row izr corresponds to TRANSP/FPP radial
c zones ir1 to ir2:
	ir1=1+(izr-1)*nxskpb
	ir2=ir1+nxskpb-1
	ir2=min(nshell,ir2)
	if(ir1 .gt. maxshe) goto 1800
 
C  BEAM ZONE INDICES.  This radial row goes from iz1 to iz2:
	IZ1=NTHZSM(lcm1+izr-1)+1
	IZ2=NTHZSM(lcm1+izr)
 
	ztotal=0.0
	    DO 1690 iz=IZ1,IZ2
	    zden=0.0
	    do 1680 j=1,nznbma
	    do 1680 i=1,nznbmea(lsbeam)
	      zdelebd=efbmb(i+1,lsbeam)-efbmb(i,lsbeam)
1680	    zden=zden+fbm(i,j,iz,lsbeam)*zdelebd
	bdens2(iz,lsbeam)=zden/nznbma*bmvol(iz)
	ztotal=ztotal+bdens2(iz,lsbeam)
1690	    continue
 
	zactual=0.0
	do 1710 irfpp=ir1,ir2
1710	zactual=zactual+dvolfp(irfpp)*rbden1(irfpp)
 
	if(ztotal .eq. 0.0) then
	    zrenorm=1.0
	else
	    zrenorm=zactual/ztotal
	endif
 
	if(zrenorm .ne. 1.0) then
	do 1780 iz=iz1,iz2
c the factor of delt gets taken back out in OUTPTB2.FNB:
	bdens2(iz,lsbeam)=bdens2(iz,lsbeam)*zrenorm*delt
	do 1780 j=1,nznbma
	do 1780 i=1,nznbmea(lsbeam)
1780	fbm(i,j,iz,lsbeam)=fbm(i,j,iz,lsbeam)*zrenorm
	endif
 
C DMC OCT 1992 -- renorm DENSFI also
 
	if(NERNGFI.GT.0) then
	  do ie=1,nerngfi
	    do jtr=lcentr,ledge
	      densfi(jtr,ie,lsbeam) = zrenorm*densfi(jtr,ie,lsbeam)
	    enddo
	  enddo
	endif
 
1800	continue
 
c**************************************************************
c Calculate the beam-target reaction rate on the 2D beam grid:
c
c identify the beam species:
c  modified DMC Feb 1992 for portability
c   let fp to integer conversion round down, to be sure that the
c    conversion of say abeams(lsbeam)=2.0 yields 2, not 1.999999
c     rounded down to 1.  I also added local var. "izbeam" instead
c     of doing potentially hazardous floating pt equality test
c
	iabeam=abeams(lsbeam)+0.5
	izbeam=xzbeams(lsbeam)+0.5
	icbeam=' '
	if(iabeam .eq. 2 .and. izbeam .eq. 1) icbeam='D'
	if(iabeam .eq. 3 .and. izbeam .eq. 1) icbeam='T'
	if(iabeam .eq. 3 .and. izbeam .eq. 2) icbeam='3'
	if(icbeam .eq. ' ')go to 1910
 
c identify the thermal ion species:
	do 1810 iion=1,ng
	ictarget(iion)=' '
	if(naions(iion) .eq. 2 .and. nzions(iion) .eq. 1)
	1   ictarget(iion)='D'
	if(naions(iion) .eq. 3 .and. nzions(iion) .eq. 1)
	1   ictarget(iion)='T'
	if(naions(iion) .eq. 3 .and. nzions(iion) .eq. 2)
	1   ictarget(iion)='3'
1810	continue
 
C  LOOP OVER BEAM ZONE RADIAL ROWS
	DO 1900 IZR=1,NZNBMR
 
c beam zone radial row izr corresponds to TRANSP/FPP radial
c zones ir1 to ir2:
	ir1=1+(izr-1)*nxskpb
	ir2=ir1+nxskpb-1
	ir2=min(nshell,ir2)
	if(ir1 .gt. maxshe) goto 1910
 
	zxi=(izr-0.5)*zdxi*nxskpb
	zTi=fpintrp(zxi,ti(lcentr,2),xi(lcentr,2),nzones)
	zomega=fpintrp(zxi,omegag(lcentr),xi(lcentr,2),nzones)
	do 1820 iion=1,ng
1820	zni(iion)=fpintrp(zxi,rhob(lcentr,iion,2),xi(lcentr,2),nzones)
 
	zromega=(romega(ir1)+romega(ir2))/2.0
 
C  BEAM ZONE INDICES.  This radial row goes from iz1 to iz2:
	IZ1=NTHZSM(lcm1+izr-1)+1
	IZ2=NTHZSM(lcm1+izr)
	zdth=xpi/(iz2-iz1+1.0)
 
	    DO 1890 iz=IZ1,IZ2
	    zth=(iz-iz1+0.5)*zdth
C  GET (R,Y) LOCATION OF CTR OF ZONE
		CALL MOMRY(zxi,zth,0,zr,zy,zjac)
c find the plasma rotation velocity at this position
		zerot=5.2197e-13*abeam*(zr*zromega)**2
		zvrot=(2*zerot)**0.5
 
	    do 1880 jba=1,nznbma
		zxlab=-1.0+(jba-0.5)*2./nznbma
 
	    do 1880 ibe=1,nznbmea(lsbeam)
		zelab=efbm(ibe,lsbeam)
		zdelebd=efbmb(ibe+1,lsbeam)-efbmb(ibe,lsbeam)
 
c transform from the lab frame (zelab,zxlab) to the rotating
c plasma frame zenergy:
		zeperp=zelab*(1.0-zxlab**2)
		zvlab=(2*zelab)**0.5
		zvparlab=zxlab*zvlab
		zenergy=zeperp+0.5*(zvparlab-zvrot)**2
 
c The coding below shares some similarities with FURATE.FOR.
 
c check for reactions with a Tritium beam:
	if(icbeam .eq. 'T') then
	do 1830 iion=1,ng
	if(ictarget(iion) .eq. 'D') then
	btntn2(iz,7)=btntn2(iz,7)+
	1   zni(iion)*sigvbt(zenergy,zTi,'TD')*fbm(ibe,jba,iz,lsbeam)
	2     *zdelebd
	endif
1830	continue
	endif
 
c check for reactions with a Deuterium beam:
	if(icbeam .eq. 'D') then
	do 1840 iion=1,ng
	if(ictarget(iion) .eq. 'D') then
	btntn2(iz,4)=btntn2(iz,4)+
	1   zni(iion)*sigvbt(zenergy,zTi,'DD')*fbm(ibe,jba,iz,lsbeam)
	2     *zdelebd
	endif
	if(ictarget(iion) .eq. 'T') then
	btntn2(iz,1)=btntn2(iz,1)+
	1   zni(iion)*sigvbt(zenergy,zTi,'DT')*fbm(ibe,jba,iz,lsbeam)
	2     *zdelebd
	endif
	if(ictarget(iion) .eq. '3') then
	btntn2(iz,2)=btntn2(iz,2)+
	1   zni(iion)*sigvbt(zenergy,zTi,'D3')*fbm(ibe,jba,iz,lsbeam)
	2     *zdelebd
	endif
1840	continue
	endif
 
c check for reactions with a Helium-3 beam:
	if(icbeam .eq. '3') then
	do 1850 iion=1,ng
	if(ictarget(iion) .eq. 'D') then
	btntn2(iz,8)=btntn2(iz,8)+
	1   zni(iion)*sigvbt(zenergy,zTi,'3D')*fbm(ibe,jba,iz,lsbeam)
	2     *zdelebd
	endif
1850	continue
	endif
 
1880	continue    ! end of ibe and jba loops
 
c normalize btntnt2 to be the volume integrated # of reactions
c during this time step (the factor of delt will be normalized
c out by OUTPTB2.FNB):
	do 1885 ireact=1,mireact
1885	btntn2(iz,ireact)=btntn2(iz,ireact)/nznbma*
	1   bmvol(iz)*delt
 
1890    continue    ! end of iz loop
 
1900	continue    ! end of izr loop
 
1910	continue
 
!***********************************************************************
c The rest of this code came from FPTOTR.FOR, which came from OUTPTB
c originally!:
 
c 18-Jul-1990, I am beginning to calculate some of the quanitities
c below, such as bdens2 and btntn2.  These have been calculated above.
c
c 17-Aug-1988, Doug McCune is upgrading transp to multiple beam species,
c and several of these 2d beam zone arrays (bdens2, eba2pp, etc.) are
c getting a second index for beam species.  For now just comment out
c the rest of this section with ccmb:
ccmb:C
ccmb:C--------------------------------------
ccmb:C  2D PROFILE DATA NORMALIZATION:  DMC MAR 1985
ccmb:C
ccmb:	DO 600 IZ=1,NFBZNS
ccmb:C  AVERAGE PERP AND PLL ENERGIES
ccmb:	  IF(BDENS2(IZ).GT.0.0) THEN
ccmb:	    EBA2PP(IZ)=EBA2PP(IZ)/BDENS2(IZ)
ccmb:	    EBA2PL(IZ)=EBA2PL(IZ)/BDENS2(IZ)
ccmb:	  ELSE
ccmb:	    EBA2PP(IZ)=0.0
ccmb:	    EBA2PL(IZ)=0.0
ccmb:	  ENDIF
ccmb:C  DENSITY
ccmb:	  BDENS2(IZ)=BDENS2(IZ)*ZDTI
ccmb:C
ccmb:C  BEAM TARGET NEUTRONS
ccmb:	  BTNTN2(IZ)=BTNTN2(IZ)*ZDTI
ccmb:C
ccmb:C  UNSHIELDED CURRENT DENSITY *DV
ccmb:	  XCURB2(IZ)=XCURB2(IZ)*XE*ZDTI
ccmb:	  XCRB2C(IZ)=XCRB2C(IZ)*XE*ZDTI
ccmb:C
ccmb:C  CXNSUM OUTPUT
ccmb:	  SRBBI2(IZ)=SRBBI2(IZ)*ZDTI
ccmb:	  SRBBX2(IZ)=SRBBX2(IZ)*ZDTI
ccmb:C
ccmb:C  SCE FCNS
ccmb:	  DO 610 IG=1,NG
ccmb:	    SRBN02(IZ,IG)=SRBN02(IZ,IG)*ZDTI
ccmb:	    SNBN02(IZ,IG)=SNBN02(IZ,IG)*ZDTI
ccmb:	    SNB0I2(IZ,IG)=SNB0I2(IZ,IG)*ZDTI
ccmb: 610	  CONTINUE
ccmb:C  NTH GENERATION CX NEUTRAL DENSITY, ENERGY
ccmb:	  IF(BN0CX2(IZ).GT.0.0) THEN
ccmb:	    ZV2AV=BE0CX2(IZ)/BN0CX2(IZ)
ccmb:	    ZVZAV=BV0CX2(IZ)/BN0CX2(IZ)
ccmb:	    BE0CX2(IZ)=(ZV2AV-ZVZAV*ZVZAV)*V2TOEV
ccmb:	    BV0CX2(IZ)=ZVZAV
ccmb:	    BN0CX2(IZ)=BN0CX2(IZ)*ZDTI
ccmb:	  ELSE
ccmb:	    BV0CX2(IZ)=0.0
ccmb:	    BE0CX2(IZ)=0.0
ccmb:	  ENDIF
ccmb:C
ccmb:	  BN0TT2(IZ)=BN0CX2(IZ)
ccmb:C
ccmb:C  1ST GEN NEUTRALS (NORMALIZED IN DEPNRM) AND REACTION RATE COEFFS
ccmb:	  DO 620 IEN=1,3
ccmb:	    DO 620 IB=1,NBEAM
ccmb:	      DO 615 INO=1,2
ccmb:	        BN0TT2(IZ)=BN0TT2(IZ)+BN002(IB,IEN,INO,IZ)
ccmb: 615	      CONTINUE
ccmb:	      SN00I2(IB,IEN,IZ)=SN00I2(IB,IEN,IZ)*ZDTI
ccmb:	      SN00X2(IB,IEN,IZ)=SN00X2(IB,IEN,IZ)*ZDTI
ccmb: 620	  CONTINUE
ccmb: 600	CONTINUE
 
	RETURN
	END
