	subroutine inputnml
c
c reads input data from lun 41, the fpp namelist file, and
c also from any default namelists using lun 43.
c writes a complete copy of the final namelist values used to
c the file on lun 42
 
c Main FPPRF common blocks:
	include 'FPPCOM'
c AZMAIN switch for Helium plasma cross-sections:
	include 'HE4COM'
c common block for standalone FPP only (and not the TRANSP version):
	include 'FPONCOM'
c Input namelist (described in fpbasic.dft):
	include 'FPPIN'
	include 'SPRINC'    !SPRUCE (ICRF wave code) parameters
 
	character ifname*78 !input file name found by INQUIRE
	character idate*9
	character itime*8
 
	integer iargc,icount
	external iargc, getarg
 
c First read in a set of default parameters as specified in a new
c namelist:
        parameter (imaxfil=100)
        character*60 deffiles(imaxfil)
        namelist /fpdflt/ deffiles
        data deffiles /imaxfil*' '/
 
c determine the name of the input file for this FPP run:
	icount=iargc()
	if(icount .ge. 1) then
	   call getarg(1,runname)
	else
	   write(6,*) 'What is the name of the *.in input file?'
	   write(6,*) '(without the .in suffix):'
	   read(5,*) runname
	endif
	lrunname=index(runname,' ')-1
	write(lsysout,*) 'Running fpp for ', runname(1:lrunname)
 
c FPPBLK2 initializes some common block (and namelist) variables:
	call fppblk2
 
c First type out some informational messages about the value of various
c fortran PARAMETERS!
c--type out size of grids:
	idum1=maxen
	idum2=maxxsi
	idum3=maxtim
	idum4=maxshe
	write(lsysout,*)
	1  '# of energy grid points      MAXEN =',idum1,' +1'
	write(lsysout,*)'# of pitch angle grid points MAXXSI=',idum2
	write(lsysout,*)'Max # of time points         MAXTIM=',idum3
	write(lsysout,*)'Max # of radial shells       MAXSHE=',idum4
 
	idum1=npsi
	idum2=nchi
	write(lsysout,*)'SPRUCE ICRF wave code being run with'
	write(lsysout,*)
	1   '     ',idum1,' radial and ',idum2,' poloidal grid points'
 
c Read in list of default files from the first namelist:
        open(unit=41,file=runname(1:lrunname)//'.in',status='old',
     >     readonly,shared)
        read(41,fpdflt,end=10,err=10)
10      close(unit=41)
 
c the file on unit 42 will contain a complete copy of the namelist
c variables after reading in all of the default files and the final
c namelist file.  Also save the list of default files there:
        open(unit=42,file=runname(1:lrunname)//'.in2',
     >       status = 'unknown', recl=75)
        write(42,fpdflt)
 
c read in the default files:
        do 11 i=1,imaxfil
        if(deffiles(i) .ne. ' ')then
            open(unit=43,status='old',file=deffiles(i),shared,readonly)
            read(43,fppin)
            close(unit=43)
        endif
11      continue
 
C     READ IN INPUT VALUES
        open(unit=41,file=runname(1:lrunname)//'.in',status = 'old',
     1        shared,readonly)
        read(41,fppin,err=97,end=97)
	ibomb=0
	goto 98
 97	ibomb=1
c get input file name for labelling plots:
 98	inquire(unit=41,name=ifname)
        close(unit=41)
 
c save final image of namelist variables:
        write(42,fppin)
        close(unit=42)
 
	if(ibomb .eq. 1) then
	   write(6,*) 'Error while reading input namelist file'
	   write(6,*) ' Try inspecting the *.in2 file to see'
	   write(6,*) ' how much of the *.in file was read.'
	   call aborter(6,'error in subroutine inputnml')
	   stop
	endif
 
c***************************************************
c						   *
c	end of reads from for041.dat		   *
c						   *
c***************************************************
 
c define a label for plots:
	call c9date(idate)    !vax routine which gives today's date
	call walltime(itime) ! indirect call to Vax routine which
! gives current time.  Can't call the VAX routine TIME directly
! from this subroutine because its name conflicts with a variable
! in FPPCOM which is also called TIME!
 
	if (lshoot .eq. 1 .or. lshoot .eq. -1) then
	runlabel='FPP/SPRUCE '//idate//' '//itime//' '//ifname
	else if (lshoot .eq. 2) then
	runlabel='FPP/SHOOT '//idate//' '//itime//' '//ifname
	else
	runlabel='FPP       '//idate//' '//itime//' '//ifname
	endif
c
c do some simple transformations and consistency checks
 
	dtFPP=dtFPP/2.0 ! two actual time steps between MOMS calls
	if(llwbnd .eq. -1) then
	   llwbnd=1
	   lfptherm=1
	endif
 
! check the # of NBI's:
	if(numnbi .gt. maxnbi)then
	numnbi=maxnbi
	write(lsysout,*) ' warning, the max allowed # of NBI''s is ',
     1	numnbi
	endif
 
	if(npltf .ne. 0 .and. (MfPrecX .ne. 40 .or.
	1	MfPrecY .ne. 25 .or. MfPrecz .ne. 100))then
	write(lsysout,*)
	1   '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
	write(lsysout,*)
	1   ' the dimensions of the fprec array in FPP''s common block'
	write(lsysout,*)
	1   ' were reduced to save space.  They must be enlarged'
	write(lsysout,*)
	1   ' and the program must be recompiled before you can '
	write(lsysout,*)
	1   ' calculate f(v-precession) for fishbone stability'
	write(lsysout,*) ' caculations.'
	stop
	endif
 
	if(nline .gt. mxslin)then
	nline=mxslin
	write(lsysout,*)
     1	'Warning: too many detector sightlines. Only the first'
	write(lsysout,*) nline,' sightlines will be used.'
	endif
 
	IF(LBeam.EQ.0) RETURN
 
c read 3-D neutral information:
c--if LBeam=0 then only unit 41 and 43, the fpp data files, are read.
c--if LBeam=1 in the FPP data file then we also read some
c	stuf on the Neutral Beam contribution to the neutral density
c	from units 21,22,23,28,29,31,and 24.
 
c	call fp3dnu
	write(lsysout,*) 'Warning:  the 3D neutral option is broken.'
	write(lsysout,*) ' It needs to be upgraded to the new multiple'
	write(lsysout,*) ' ion model instead of the old single impurity'
	write(lsysout,*) ' model.'
 
	stop
	END
