      subroutine mrsolv(wall)
      include 'SPRINC'
      real scale(nchi*npsi)
      complex hvec(nchi*npsi),rvec(nchi*npsi),pvec(nchi*npsi),
     %        cwrk1(nchi*npsi),cwrk2(nchi*npsi),wall(nchi),
     %        alpha,csum
      namelist /mrinfo/ iter,dhtot,ndhvec,rsdl,nrsdl,alpha,dhcjg
c
c     MRSOLV can utilize spectral wave operators to improve the
c     solution of the spectral-finite-difference wave equation
c     by a minimal residual method.  In practice the iteration
c     is very slow to converge.  Empirically, though, the first
c     couple iterations might provide useful corrections to the
c     solution vector, at little extra cost.
c
c     Set niter = # of iterations, or niter = 0 for none.
      data niter /5/
c
c     "oflow" helps prevent overflow errors, it has no effect
c     whatsoever on the results.
      data oflow /1.e-10/
c
      ntot=nchi*npsi
      nbodyl=ntot-nchi
      nwallf=ntot-nchi+1
c
c     Inverse of finite-difference wave equation
      call fdsolv(scale)
c
c     Source vector
      do 20 i=1,nbodyl
   20 cwrk1(i)=(0.,0.)
      do 25 i=nwallf,ntot
      ichi=i-nbodyl
   25 cwrk1(i)=wall(ichi)
c
c     finite difference solution and residual
      call precnd(cwrk1,hvec)
      call waveop(hvec,cwrk2)
      do 30 i=1,ntot
   30 rvec(i)=cwrk1(i)-cwrk2(i)
c
c==================== Minimum Residual solver ====================
      do 35 iter=1,niter
      do 34 i=1,ntot
   34 cwrk1(i)=rvec(i)*cmplx(scale(i),0.)
      call wavead(cwrk1,cwrk2)
      call precad(cwrk2,cwrk1)
      do 33 i=1,ntot
   33 pvec(i)=cwrk1(i)*cmplx(1./scale(i),0.)
      call precnd(pvec,cwrk1)
      call waveop(cwrk1,cwrk2)
      psum=0.
      dhsum=0.
      dhmax=0.
      drsum=0.
      csum=(0.,0.)
 
c rather than use a fixed value for oflow, estimate what values
c for zpnorm and zwnorm will prevent overflow errors:
 
	zpnorm=1.0e-20
	zwnorm=1.0e-20
	do 331 i=1,ntot
	zpnorm=max(zpnorm,abs(real(pvec(i))))
	zpnorm=max(zpnorm,abs(aimag(pvec(i))))
	zwnorm=max(zwnorm,abs(real(cwrk2(i))))
	zwnorm=max(zwnorm,abs(aimag(cwrk2(i))))
331	continue
 
	zpnorm=1.0/zpnorm
	zwnorm=1.0/zwnorm
 
      do 31 i=1,ntot
      psum=psum+((zpnorm*real(pvec(i)))**2+
     %           (zpnorm*aimag(pvec(i)))**2)*scale(i)
      drsum=drsum+((zwnorm*real(cwrk2(i)))**2+
     %             (zwnorm*aimag(cwrk2(i)))**2)*scale(i)
 
c this next section is needed only when debugging the iteration method:
c      dhwork=(oflow*real(cwrk1(i)))**2+(oflow*aimag(cwrk1(i)))**2
c      if (dhmax.lt.dhwork) dhmax=dhwork
c      dhsum=dhsum+dhwork
c      csum=csum+conjg(rvec(i))*cmplx(oflow**2*scale(i),0.)*cwrk2(i)
 
31	continue
 
      hsum=0.
      rsum=0.
      rmax=0.
      alpha=cmplx( (psum/drsum) * (zwnorm/zpnorm)**2 ,0.)
      do 32 i=1,ntot
      hvec(i)=hvec(i)+alpha*cwrk1(i)
      rvec(i)=rvec(i)-alpha*cwrk2(i)
 
c this next section is needed only when debugging the iteration method:
c      hsum=hsum+(oflow*real(hvec(i)))**2+(oflow*aimag(hvec(i)))**2
c      rwork=(real(rvec(i))**2+aimag(rvec(i))**2)*scale(i)
c      if (rmax.lt.rwork) rmax=rwork
c      rsum=rsum+rwork
 
32	continue
 
c this next section is needed only when debugging the iteration method
c (i.e., its only purpose is to write some convergence information
c out to the terminal):
 
c      dhtot=sqrt(dhsum/hsum)*psum/drsum
c      ndhvec=ifix(dhsum/dhmax)
c      rsdl=sqrt(rsum)
c      nrsdl=ifix(rsum/rmax)
c      dhcjg=cabs(csum-cmplx(psum,0.))/psum
c     write (6,mrinfo)
   35 continue
c=================================================================
c
c     Calculate fields from solution vector
      call fields(hvec)
c
      return
      end
