C
C THIS subroutine PLOTS THE MOMENTS OF THE BEAM DISTRIBUTION FUNCTION
c
c The diamagnetic loop measures beta_perp = Pperp/B**2.
c The equilibrium field measures (beta_perp+beta_par)/2, where
c beta_par = Ppar/B**2.
c Note that in an isotropic plasma, Pperp = Ppar = nT, although
c <Wperp> = 2 * <Wpar>.  Right.
C
      SUBROUTINE PLOTMO
C
C
C     COMMON BLOCKS
C
	include 'FPPCOM'
 
	dimension  BETPER(0:maxtim), BETEQ(0:maxtim)
	DIMENSION VPLOT(101),FPLOT(101)
C
C       SET UP PLOTTING ARRAYS
C
	AMMAX=0.0
	DO 500 II=0,LOOMAX
	BETPER(II)=1.5*BETBM(II)-0.5*BetaPar(II)
	BETEQ(II)=0.5*(BETPER(II)+BetaPar(II))
	BETBM(II)=BETBM(II)*100.0
	BETEQ(II)=BETEQ(II)*100.0
	BETPER(II)=BETPER(II)*100.0
	IF(BETBM(II).GT.AMMAX) AMMAX=BETBM(II)
	IF(BETEQ(II).GT.AMMAX) AMMAX=BETEQ(II)
	IF(BETPER(II).GT.AMMAX) AMMAX=BETPER(II)
500     CONTINUE
 
	call initt(0)
	CALL BINITT
	call erase
	call fpgrlab
 
	CALL NPTS(LOOMAX+1)
C       CALL YTYPE(2)
	CALL DLIMX(Timmy(0),Timmy(loomax))
	CALL DLIMY(0.,AMMAX)
	CALL CHECK(TIMMY,BETBM)
	CALL DSPLAY(TIMMY,BETBM)
	CALL LINE(7474)
	CALL CPLOT(TIMMY, BETPER)
	CALL LINE(5454)
	CALL CPLOT(TIMMY, BETEQ)
	WRITE(TEXT1,10)
10      FORMAT('BEAM TOROIDAL BETA, BETA PERP., BETA EQ. ( % )$')
	CALL AGTCGT(750,TEXT1)
	WRITE(TEXT1,30) AMMAX,NSHE-1
30      FORMAT('MAX =',1pE8.2,'	    ',I2,' SHELLS$')
	CALL AGTCGT(0,TEXT1)
	WRITE(TEXT1,85)
85      FORMAT('time (secs)$')
	CALL AGTCHS(TEXT1)
	NSHEL=NSHE-1
 
c--convert everything from a percentage to a fraction.  This
c       allows multiple calls to PLOTMO without beta getting
c       bigger by 100 everytime:
	do 40 ii=0,loomax
	betbm(ii)=betbm(ii)/100.0
	beteq(ii)=beteq(ii)/100.0
40      betper(ii)=betper(ii)/100.0
 
	CALL UPAUSE(0)
	call finitt(0,0)
C
C       ==========================================
C       PLOT THE DISTRIBUTION VS. PRECESSION SPEED
C       MAXIMUM 5 ZONES AT A TIME
C       =========================
C
C       LOOP OVER TIME VALUES, NPLTF SPECTRA VS. PRECESSION SPEED
C
	NPLTS=(NSHELL-1)/5 + 1
C
	DO 2000 ITMPR = 1, NPLTF
 
c--type out warning message:
	if(itmpr .eq. 1)write(lsysout,*)' precession section of plotmo',
	1       ' has not yet been written for arbitrary grids or times'
C
	DO 1000 IPLTS=1,NPLTS
C
	IRMAX=IPLTS*5
	IF(IRMAX.GT.NSHELL) IRMAX=NSHELL
	IRMIN=(IPLTS-1)*5 + 1
C
C       FIND MAXIMUM DATUM TO BE PLOTTED
C
	SMAX=-1.0E20
C
	DO 1300 I=IRMIN,IRMAX
	DO 1300 J=1,100
C
	IF(FPREC(ITMPR,I,J).GT.SMAX) SMAX=FPREC(ITMPR,I,J)
C
1300    CONTINUE
C
C
C     PLOT AT UP TO 5 RADII, NORMALIZED TO MAXIMUM
C
      FPLOT(1)=100.
      VPLOT(1)=100.
C
C       SET UP VPLOT ON FIRST CALL
C
	DATA ICAL /0/
	ICAL=ICAL+1
	IF(ICAL.EQ.1) THEN
		DO 3000 IVEL=2,101
		VPLOT(IVEL)=(IVEL-1.5)*VPRCMX/100.
3000	    CONTINUE
	ENDIF
C
	call initt(0)
	CALL BINITT
	call erase
	call fpgrlab
 
      DO 100 J=IRMIN,IRMAX
      DO 200 I=1,100
      FPLOT(I+1)=FPREC(ITMPR,J,I)/SMAX
  200 CONTINUE
C
      IF(J.EQ.IRMIN) CALL SLIMX(125,852)
      IF(J.EQ.IRMIN) CALL SLIMY(130,690)
      IF(J.EQ.IRMIN) CALL DLIMY(.01,.99)
      IF(J.EQ.IRMIN) CALL YTYPE(2)
	IF(J.EQ.IRMIN) CALL XTYPE(2)
	IF(J.EQ.IRMIN) CALL DLIMX(VPRCMX/90.,VPRCMX)
      IF(J.EQ.IRMIN) CALL CHECK(VPLOT,FPLOT)
      IF(J.EQ.IRMIN+1) CALL LINE(7474)
      IF(J.EQ.IRMIN+2) CALL LINE(5454)
      IF(J.EQ.IRMIN+3) CALL LINE(3434)
      IF(J.EQ.IRMIN+4) CALL LINE(1414)
      IF(J.EQ.IRMIN) CALL DSPLAY(VPLOT,FPLOT)
      IF(J.NE.IRMIN) CALL CPLOT(VPLOT,FPLOT)
  100 CONTINUE
      TIMM=MAX(1,LOOMAX/NPLTF)*2.*ITMPR*dtFPP*1000.
	WRITE(TEXT1,11)
11      FORMAT('F(V-PREC)$')
	CALL AGTCGT(750,TEXT1)
	WRITE(TEXT1,21) TIMM,IRMIN,IRMAX
21      FORMAT('T = ',F5.1,1X,'MSEC',5X,' SHELLS  ',I2,' ->',I2,'$')
	CALL AGTCGT(0,TEXT1)
	WRITE(TEXT1,31)
31      FORMAT('PRECESSION SPEED (CM/SEC)	      $')
	CALL AGTCHS(TEXT1)
	WRITE(TEXT1,41) SMAX
41      FORMAT(30X,'MAX = ',E10.3,'$')
	CALL AGTCGT(1,TEXT1)
	CALL UPAUSE(0)
	call finitt(0,0)
C
1000    CONTINUE
C
2000    CONTINUE
C
	RETURN
	END
