	real function Pphiorb(ZR)
c evaluate canonical toroidal angular momentum (minus a constant)
c for a given major radius ZR.
c
c Assuming that Psi(R,Y) increases monotonically in the vertical direciton,
c then setting zPphi0= Psi(r) - M R Vphi results in:
c
c 1. the roots of Pphiorb(R)=0 define where the orbit crosses the midplane.
c 2. Pphiorb(R) < 0 means that the orbit reaches R but with Y .ne. 0.
c 3. Pphiorb(Rmaj+rmin) < 0 or Pphiorb(Rmaj-rmin) < 0 means the orbit
c	is unconfined.
c 4. Pphiorb(R) > 0 means the orbit never reaches this R (for any value
c	of Y).
c 5. The minimum of Pphiorb(R) gives the R where the orbit reaches the
c	maximum Y above the midplane.
c
 
	include 'FPPCOM'
 
	common /zPphiCmn/ ZRtip, zmv, zPphi0  ! Rtip = R of banana tip
				       ! zmv = m * v of particle
				       ! zPphi0 = Pphi at banana tip
 
	zPsiofR = finteq1d(zR,PsiRj,RjPsi1,delRjPsi,numPsij)
 
	Pphiorb=zPsiofR - zmv * ZR * sqrt(max(0.0,1-ZRtip/ZR)) -zPphi0
 
	if(zr .lt. zrtip) then
	type *, ' PPHIORB warning, zR < zRtip!'
	endif
 
	return
	end
