C
C       THIS SUBROUTINE CALCULATES THE FAST ION PRECESSION
C       DISTRIBUTION.  WE CONVERT THE 2-D BEAM DISTRIBUTION
C       F(ENERGY, VPAR/V) INTO A 1-D DISTRIBUTION F(V-PRECESSION).
C       IN THE PROCESS WE CLEAN UP THE UNITS OF F, AND GET F(V)
C       IN UNITS OF PARTICLES/(CM**3 * CM/SEC)
C
C       INTEGRAL OF F(V-PREC) OVER V-PREC SHOULD GIVE PARTICLES
C       PER CUBIC CENTIMETER.  NOTE THAT F IS BOUNCE AVERAGED, SO
C       THIS ALL REFERS TO THE DISTRIBUTION FUNCTION AT THETA = 0.
C
C       IN THIS CASE WE ARE INCLUDING ONLY TRAPPED PARTICLES,
C       SO THE INTEGRAL WILL GIVE THE TRAPPED PARTICLE DENSITY ONLY
C
C       ROB GOLDSTON  6-6-83
C
C
	SUBROUTINE PRCDIS
C
	include 'FPPCOM'
C
 
c--type warning message:
	data iftime/0/
	if(iftime .eq. 0)
     1  write(lsysout,*) 'prcdis has not been rewritten for',
     1		   ' generalized grids yet.'			
	write(lsysout,*) 'and will not work until it is updated.'
 
c
c This subroutine uses Rob's original pitch angle grid, which was
c 50 points uniformly spaced in pitch angle (and didn't quite reach
c xsi= +/-1).  Furthermore, the vprec array was indexed differently.
c Rob only kept vprec for xsi>0 (since vprec is even in xsi), and
c so the vprec corresponding to xsi(j) was vprec(j-25+1), and
c vprec(1)=25 was the number of points in vprec.
c
c Vprec as calculated by SCCF is now indexed more straigtforwardly,
c with vprec(j) corresponding to xsi(j).
c
c This code has to be upgraded to the current grid system before
c it will work.
c
	if(iftime .eq. 0)stop
	iftime=iftime+1
C       SET UP MAXIMUM OF PRECESSION SPEED
C       AND GET NORMALIZATION OF SUM UP OF F (NOTE WE TAKE INTO
C       ACCOUNT THE BIN WIDTH IN V AND IN PITCH ANGLE HERE)
C       THE DISTRIBUTION WILL BE TRUNCATED AT VPRCMX
C
	VPNORM=10.0*VPRCMX/100.0
C
C
C       ZERO OUT THE DISTRIBUTION ON THE FIRST CALL
C
	DATA ICAL/0/
	ICAL=ICAL+1
	IF(ICAL.EQ.1) THEN
C
		DO 100 ITIMS=1,40
		DO 100 IRADS=1,25
		DO 100 IVELS=1,100
			FPREC(ITIMS,IRADS,IVELS)=0.0
100	     CONTINUE
	ENDIF
C
C
C       BUILD UP DISTRIBUTION FUNCTION, USING A SIMPLE ALGORITHM
C       WHICH TRACKS IN VPREC AND IN VPAR/V, USING A 10X SMALLER
C       GRID IN VPAR/V THAN THE MAIN CODE
C
C       SET UP TIME GROUP #
C
	ITMPR=niTime/(MAX(1,LOOMAX/NPLTF))
C
c       test to see where glitches come from:
	DO 200 IANG=1,njTrap
C
	DO 200 IASUB=1,10
C
	IF(IASUB.LE.5) THEN
		IANLS=25+IANG
	ELSE
		IANLS=26+IANG
	ENDIF
C
	IANMR=IANLS+1
	IANLS=MAX(27,IANLS)
	IANMR=MIN(26+njTrap,IANMR)
C
	IF((IANG.EQ.1.AND.IASUB.LE.5)
	1   .OR.(IANG.EQ.njTrap.AND.IASUB.GE.6)) THEN
		TERPA=0.5
	ELSE IF (IASUB.LE.5) THEN
		TERPA=(IASUB-0.5)*0.1+0.5
	ELSE IF (IASUB.GT.5) THEN
		TERPA=(IASUB-5.5)*0.1
	ENDIF
C
	VINTRP=(1.0-TERPA)*VPREC(IANLS-25)+TERPA*VPREC(IANMR-25)
	IF(VINTRP.LE.0.0) GOTO 200
	VPR50=VINTRP*enerev(maxen)*1.9155E12/(9578.*BTOR*RMAJ)
	DENP=VPRCMX/VPR50*enerev(maxen)/100.0
C
	DO 300 IPRC=1,100
C
	VPR=(IPRC-0.5)*VPRCMX/100.0
	IF(VPR.GT.VPR50) GOTO 300
C
	IENLS=INT(50.*VPR/VPR50)+1
	VPRLS=(IENLS-1.0)/50.0*VPR50
	IENLS=MIN(51,MAX(IENLS,2))
C
	IENMR=IENLS+1
	VPRMR=(IENMR-1.0)/50.0*VPR50
	IENMR=MAX(2,MIN(IENMR,51))
C
	TERPE=(VPR-VPRLS)/(VPRMR-VPRLS)
	zfintp=(1.0-TERPE)*(1.0-TERPA)*F(IENLS,IANLS) +
     >	  (1.0-TERPE)*TERPA*F(IENLS,IANMR) +
     >	  TERPE*(1.0-TERPA)*F(IENMR,IANLS) +
     >	  TERPE*TERPA*F(IENMR,IANMR)
C
	EINTRP=enerev(maxen)/50.0*(TERPE+(IENLS-1.0))
C
C       NOTE THAT THE DISTRIBUTION IS SUPPOSED TO BE SYMMETRIC IN
C       THE TRAPPED ZONE, WHICH IS ALL THAT WE ARE INCLUDING
C
	FPREC(ITMPR,NSHE,IPRC)=FPREC(ITMPR,NSHE,IPRC)+zfintp*2.0
     >	  *DENP*SQRT(EINTRP)/VPNORM
C
300     CONTINUE
C
200     CONTINUE
C
C
	RETURN
	END
