	subroutine preadi
C
C     THIS SUBROUTINE SETS UP MATRICES FOR THE ALTERNATING-
C     DIRECTION-IMPLICIT INTEGRATION OF THE FOKKER-
C     PLANCK EQUATION
C
c gwh 28-Mar-1989: Try solving energy step in the opposite direction
c in order to implement the boundary conditions at E=0 in a fully
c fully implicit way.
c
! gwh July 16, 1984: Modified to eliminate the problem of f going
! negative if the energy grid is too coarse.  This is done by
! evaluating the slowing down term on the higher grid point, not
! averaging between grid points.  While this is only accurate
! to order (dE/T), and the old method is accurate to order (dE/T)**2,
! that is only significant for dE/T small.  For dE/T big, the old
! method would cause f to oscillate about zero.  The new method
! keeps f positive definite while it approaches zero. Formally:
!
!	f(i+1) = f(i) * exp(-dE/T)		!exact
!	f(i+1) = f(i) * (1-.5*dE/T)/(1+.5*dE/T)	!old method
!	f(i+1) = f(i)              /(1+   dE/T)	!new method
 
c--gwh:modified to add collisional contributions to already
c	calculated RF terms.
c--gwh:modified to generalize B.C.'s to arbitrary F.P. operator
c--gwh:modified to allow conservative pitch angle scattering
C
C
	include 'FPPCOM'
 
c**************************************************************
c                                                             *
c	ENERGY SPACE					      *
c                                                             *
c**************************************************************
c
C     ...MATRIX COEFFICIENTS
C
	DO 200 J=1,maxxsi
	DO 100 I=1,maxen-1
 
c-- *** the general grid version of this: ***
c--dtFPP=time step
c--den=energy grid spacing
c--add collisional contributions to already calculated RF terms:
c	ALPHE(I,J)=alphe(i,j)-dtFPP*OUT(I)*DRAG(I+1)/(2.*DEN)-dtFPP*OUT(I)*
c     >  DIFFE(I)/DEN**2-ELDR(I,J)*dtFPP/(2.*DEN)
c	BETE(I,J)=bete(i,j)+dtFPP*OUT(I)*(DIFFE(I)+DIFFE(I-1))/DEN**2
c	1	+dtFPP*RATE(I)
c	GAME(I,J)=game(i,j)+dtFPP*OUT(I)*DRAG(I-1)/(2.*DEN)-dtFPP*OUT(I)
c     >  *DIFFE(I-1)/DEN**2+ELDR(I,J)*dtFPP/(2.*DEN)
 
c-- *** is:                              ***
 
	alphe(i,j)=alphe(i,j) -dtFPP*out(i)*(drag(i+1)
	1	+diffe(i)/(enerev(i+1)-enerev(i)))/delten(i)
 
	bete(i,j)=bete(i,j) +dtFPP*out(i)*
	1	(drag(i)
	1	 +diffe(i)/(enerev(i+1)-enerev(i))
	1	  +diffe(i-1)/(enerev(i)-enerev(i-1)) )/delten(i)
	1	+dtFPP*(rate(i)+ratedl(i,j))
 
	game(i,j)=game(i,j) +dtFPP*out(i)*(
	1	-diffe(i-1)/(enerev(i)-enerev(i-1)) )/delten(i)
 
c--note that the ohmic electric field does not appear in these
c	equations, since it has already been included by ELCF.
 
100	continue
 
c top boundary:
 
	i=maxen
	alphe(i,j)=0.0
	bete(i,j)=bete(i,j)+ dtFPP*out(i)*(drag(i)
	1	 + diffe(i-1)/(enerev(i)-enerev(i-1)) )/delten(i)
	1	 +dtFPP*(rate(i)+ratedl(i,j))
 
	game(i,j)=game(i,j)+dtFPP*out(i)*(
	1		 - diffe(i-1)/(enerev(i)-enerev(i-1)) )
	1		/delten(i)
 
c bottom boundary:
 
	alphe(0,j)=alphe(0,j)-dtFPP*out(1)
	1		*(drag(1) +diffe(0)/(enerev(1)-enerev(0)))
	1		*sqrt(enerev(1))/delten(0)/rjac0
 
	bete(0,j)=bete(0,j)-dtFPP*out(1)
	1		*(-diffe(0)/(enerev(1)-enerev(0)))
	1		*sqrt(enerev(1))/delten(0)/rjac0
	1		+dtFPP*(rate(0)+ratedl(0,j))
 
	game(0,j)=0.0
 
C
C     ...E COEFFICIENTS
C
 
c top boundary condition:
	i=maxen
	EE(I,J)=-loss(i,j)*game(I,J)/BETE(I,J)
 
	DO 150 I=maxen-1,0,-1
	EE(I,J)=-loss(i,j)*game(I,J)/(BETE(I,J)+alphe(I,J)*EE(I+1,J))
150	continue
200	continue
 
c**************************************************************
c                                                             *
c	PITCH ANGLE SPACE				      *
c                                                             *
c**************************************************************
c
C     ...MATRIX COEFFICIENTS
C
 
c--pitch angle scattering is infinite at i=0
	DO 300 I=1,maxen
 
C
C     ...E COEFFICIENTS
C
C     ......COUNTER
C
	ea(i,1)=-loss(i,j)*alpha(i,1)/beta(i,1)
	M2=maxxsi/2-njTrap
	DO 400 J=2,M2
400	EA(I,J)=-loss(i,j)*ALPHA(I,J)/(BETA(I,J)+GAMA(I,J)*EA(I,J-1))
C
C     ......CO
C
	ea(i,maxxsi)=-loss(i,j)*gama(i,maxxsi)/beta(i,maxxsi)
	M3=maxxsi/2+1+njTrap
	DO 500 J=maxxsi-1, m3, -1
500	EA(I,J)=-loss(i,j)*GAMA(I,J)/(BETA(I,J)+ALPHA(I,J)*EA(I,J+1))
C
C     ......TRAPPED
C
	EA(I,maxxsi/2)=-loss(i,j)*GAMA(I,maxxsi/2)
	1	/(ALPHA(I,maxxsi/2)+BETA(I,maxxsi/2))
	M4=maxxsi/2+2-njTrap
	DO 600 J=maxxsi/2-1, m4, -1
600	EA(I,J)=-loss(i,j)*GAMA(I,J)/(BETA(I,J)+ALPHA(I,J)*EA(I,J+1))
 
300	continue
 
	return
	end
