	subroutine prepre
c--called just before adi.
 
c--The collisional drag term is discretized in a slightly different
c	way in subroutine PREADI to maintain f>0.  Here, we continue
c	to discretize dea in the old manner.  dea only contains
c	ohmic electric field terms and hopefully won't cause any
c	problems.
 
c--The discretized diffusion equation can be written as
c	a huge matrix equation:
c	old f = matrix * new f
c on the implicit time step, or on the explicit time step:
c	new f = matrix * old f
c On the implicit time step, the matrix must be inverted to find
c the solution for the new f.  For a diffusion equation in 1 space
c dimension, the implicit equation can be written as:
c    fold(i) = alpha(i)*f(i+1) + beta(i)*f(i) + gamma(i)*f(i-1)
c Which can be easily inverted with special tricks for tri-diagonal
c matrices.  The ADI method involves implicit solution in only
c one dimension at a time so that the tri-diagonal trick works.
c The alpha, beta, and gamma coefficients are calculated here and
c in subroutine preadi.  preadi also calculates some numbers
c needed for the tri-diagonal trick.  Subroutine adi does the
c final matrix inversion.
c--This subroutine evaluates RF contributions to the matrix elements.
c--These formulas can be found in "Particle conserving B.C.'s for FPP",
c	Oct. 17, 1983.
!
! gwh, Oct. 14, 1985, modified to evaluate cross-derivative
! diffusion coefficients between grid points.
!
 
	include 'FPPCOM'
 
c--It might be possible to simplify or optimize some of these
c	equations, by merging the general, top, bottom, left and
c	right calculations together:
 
c--loop over interior points where discretization formulas are all
c	the same:
 
	do 100 i=1,maxen-1
	do 100 j=2,maxxsi-1
 
	deltep=enerev(i+1)-enerev(i)
	deltem=enerev(i)-enerev(i-1)
	qjac=enerev(i)**.5 *abs(xsi(j))*btime(j)
 
	const=dtFPP/qjac/delten(i)
	alphe(i,j)=-const*(.5*dea(i+1,j)+deb(i,j)/deltep)
	bete(i,j)=1+const*(deb(i,j)/deltep+deb(i-1,j)/deltem)
	game(i,j)=-const*(-.5*dea(i-1,j)+deb(i-1,j)/deltem)
 
	deltkp=xsi(j+1)-xsi(j)
	deltkm=xsi(j)-xsi(j-1)
	const=dtFPP/qjac/delxsi(j)
	alpha(i,j)=-const*(.5*daa(i,j+1)+dab(i,j)/deltkp)
	beta(i,j)=1+const*(dab(i,j)/deltkp+dab(i,j-1)/deltkm)
	gama(i,j)=-const*(-.5*daa(i,j-1)+dab(i,j-1)/deltkm)
 
100	continue
 
c--calculate gamap, a special coefficient which comes out of
c	averaging barely trapped co- and counter- particles:
	j=maxxsi/2+njTrap
	deltkp=xsi(j+1)-xsi(j)
	do 150 i=1,maxen
	qjac=enerev(i)**.5*abs(xsi(j))*btime(j)
	const=dtFPP/qjac/delxsi(j)
150	gamap(i)=-const*(.5*daa(i,j+1)+dab(i,j)/deltkp)
 
 
c--right boundary
	j=maxxsi
 
	do 200 i=1,maxen-1
c--xsi(maxxsi)=1, so btime should =1, but is slightly off.
c	to be consistent with moms, we use the given value of btime:
	qjac=enerev(i)**.5*btime(j)
	deltep=enerev(i+1)-enerev(i)
	deltem=enerev(i)-enerev(i-1)
 
c--these formulas assume dac(i,maxxsi)=dec(i,maxxsi)=0.0,
c	which is true for rf terms, and is probably always true.
	const=dtFPP/qjac/delten(i)
	alphe(i,j)=-const*(.5*dea(i+1,j)+deb(i,j)/deltep)
	bete(i,j)=1+const*(deb(i,j)/deltep+deb(i-1,j)/deltem)
	game(i,j)=-const*(-.5*dea(i-1,j)+deb(i-1,j)/deltem)
 
	const=dtFPP/qjac/delxsi(j)
	alpha(i,j)=0.0
	beta(i,j)=1+const*.5*(daa(i,j)+dab(i,j-1)/delxsi(j))
	gama(i,j)=const*.5*(daa(i,j-1)-dab(i,j-1)/delxsi(j))
 
200	continue
 
c--left boundary
	j=1
 
	do 300 i=1,maxen-1
c--xsi(maxxsi)=1, so btime should =1, but is slightly off.
c	to be consistent with moms, we use the given value of btime:
	qjac=enerev(i)**.5*btime(j)
	deltep=enerev(i+1)-enerev(i)
	deltem=enerev(i)-enerev(i-1)
 
c--these formulas assume dac(i,1)=dec(i,1)=0.0,
c (note, dac(i,j) is now DAC evaluated at (i,j+1/2), so the condition
c dac(i,1)=0 no longer holds.  I presume that the formulas have been
c corrected properly, but I should probably double check before
c removing the above comment.)
c which is true for rf terms, and is probably always true.
	const=dtFPP/qjac/delten(i)
	alphe(i,j)=-const*(.5*dea(i+1,j)+deb(i,j)/deltep)
	bete(i,j)=1+const*(deb(i,j)/deltep+deb(i-1,j)/deltem)
	game(i,j)=-const*(-.5*dea(i-1,j)+deb(i-1,j)/deltem)
 
	const=dtFPP/qjac/delxsi(j)
	alpha(i,j)=-const*.5*(daa(i,2)+dab(i,1)/delxsi(j))
	beta(i,j)=1-const*.5*(daa(i,j)-dab(i,j)/delxsi(j))
	gama(i,j)=0.0
 
300	continue
 
c--top boundary
	i=maxen
	deltem=enerev(maxen)-enerev(maxen-1)
 
	do 400 j=1,maxxsi
	qjac=enerev(i)**.5*abs(xsi(j))*btime(j)
	const=dtFPP/qjac/delten(i)
c alphe is zero because there is no flux out the top of our grid:
	alphe(i,j)=0.0
	bete(i,j)=1.0 +const*(.5*dea(i,j)+deb(i-1,j)/deltem)
	game(i,j)=const*(.5*dea(i-1,j)-deb(i-1,j)/deltem)
 
c Include Pitch angle scattering at the top of our grid:
	const=dtFPP/qjac/delxsi(j)
 
	if(j .eq. 1) then
 
	alpha(i,j)=-const*.5*(daa(i,2)+dab(i,1)/delxsi(j))
	beta(i,j)=1-const*.5*(daa(i,j)-dab(i,j)/delxsi(j))
	gama(i,j)=0.0
 
	else if (j .eq. maxxsi)then
 
	alpha(i,j)=0.0
	beta(i,j)=1+const*.5*(daa(i,j)+dab(i,j-1)/delxsi(j))
	gama(i,j)=const*.5*(daa(i,j-1)-dab(i,j-1)/delxsi(j))
 
	else
 
 	deltkp=xsi(j+1)-xsi(j)
	deltkm=xsi(j)-xsi(j-1)
	alpha(i,j)=-const*(.5*daa(i,j+1)+dab(i,j)/deltkp)
	beta(i,j)=1+const*(dab(i,j)/deltkp+dab(i,j-1)/deltkm)
	gama(i,j)=-const*(-.5*daa(i,j-1)+dab(i,j-1)/deltkm)
 
	endif
 
400	continue
 
c--bottom boundary:
	i=0
	deltep=enerev(1)-enerev(0)
	do 500 j=1,maxxsi
 
	qjac=rjac0*abs(xsi(j))*btime(j)
	const=dtFPP/qjac/delten(i)
	alphe(i,j)=-const*(.5*dea(i+1,j)+deb(i,j)/deltep)
	bete(i,j)=1-const*(.5*dea(i,j)-deb(i,j)/deltep)
	game(i,j)=0.0
 
	alpha(i,j)=0.0
	beta(i,j)=1.0
	gama(i,j)=0.0
 
500	continue
 
c--calculate cross-derivative terms:
	do 550 i=0,maxen
	do 550 j=1,maxxsi
	const=dtFPP/(qjace(i)*abs(xsi(j))*btime(j)*4*delxsi(j))
	decp(i,j)=const*dec(i,j)
	decm(i,j)=const*dec(i-1,j)
	dacp(i,j)=const*dac(i,j)
550	dacm(i,j)=const*dac(i,j-1)
 
c  make sure there is no flux in angle from the cross-derivative terms
c at the top of our grid:
	i=maxen
	do 575 j=1,maxxsi
	decp(i,j)=0.0
	dacp(i,j)=0.0
575	dacm(i,j)=0.0
 
	return
	end
