C
C	THIS PROGRAM CALCULATES THE POLOIDAL FLUX PROFILE
C	FOR THE BOUNCE-AVERAGED FOKKER-PLANCK CODE
C
C	IT FILLS AN ARRAY WITH PSI(R), AND A SECOND
C	ARRAY WITH R(PSI) FOR USE IN THE FINITE BANANA
C	WIDTH INJECTION, DETECTION, AND LOSS-CONE SECTIONS
C
C	PSI(1) IS PSI @ R=0 (= 0), PSI(numpsi) IS PSI @ R=A
C	RPSI(1) IS R @ PSI=0 (=0), RPSI(numpsi) IS R AT PSI=PSI(A) (=A)
C
C	IT USES q(r) as specified in the array qpro.
c
c--Particle orbits in a tokamak can be calculated from
c	conservation of canonical angular momentum:
c	Pphi = m R vparallel - psiec = constant
c
c	psiec = -Aphi R e/c
c
c	d(psiec)/dr = (e/c)*R*Bp
C
C	ROB GOLDSTON OCT. 4, 1981
C
	SUBROUTINE PSICAL
C
C	COMMON BLOCKS
C
	include 'FPPCOM'
 
	external fpgbinv ! calling root finder with this routine will
	real zBrequ ! find the theta giving this zBrequ
	real zXi    ! on this flux surface (normalized rho/rho_lim)
	common /fpgbcom/ zxi, zBrequ
	real zJac(2,2)	! jacobian calculated by MOMRY
c
c  dmc 22 Feb 1993
c  replacing IMSL root finder ZBREN with call to ZEROIN
c
	data zzeps/0.0/
c
C
c--eovc = (e/c) = electron charge over the speed of light (cgs units):
	DATA EOVC/1.6021E-20/
	zpi=abs(acos(-1.0))
c
c get machine epslon for root finder
	if(zzeps.eq.0.0) zzeps=x02aje()
 
c calculate profile of dq/dr for future subroutines:
c use the fact that q is symmetric in r so that
c dqdr=(q(2)-q(-1))/(r(2)-r(-2)) yields:
	dqdrpro(1)=(qpro(2)-qpro(1))/(rminpro(2)+rminpro(1))
	do 57 i=2,nshell-1
57	dqdrpro(i)=(qpro(i+1)-qpro(i-1))/(rminpro(i+1)-rminpro(i-1))
	dqdrpro(nshell)=(qpro(nshell)-qpro(nshell-1))/
	1   (rminpro(nshell)-rminpro(nshell-1))
 
C
C calculate Psi(r) from
c
c d(Psi)/dr = (ze/c 2 pi) (1/q) d(Phi)/dr
c
c where Phi is the toroidal flux and Psi is our definition
c of poloidal flux. (dPsi/dr = (1/q) dPhi/dr is the usual definition
c but we have an extra factor in our Psi.)
C
C note that Psi has been rescaled to be ze/c/(2 pi) times the raw
C poloidal flux.
 
	delrPsi=rmin/(numpsi-1.0)
	PSI(1)=0.0
	DO 100 I=1,numpsi-1
	zrloc=rmin*(I-0.5)/(numpsi-1.)
	zqloc=fpintrp(zrloc,qpro,rminpro,nshell)
	PSI(I+1)=PSI(I)
	1   +rZbeam*eovc*delrPsi*zrloc/zqloc*tflxtot/(zpi*rmin**2)
	
100	CONTINUE
 
c calculate Psi(R), i.e., Psi versus Rmajor on the midplane
 
	call momry(1.0,zpi,0,Rjpsi1,zy0,zjac)
	call momry(1.0,0.0,0,zRmajmax,zy0,zjac)
	delRjPsi=(zRmajmax-Rjpsi1)/(numPsij-1.0)
	RplaMin=Rjpsi1	    ! save for use by routine ORBSHIFT
	RplaMax=zRmajmax    ! save for use by routine ORBSHIFT
 
	do 120 i=1,numPsij
	zRmid=Rjpsi1+(i-1)*delRjPsi
	call nbimap(zRmid,0.0,zxi,zth,ier)
	PsiRj(i)=finteq1d(zxi*rmin,Psi,0.0,delrPsi,numPsi)
120	continue
 
c set minimum of Psi(Rmid) = 0:
	imin=1
	zmin=PsiRj(1)
	do 125 i=2,numPsij
	if(PsiRj(i) .lt. zmin) then
	    zmin=PsiRj(i)
	    imin=i
	endif
125	continue
	PsiRj(imin)=0.0
 
C
C	CALCULATE R(PSI)
C
	delPsir=Psi(numpsi)/(numpsi-1.0)
	RPSI(1)=0.0
	RPSI(numpsi)=RMIN
	ILAST=2
	DO 200 J=2,numpsi-1
	PSIPR=FLOAT(J-1)*delPsir
	DO 150 I=ILAST-1,numpsi
	IF(PSI(I).LT.PSIPR) GOTO 150
	ILAST=I
	GOTO 250
150	CONTINUE
250	RIPSI=((PSIPR-PSI(ILAST-1))*FLOAT(ILAST)
     >	+(PSI(ILAST)-PSIPR)*FLOAT(ILAST-1))/(PSI(ILAST)-PSI(ILAST-1))
	RPSI(J)=(RIPSI-1.0)*delrPsi
200	CONTINUE
 
c calculate a courser array of psipro(r):
	do 300 i=1,nshell
	index=(rminpro(i)/rmin)*(numpsi-1)+1
300	psipro(i)=psi(index)/rZbeam/eovc
 
c
c calculate arrays to do a quick inverse map from (B,r) to theta
c (poloidal angle), or from (B,r) to Y (vertical position).
c This is used if you want to know where a particle's banana tip
c is, or where its resonance is.
c
 
c ir index runs over zone centers and zone boundaries:
 
	do 400 i=1,nshell
	do 400 iz=0,1
	ir=2*i-1+iz
	if(iz .eq. 0) zxi=xradzc(i)
	if(iz .eq. 1) zxi=xradzb(i)
c ensure MOMRY's jacobian is evaluated just inside the boundary:
	if(zxi .ge. 1.0) zxi=0.9999
	zBrequ=0.0
	Bminofr(ir)=fpgbinv(0.0)
	Bmaxofr(ir)=fpgbinv(zpi)
	Bavgofr(ir)=(Bmaxofr(ir)+Bminofr(ir))/2.
	Bdelofr(ir)=(Bmaxofr(ir)-Bminofr(ir))/2.
 
	numthofB(ir)=maxthofb*sqrt(float(i)/float(nshell))
	if(numthofB(ir) .lt. 3) numthofB(ir)=3
 
	delthofB(ir)=zpi/(numthofB(ir)-1)
 
	ThofBr(1,ir)=0.0
	ThofBr(numthofB(ir),ir)=zpi
 
c calculate omppofr = d**2 Omega / d l**2 at theta=0 and pi:
	zqloc=fpintrp(zxi*rmin,qpro,rminpro,nshell)
	jb=1
	zth1=0.0
 
	call momry(zxi,zth1,1,zRdum,zYdum,zJac)
	zj0=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zB0=Btor*Rmaj/zRdum
	zdth=zpi/numthofB(ir)
	call momry(zxi,zth1+zdth/2.,1,zRdum,zYdum,zJac)
	zjp=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zdRp=zjac(1,2)/zRdum
	call momry(zxi,zth1-zdth/2.,1,zRdum,zYdum,zJac)
	zjm=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zdRm=zjac(1,2)/zRdum
	omppofr(jb,ir)=-9.579e3*rZbeam/Ab/zB0/zj0*(zdRp/zjp-zdRm/zjm)
	1   /zdth
 
	jb=numthofB(ir)
	zth1=zpi
 
	call momry(zxi,zth1,1,zRdum,zYdum,zJac)
	zj0=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zB0=Btor*Rmaj/zRdum
	zdth=zpi/numthofB(ir)
	call momry(zxi,zth1+zdth/2.,1,zRdum,zYdum,zJac)
	zjp=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zdRp=zjac(1,2)/zRdum
	call momry(zxi,zth1-zdth/2.,1,zRdum,zYdum,zJac)
	zjm=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zdRm=zjac(1,2)/zRdum
	omppofr(jb,ir)=-9.579e3*rZbeam/Ab/zB0/zj0*(zdRp/zjp-zdRm/zjm)
	1   /zdth
 
c loop around from theta=0 to pi to find the various inverse maps:
 
	do 390 jb=2,numthofB(ir)-1
	zthstar=(jb-1)*delthofB(ir)
	zBrequ=Bavgofr(ir)-Bdelofr(ir)*cos(zthstar)
	zth0=ThofBr(jb-1,ir)
	zth1=zpi
	imaxit=100
c finds theta to an accuracy of +/- 1.e-5*1.e-1:
	zth1=zeroin(zth0,zth1,fpgbinv,1.0e-6,zzeps)
CIMSL	call zbren(fpgbinv,0.0,1.e-5,zth0,zth1,imaxit)
	ThofBr(jb,ir)=zth1
	call momry(zxi,zth1,1,zRdum,YofBr(jb,ir),zJac)
 
c calculate omppofr = d**2 Omega / d l**2:
	zj0=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zB0=Btor*Rmaj/zRdum
	zdth=zpi/numthofB(ir)
	call momry(zxi,zth1+zdth/2.,1,zRdum,zYdum,zJac)
	zjp=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zdRp=zjac(1,2)/zRdum
	call momry(zxi,zth1-zdth/2.,1,zRdum,zYdum,zJac)
	zjm=zpi*zqloc/zxi/tflxtot*zRdum*(zjac(1,1)*zjac(2,2)-
	1   zjac(2,1)*zjac(1,2))
	zdRm=zjac(1,2)/zRdum
	omppofr(jb,ir)=-9.579e3*rZbeam/Ab/zB0/zj0*(zdRp/zjp-zdRm/zjm)
	1   /zdth
 
390	continue
400	continue
 
	return
	end
