	SUBROUTINE rdstepdw(f1rad,fnbsours)
c
c take a radial transport time step due to drift-wave turbulence
c Greg Hammett  25-May-1991  (this subroutine grew from rdstep).
c
c Solves the equation:
c
c d/dt f = (1/Jacobian(r)) (1/r) d/dr ( r Jacobian(r) D (df/dr) )
c
c where Jacobian(r) = btime(xsi,r) = btxsi.
c
c Take care to difference this equation conservatively.
c Unlike subroutine rdstep, which had to take great pains to ensure
c that the radial transport mechanism conserved E and mu, in this
c rdstepdw routine, we will simply assume that the conserved
c quantities are FPP's natural coordinates E and xsi.
c
	include 'FPPCOM'
	include 'F1COM'
 
! coefficients of tri-diagonal matrix resulting from the diffusion
! equation:
	real zalpha(maxshe)
	real zbeta(maxshe)
	real gamma(maxshe)
	real source(maxshe)
! and the coeficients of the recursive solution:
	real epsilon(maxshe)
	real delta(maxshe)
 
	real fmu1(maxshe)   ! f(r) before radial time step
	real fmu2(maxshe)   ! f after radial time step
 
! declare local scalars:
	real dtradial,dr,zdrp,zdrm,zJac,zJacp,zJacm,zRtip,zavg
	integer ien,i,j,ishell,msconf
 
! define the radial time step:
	dtradial=timmy(loomax)-timmy(loomin)
 
	do  900 ien = 1,maxen  !ignore transport for 0 energy particles
	    do 800  j=1,maxxsi
 
! zero out arrays:
	    do 50 ishell=1,nshell
		fmu1(ishell)=0.0
		fmu2(ishell)=0.0
50	  continue
 
! maximum confined shell at this energy and pitch angle:
! ignore finite banana widths and just set it to be nshell for now:
		msconf=nshell
		if(msconf .lt. 1) go to 800
 
! find initial fmu1(ishell):
		do 100 ishell=1,msconf
		fmu1(ishell)=f1rad(ien,j,ishell)
100	     continue
	
c find the coefficients of the transport equation versus radius:
 
	do 199 i=1,msconf
 
c find grid step sizes:
 
	if(i .gt. 1 .and. i .lt. nshell)then
	    dr=(rminpro(i+1)-rminpro(i-1))/2.0
	    zdrp=(rminpro(i+1)-rminpro(i))
	    zdrm=(rminpro(i)-rminpro(i-1))
	endif
	if(i .eq. 1) then
	    dr=(rminpro(2)+rminpro(1))/2.
	    zdrp=(rminpro(i+1)-rminpro(i))
	    zdrm=rminpro(i)
	endif
	if(i .eq. nshell) then
	    dr=rmin-(rminpro(nshell)+rminpro(nshell-1))/2.
	    zdrp=rmin-rminpro(i)
	    zdrm=(rminpro(i)-rminpro(i-1))
	endif
 
	zJac=rminpro(i)*btxsi(j,i)
 
c evaluate coefficients of the flux at i+(1/2):
 
	if(i .lt. nshell)then
	    zJacp=0.25*(rminpro(i)+rminpro(i+1))*
	1	(btxsi(j,i)+btxsi(j,i+1))
	else
	    zJacp=0.5*(rmin+rminpro(nshell))*btxsi(j,nshell-1)
	endif
 
c evaluate coefficients of the flux at i-(1/2):
 
	if(i .gt. 1)then
	    zJacm=0.25*(rminpro(i)+rminpro(i-1))*
	1	(btxsi(j,i)+btxsi(j,i-1))
	else
	    zJacm=0.0
	endif
 
c find coefficients of the tridiagonal equation to be inverted:
c
c	alpha(i) f(i+1)  +  beta(i) f(i)  +  gamma(i) f(i-1)  =  S(i)
c
	    zalpha(i)=dtradial*(zJacp*fpdifdw)
	1       / (zJac*dr*zdrp)
	    gamma(i)=dtradial*(zJacm*fpdifdw)
	1       / (zJac*dr*zdrm)
	    zbeta(i)=-1.0 -dtradial*(zJacp/zdrp+zJacm/zdrm)*fpdifdw
	1       / (zJac*dr)
 
	source(i)=-fmu1(i)
 
199     continue
 
 
!solve the tri-diagonal matrix:
	delta(1)=source(1)/zbeta(1)     !boundary conditions
	epsilon(1)=-zalpha(1)/zbeta(1)
 
	if(msconf .ge. 2)then
	do 250 i=2,msconf
	epsilon(i)=-zalpha(i)/(zbeta(i)+gamma(i)*epsilon(i-1))
250     delta(i)=(source(i)-gamma(i)*delta(i-1))
	1       /(zbeta(i)+gamma(i)*epsilon(i-1))
	endif
 
	fmu2(msconf)=delta(msconf)
	if(msconf .ge. 2)then
	do 270 i=msconf-1,1,-1
270     fmu2(i)=epsilon(i)*fmu2(i+1)+delta(i)
	endif
 
c debug:  check for particle conservation (relevant only
c if conservative BC's are being used so there is no flux
c across the plasma boundary:
c	zsum1=0.0
c	zsum2=0.0
c	do 300 i=1,nshell
c	zsum1=zsum1+fmu1(i)*btxsi(j,i)*rminpro(i)*dr
c300	zsum2=zsum2+fmu2(i)*btxsi(j,i)*rminpro(i)*dr
c	if(abs(zsum1-zsum2) .gt. .01*zsum1 .and. zsum1 .gt.
c	1   1.e-30)then
c	    type *,' Warning in rdstep:  particles not conserved!'
c	endif
 
!***************************************************************
!
! copy new fmu2(r) back into f1rad(energy,xsi,r)
!
!***************************************************************
 
	do 325 ishell=1,nshell
	f1rad(ien,j,ishell)=fmu2(ishell)
325	continue
 
800     continue      !end xsi(j) loop
 
 
c symmetrize the trapped region of f:
	do 850 ishell=1,nshell
	do 830 j=maxxsi/2+1,maxxsi
	zRtip=(1.0-xsi(j)**2)*(Rmaj+rminpro(ishell))
	if(zRtip .lt. (Rmaj-rminpro(ishell))) goto 850
	zavg=(f1rad(ien,j,ishell)+f1rad(ien,maxxsi+1-j,ishell))/2
	f1rad(ien,j,ishell)=zavg
	f1rad(ien,maxxsi+1-j,ishell)=zavg
830	continue
850	continue
 
900     continue  !end ien loop
 
	return
	end
