      subroutine root(ckrt,
     %          w,akz,nions,tplus0,tplus1,tmins0,tmins1,alpha,alfven)
      include 'SPRINC'
      complex ckrt(4),dc(4),tperp0,tperp1,tcrss0,tcrss1,alp,
     %        tplus0(ionmax+1),tplus1(ionmax+1),tmins0(ionmax+1),
     %        tmins1(ionmax+1),alpha(ionmax+1)
      complex z1,z2
      integer nwarns,mxwarns
      data nwarns /0/, mxwarns /10/
      save nwarns
c
c     ROOT finds the four complex roots of the dispersion relation.
c     The roots are ordered to identify their asymptotic behavior:
c     1. FW with Re(kx)>0 or Im(kx)>0, 2. FW with Re(kx)<0 or Im(kx)<0,
c     3. BW with Re(kx)>0 or Im(kx)<0, 4. BW with Re(kx)<0 or Im(kx)>0.
c
c GWH:  Modified to return the Fast Wave kx**2 in ckrt(1), and the
c Bernstein Wave kx**2 in ckrt(3), rather than kx itself.  This avoids
c taking a square root.
c
c GWH: Define the function CABS1 to be a 1-norm of a complex number.
c This will evaluate much faster than CABS which involves taking
c a square root.
c
	real cabs1
	cabs1(z1) = abs(real(z1)) + abs(aimag(z1))
 
      ako2=1.11265e-17*w**2
      nspec=nions+1
      tperp0=(1.,0.)
      tperp1=0.
      tcrss0=0.
      tcrss1=0.
      alp=0.
      do 5 isp=1,nspec
      tperp0=tperp0+tplus0(isp)+tmins0(isp)
      tcrss0=tcrss0+tplus0(isp)-tmins0(isp)
      tperp1=tperp1+tplus1(isp)+tmins1(isp)
      tcrss1=tcrss1+tplus1(isp)-tmins1(isp)
    5 alp=alp+alpha(isp)
c
c     Calculate polynomial coefficients
      tperp0=(cmplx(ako2,0.)*tperp0-cmplx(akz**2,0.))/((1.,0.)-alp)
      tcrss0=cmplx(ako2,0.)*tcrss0/((1.,0.)-alp)
      tperp1=(tperp1-alp)/((1.,0.)-alp)
      tcrss1=tcrss1/((1.,0.)-alp)
      dc(3)=tperp0**2-tcrss0**2
      dc(2)=(2.,0.)*(tperp0*tperp1-tcrss0*tcrss1)-tperp0
      dc(1)=tperp1**2-tcrss1**2-tperp1
c
c     Solve for polynomial roots:  a k**4 + b k**2 + c =0
c	where a=dc(1), b=dc(2), c=dc(3).
c     (Use asymptotic expressions if a->0.)
 
c      dc(4)=-.5*(dc(2)+csqrt(dc(2)**2-4.*dc(1)*dc(3)))
c      ckrt(1)=dc(3)/dc(4)
c      ckrt(3)=dc(4)/dc(1)
 
	z1=4*dc(1)*dc(3)
	z2=dc(2)**2
	relacc=1.e-4
 
c test the magnitude of 4*a*c relative to b**2:
 
	if(cabs1(z1) .gt. relacc*cabs1(z2)) then
 
	ckrt(1)= (-dc(2) + csqrt(z2-z1))/(2.*dc(1))
	ckrt(3)= (-dc(2) - csqrt(z2-z1))/(2.*dc(1))
 
	else
 
	ckrt(1)=-dc(3)/dc(2)
	ckrt(3)=-dc(2)/dc(1)
 
	endif
c
c     Distinguish between fast and Bernstein modes
      izone=0
      iwrong=0
c     Mode-conversion zone or asymptotic zone selection
      if (cabs1(ckrt(1)-conjg(ckrt(3)))
     %        .lt..1*(cabs1(ckrt(1))+cabs1(ckrt(3)))
     %               .and.aimag(ckrt(1))*aimag(ckrt(3)).lt.0.) izone=1
c     Root selection by asymptotic electrostatic property
      if (izone.eq.0.and.
     % cabs1(ckrt(3)).lt.(cabs1(ckrt(1))+2.*alfven+2.*akz**2).and.
     % cabs1(tperp0+ckrt(1)*tperp1).lt.cabs1(tperp0+ckrt(3)*tperp1))
     % iwrong=1
c     Root selection by damped root
      if (izone.eq.1
     %            .and.cabs1(ckrt(3)).lt.(cabs1(ckrt(1))+alfven)
     %                            .and.aimag(ckrt(3)).gt.0.) iwrong=1
c
c     Switch roots if needed
      if (iwrong.eq.1) then
      dc(4)=ckrt(1)
      ckrt(1)=ckrt(3)
      ckrt(3)=dc(4)
      endif
c
c GWH:  generate a warning message if the root selection algorithm
c may not have worked:
c
	data zmagrat2 /1.e-2/
	if(aimag(ckrt(1)) .lt. -zmagrat2*cabs1(ckrt(1))) then
           nwarns=nwarns+1
           if(nwarns .le. mxwarns) then
              write(6,*) 'growing ROOT warning (usually ignorable):'
              write(6,*) ' ckrt(1), ckrt(3) =', ckrt(1),ckrt(3)
           endif
        if(nwarns .eq. mxwarns) then
           write(6,*) '# of fppmod/root.f ROOT warnings > ',mxwarns
           write(6,*) 'suppressing further warning messages...'
        endif
        endif
	data zmagrat /4.0/
	if(cabs1(ckrt(1)) .gt. zmagrat*cabs1(ckrt(3))) then
           nwarns=nwarns+1
           if(nwarns .le. mxwarns) then
              write(6,*) 'larger ROOT warning (usually ignorable):'
              write(6,*) ' ckrt(1), ckrt(3) =', ckrt(1),ckrt(3)
           endif
        if(nwarns .eq. mxwarns) then
           write(6,*) '# of fppmod/root.f ROOT warnings > ',mxwarns
           write(6,*) 'suppressing further warning messages...'
        endif
	endif

c
c     Take square roots
c      ckrt(1)=csqrt(ckrt(1))
c      ckrt(2)=-ckrt(1)
c      ckrt(3)=csqrt(ckrt(3))
c      ckrt(4)=-ckrt(3)
c
c     Switch signs if necessary
c      if ((real(ckrt(1)-ckrt(2))+aimag(ckrt(1)-ckrt(2))).lt.0.) then
c      ckrt(1)=-ckrt(1)
c      ckrt(2)=-ckrt(2)
c      endif
c      if ((real(ckrt(3)-ckrt(4))+aimag(ckrt(4)-ckrt(3))).lt.0.) then
c      ckrt(3)=-ckrt(3)
c      ckrt(4)=-ckrt(4)
c      endif
 
      return
      end
