      subroutine spruvv(VVRmoms,VVZmoms,mxVVmoms,
     %	      vvRmax,vvRmin,vvZmax)
c
c initialize the Vacuum Vessel used by SPRUCE
c
c VV shape described by fourier moments:
c
c R(th) = Sum VVRmoms(i) cos(i th)
c Z(th) = Sum VVZmoms(i) sin(i th)
c
c where VVRmoms and VVzmoms are in meters.
c
C also set rminor to be related to the maximum minor radius of the VV
C relative to Rmajor, in order to make sure that the transition of the
C grid shape from being like the VV near the edge to being circular
C near center happens slow enough.
C
      real VVRmoms(0:mxVVmoms)
      real VVZmoms(0:mxVVmoms)
      real vvRmax,vvRmin,vvZmax
 
      include 'SPRINC'
      real radius(nvvgrid),drdth(nvvgrid),theta(nvvgrid),
     %	   rscat(nvvgrid),thscat(nvvgrid),uwork(nvvgrid)
      real vvperim(nvvgrid)
      real draytol
      real findR,findY
      common /vessel/ rmajor,rminor,radius,drdth,theta,findR,findY,
     %	              draytol,vvperim
 
      pi=abs(acos(-1.0))
 
c Get a fine mesh of the VV, but unevenly spaced in theta as
c measured from rmajor:
 
      dth=2.*pi/(nvvgrid-1)
      rminor=0.0
      do 5 i=1,nvvgrid-1
      th=(i-1)*dth
      rmjppl=0.0
      zedppl=0.0
 
      do 3 imom=0,mxVVmoms
      rmjppl=rmjppl+VVRmoms(imom)*cos(imom*th)
      zedppl=zedppl+VVZmoms(imom)*sin(imom*th)
3     continue
 
      rscat(i)=sqrt( (rmjppl-rmajor)**2 + zedppl**2)
      thscat(i)=atan2(zedppl,rmjppl-rmajor)
      if(thscat(i) .lt. 0.) thscat(i)=thscat(i)+2*pi
      rminor=max(rminor,rscat(i))
 
5     continue
 
c rminor > (2./3.)*max(rscat) is the minimum requirement to insure
c that Psi doesn't reach 1 until it gets to the VV.
c set rminor a little bit larger than that to give some breathing
c room while still putting most of the grid points nearer the
c plasma center.
 
      rminor=rminor*5./6.
      rscat(nvvgrid)=rscat(1)
      thscat(nvvgrid)=2*pi
 
c interpolate this to a mesh which is evenly spaced in theta:
 
      do 10 i=1,nvvgrid-1
      theta(i)=(i-1)*dth
      radius(i)=fpintrp(theta(i),rscat,thscat,nvvgrid)
10    continue
      theta(nvvgrid)=2*pi
      radius(nvvgrid)=radius(1)
 
c find perimeter around the Vaccuum Vessel:
      vvperim(1)=0.0
      zR0=rmajor+radius(i)
      zZ0=0.0
      vvRmax=rmajor
      vvRmin=rmajor
      vvZmax=0.0
      do 15 i=2,nvvgrid
 
      vvRmax=max(vvRmax,zR0)
      vvRmin=min(vvRmin,zR0)
      vvZmax=max(vvZmax,zZ0)
 
      zR1=rmajor+radius(i)*cos(theta(i))
      zZ1=	 radius(i)*sin(theta(i))
      vvperim(i)=vvperim(i-1)+sqrt((zR1-zR0)**2+(zZ1-zZ0)**2)
      zR0=zR1
      zZ0=zZ1
15    continue
 
c
c     d(radius)/d(theta) for clamped cubic splines
c
      nlu=(nvvgrid-1)/2
      nlup1=nlu+1
      do 20 i=2,nlu
      dth0=theta(i)-theta(i-1)
      dth1=theta(i+1)-theta(i)
   20 drdth(i)=3.*(radius(i+1)-radius(i))*dth0/dth1+
     %         3.*(radius(i)-radius(i-1))*dth1/dth0
      uwork(1)=0.
      drdth(1)=0.
      do 21 i=2,nlu
      dth0=theta(i)-theta(i-1)
      dth1=theta(i+1)-theta(i)
      diag=2.*(dth0+dth1)-dth1*uwork(i-1)
      uwork(i)=dth0/diag
   21 drdth(i)=(drdth(i)-dth1*drdth(i-1))/diag
      drdth(nlup1)=0.
      do 22 i=nlu,2,-1
   22 drdth(i)=drdth(i)-uwork(i)*drdth(i+1)
      do 23 i=nlup1+1,nvvgrid
   23 drdth(i)=-drdth(nvvgrid-i+1)
c
      return
      end
