      subroutine zdile(power,nions,rbins,rmjbins,nbin,
     %                 aprsp,apion,apelc,apmc,resist,react,
     %                 hzplt,ap,akreal,akimag,akrmaj,akprav,
     %		       transm,dRkmax)
c
c     ZDILE is the main subroutine of the ICRF wave package.
c     It directs the kz iterations and the radial power binning.
c
c Modified by Greg Hammett to allow for unequally spaced radial bins.
c
c 25-jan-1991 GWH: fix bug in antenna current profile, curchi.
c The antenna was being set about 50% taller than it really was.
c
      include 'SPRINC'
      parameter (maxbin=31,maxkz=50)
      real rmaj(nchi,npsi),zed(nchi,npsi),hpsi(nchi,npsi),
     %     hchi(nchi,npsi),psi(npsi),chi(nchi),pquad(npsi),pcoef(nchi),
     %     hzplt(nchi,npsi),ap(nchi,npsi),
     %     aprsp(maxbin,ionmax+2),apion(ionmax),
     %     akreal(2*npsi-1),akimag(2*npsi-1),akrmaj(2*npsi-1),
     %     akzgrd(maxkz),transm,dRkmax
      complex pwrden,pwrspc(ionmax+1),pwrmc,pwre,pwrh,cnprp2,emtoep,
     %        apofkz(maxkz),aiofkz(maxkz),apt,phsz,pzcur,pzcurp,pzcurm,
     %        hphi(nchi,npsi),echi(nchi,npsi),epsi(nchi,npsi),
     %        wall(nchi),curchi(nchi)
      common /metric/ rmaj,zed,hpsi,hchi,psi,chi,pquad,pcoef
     %       /antena/ freq,dkz,nstep,rmajkz,awidth,antrmj,antzed,
     %                zphas1,zphas2,antspc,boxspc,pfrac1
     %       /spfldcom/ hphi,echi,epsi
     %       /spctrm/ apofkz,aiofkz,akzgrd
	real rbins(nbin)    ! outer boundary minor radius of each bin
	real rmjbins(nbin)  ! major radius of each bin
 
	real alength(nchi)  ! path length along the antenna
 
c The volume of each bin is then
c 2 pi**2 rmjbins(i) * (rbins(i)**2-rbins(i-1)**2)
 
c
c------------- antenna's kz spectrum --------------------------------
c     two boxes, in phase, separated by "boxspc", each containing
c     two strips of width "awidth", separation "antspc", and phasing
c     "zphas", in parallel, with flat profile in toroidal direction,
c     with a fraction "pfrac1" of the power from box #1.
c
c GWH: multiplied pzcur by i so that Hzplt=Real(Hphi) shows the
c large evanescent fields in front of the antenna:
c
      pzcur(xkz)=(0.,1.)*sqrt(pfrac1)*sin(.5*xkz*awidth)*
     %                   cos(.5*xkz*antspc+.5*zphas1)/(pi*xkz*awidth)
     % +cexp((1.,0.)*xkz*boxspc)*sqrt(1-pfrac1)*sin(.5*xkz*awidth)*
     %                   cos(.5*xkz*antspc+.5*zphas2)/(pi*xkz*awidth)
c--------------------------------------------------------------------
c
c     zloc/tphase = toroidal location / time of hzplt
      zloc=.5*(antspc+awidth)*rmajkz/antrmj
      tphase=0.
c
c     initialize constants
      pi=abs(acos(-1.0))
      w=2.*pi*freq
      ako2=1.11265e-17*w**2
      snorm=1.25664e-6*w
      delchi=2.*pi/float(nchi)
      nspec=nions+1
      nmc=nspec+1
      apt=0.
      apelc=0.
      apmc=0.
      akprav=0.
      area=0.
      do 16 i=1,nions
   16 apion(i)=0.
      do 17 isp=1,nmc
      do 17 j=1,nbin
   17 aprsp(j,isp)=0.
      do 18 j=1,npsi
      do 18 i=1,nchi
      hzplt(i,j)=0.
   18 ap(i,j)=0.
 
c------------------- Antenna's Poloidal Current -----------------------
c
 
c determine if this is antenna is on the low-field or high-field side:
	thcur0=0.0
	if( abs(antrmj-rmaj(1,npsi)) .gt.
	1   abs(antrmj-rmaj(nchi/2,npsi))) thcur0=pi
 
C find poloidal angle subtended by antenna, by starting at the midplane
C and working our way up (which may be up or down in the angle chi,
C depending on whether the antenna is on the low-field or high-field
C side.
 
	if(thcur0 .eq. 0.0) then
	istart=1
	istop=nchi/2+1
	istep=1
	endif
 
	if(thcur0 .eq. pi) then
	istart=nchi/2+1
	istop=1
	istep=-1
	endif
 
	imax=istart
	zedmax=0.0
 
	do 19 i=istart+istep,istop,istep
 
	if (zed(i,npsi).gt.antzed) then
	    itip=i
	    go to 30
	endif
c antenna not allowed to go more than half-way around poloidally:
	if(zed(i,npsi) .gt. zedmax) then
	    zedmax=zed(i,npsi)
	    imax=i
	endif
19 	continue
	itip=imax
30	continue
 
	alength(istart)=0.0
	do 332 i=istart+istep,istop,istep
	alength(i)=alength(i-istep)
	1   +sqrt( (rmaj(i,npsi)-rmaj(i-istep,npsi))**2
	1	+  ( zed(i,npsi)- zed(i-istep,npsi))**2 )
	curchi(i)=0.0
332	continue
 
c free space wavelength:
	antk=2*pi*freq/2.9979e8
c increase slightly to account for faraday shield, etc.,
c which slows down the wave a bit:
	antk=antk*1.3
c assume the antenna was designed to fit no more than
c half-wave length:
	if(antk*alength(itip) .gt. pi/2) antk=antk/1.3
 
c Assumed current distribution is J=cos(k*length) to the tip of
c the antenna, and J=0 elsewhere.  k is assumed to be 30%
c larger than the free space wavenumber.
c
	do 340 i=istart,itip-istep,istep
	curchi(i)=cos(antk*alength(i))
340	continue
 
c impose up-down symmetry:
	do 350 i=nchi/2+2,nchi
350	curchi(i)=curchi(nchi-i+2)
 
c----------------------------------------------------------------------
c
c+++++++++++++++++++++ start kz iterations +++++++++++++++++++++++++++
      if (nstep.gt.maxkz) nstep=maxkz
      nkz=nstep
      do 20 ikz=1,nstep
      if (nstep.eq.1) then
      akz=dkz
      else
      akz=(float(ikz)-.5)*dkz
      endif
      akzgrd(ikz)=akz
      entor=akz*rmajkz
      antkz=akz*rmajkz/antrmj
c
c     toroidal spectrum factors
      pzcurp=pzcur(antkz)
      aiofkz(ikz)=pzcurp
      pzcurm=pzcur(-antkz)
      pzsqr=cabs(pzcurp)**2+cabs(pzcurm)**2
      phsz=pzcurp*cexp(cmplx(0.,akz*zloc-tphase))+
     %     pzcurm*cexp(cmplx(0.,-akz*zloc-tphase))
c
c     project antenna current to "wall" = hchi*Echi
      dwall=rmaj(1,npsi)-antrmj
      do 33 i=1,nchi
   33 wall(i)=cmplx(0.,snorm*dwall*hchi(i,npsi))*curchi(i)
c
c     calculate coefts of wave equation
      call coefts(w,entor)
c
c     find solution of wave equation
      call mrsolv(wall)
c
c     power deposition in poloidal plane
      apofkz(ikz)=(0.,0.)
      do 130 j=1,npsi
      do 130 i=1,nchi
      akzt=akz*rmajkz/rmaj(i,j)
      dkzt=dkz*rmajkz/rmaj(i,j)
      darea=hchi(i,j)*hpsi(i,j)*pquad(j)*delchi
 
c
c calculate the range of R covered by this grid point to make sure
c that we pick up the Z-function resonance in sptensor.
c (The same algorithm must be used in subroutines ZDILE and COEFTS):
c
      zdR=0.0
      if(i .ne. 1 .and. i .ne. nchi)
     %      zdR=abs(Rmaj(i+1,j)-Rmaj(i-1,j))
      if(i .eq. nchi) zdR=abs(Rmaj(1,j)-Rmaj(i-1,j))
      if(j .eq. 1)then
           zdR=max(zdR,abs(Rmaj(i,j+1)-Rmaj(i,j)))
      else if(j .eq. npsi)then
           zdR=max(zdR,abs(Rmaj(i,j)-Rmaj(i,j-1)))
      else
           zdR=max(zdR,abs(Rmaj(i,j+1)-Rmaj(i,j-1)))
      endif
      zdRovR=zdR/4./Rmaj(i,j)
 
 
c
      call powers(w,akzt,i,j,zdRovR,
     %                  pwrden,pwrspc,pwrmc,pwre,pwrh,cnprp2,emtoep)
c
      ap(i,j)=ap(i,j)+real(pwrden)*pzsqr*dkzt
      hzplt(i,j)=hzplt(i,j)+real(hphi(i,j)*phsz)*dkzt
      apmc=apmc+real(pwrmc)*darea*pzsqr*dkzt
      apelc=apelc+real(pwrspc(nspec))*darea*pzsqr*dkzt
      do 137 ion=1,nions
  137 apion(ion)=apion(ion)+real(pwrspc(ion))*darea*pzsqr*dkzt
      akprav=akprav+cabs(cnprp2)*real(pwrden)*darea*pzsqr*dkzt
      area=area+darea
c
      if (ikz.eq.1) then
c     save dispersion relation
      if (i.eq.1) then
      jmem=npsi+j-1
      akreal(jmem)=real(cnprp2)
      akimag(jmem)=aimag(cnprp2)
      akrmaj(jmem)=rmaj(i,j)
      endif
      if (i.eq.((nchi/2)+1)) then
      jmem=npsi-j+1
      akreal(jmem)=real(cnprp2)
      akimag(jmem)=aimag(cnprp2)
      akrmaj(jmem)=rmaj(i,j)
      endif
      endif
c
c     distribute power in radial bins. rbins(i) gives the outer r
c boundary of the i'th bin.
      call ctrans(i,j,r)
	ibin=Lkupr(r,rbins,nbin)+1
	if(ibin .gt. nbin) ibin=nbin
      aprsp(ibin,nmc)=aprsp(ibin,nmc)+real(pwrmc)*darea*pzsqr*dkzt
      do 142 ion=1,nspec
  142 aprsp(ibin,ion)=aprsp(ibin,ion)+real(pwrspc(ion))*darea*pzsqr*dkzt
c
  130 apofkz(ikz)=apofkz(ikz)+pwrden*cmplx(darea*pzsqr*dkzt/dkz,0.)
   20 apt=apt+apofkz(ikz)*cmplx(dkz,0.)
c++++++++++++++++++++++++++++ end kz iterations ++++++++++++++++++++
c
      resist=real(apt)
      react=aimag(apt)
      akprav=sqrt(abs(ako2*akprav/resist))
c
c     renormalize radial loading from [ohms] to [watts/cm**3]
      do 145 i=1,nbin
      if (i.eq.1) then
      rbin1=0.
      else
      rbin1=rbins(i-1)
      endif
	rbin2=rbins(i)
      binvol=2.*pi**2*(rbin2**2-rbin1**2)*rmjbins(i)
      pnorm=(power/resist)/binvol
      do 145 j=1,nmc
  145 aprsp(i,j)=aprsp(i,j)*pnorm
c
c      call output(-8)
 
c use the midplane dispersion relation to calculate the transmission
c coefficient, ignoring all evanescent regions (real(nperp**2) < 0).
c also find the max k*dR, which will be checked later to make sure it
c is not too big.
 
      transm=0.0
      dRkmax=0.0
      do 280 i=2,2*npsi-2
      zdr= 0.5*(akRmaj(i+1)-akRmaj(i-1))
      dRkmax=max(dRkmax,zdr**2*abs(cmplx(akreal(i),akimag(i))))
      if(akreal(i) .ge. 0.0) then
      transm=transm+aimag(sqrt(cmplx(akreal(i),akimag(i))))*zdr
      endif
280   continue
      transm=exp(-2*transm*sqrt(ako2))
 
      dRkmax=sqrt(ako2*dRkmax)
 
      return
      end
